Attribute VB_Name = "ModTZX"

' ******************************************
' *                                        *
' *               TZX format               *
' *         format Revision: v1 0.2        *
' *             (19 Dec 2006)              *
' *                                        *
' ******************************************
' * Tomaz Kac until revision 1.13          *
' * Martijn v.d. Heide and Ramsoft. 1 0.2  *
' * ZX81/TS1000 Specs from Michael D Wynne *
' *  for the Eighty-One project            *
' ******************************************
' http://www.worldofspectrum.org/tzxformat.html
'
'      Vb module By XavSnap 07/2007.
'         For VB81 XuR Project.
'
'
' Compatibility regards old EO format:
' TZX hardware may be corrupt on
' all 43 release.
' The 51 release seem fully compatible with 1.20 Tzx
' hardware specs.


Public hTZXfile As Integer
Public bPicture As Boolean
Public bLoadBloc As Boolean
Public bSaveP As Boolean
Public bEnumTXT As Boolean
Public bEnumHard As Boolean
Public TZX_Corrupt As Boolean
Public TZXname As String ' Public TZX name (Opened Tzx file name)
Public Msg_Tempo As Byte
Public Msg_Text As String

Public ZX81name As String
Public TZXBuffer() As Byte
Public Blocks_Buffer() As Byte
Public Block_START() As Long
Public Block_End() As Long
Public Block_ID As Integer
Public Hard_Info() As eHard
Public Nb_Files As Integer
Public Tzx_Spectrum As Boolean

'Notes:
' Byte    [BYTE]= 1Byte.  ( 00 )
' Boolean       = 2Bytes. ( 00;00 ou FF;FF )
' Integer [WORD]= 2Bytes. ( 00;00 )
' Single        = 4Bytes.
' Long   [DWORD]= 4Bytes. ( 1 => 01;00;00;00 )
' Double        = 8Bytes.
' Currency      = 8Bytes.
'

Type Blocinfo
BlocID As Byte
BlocType As String
Description As String
Tag As String
Offset As Long
Len As Long
End Type

 
Type TZX_header
Signature As String * 7 ' Header "ZXTape!"
EoTF As Byte ' End of text file marker = &h1A
Maj_Version As Byte 'TZX major revision number
Min_Version As Byte 'TZX minor revision number
End Type

Type eHard
TypeID As Byte
NameID As Byte
Description As String
StatID As String
End Type

Type eStand_DB
Pause As Integer
Len_of_Block As Integer
End Type

' ID 10 - Standard speed data block
' ---------------------------------
'0x00 - WORD Pause after this block (ms.) [1000]
'0x02 N WORD Length of following data
'0x04 - BYTE[N] Data as in .TAP files



' ID 11 - Turbo loading data block
' ---------------------------------
'0x00 - WORD Length of PILOT pulse [2168]
'0x02 - WORD Length of SYNC first pulse [667]
'0x04 - WORD Length of SYNC second pulse [735]
'0x06 - WORD Length of ZERO bit pulse [855]
'0x08 - WORD Length of ONE bit pulse [1710]
'0x0A - WORD Length of PILOT tone (number of pulses) [8063 header (flag<128), 3223 data (flag>=128)]
'0x0C - BYTE Used bits in last byte (other bits should be 0) [8]
'(e.g. if this is 6, then the bits used (x) in last byte are: xxxxxx00)
'0x0D - BYTE Pause after this block (ms.) [1000]
'0x0F N BYTE[3] Length of following data
'0x12 - BYTE[N] Data as in .TAP files


' ID 12 - Pure tone
'0x00 - WORD Length of pulse in T-states
'0x02 - WORD Number of pulses

' ID 13 - Sequence of pulses of different lengths
'0x00 N BYTE Number of pulses
'0x01 - WORD[N] Pulses' lengths

' ID 14 - Pure data block
'0x00 - WORD Length of ZERO bit pulse
'0x02 - WORD Length of ONE bit pulse
'0x04 - BYTE Used bits in last byte (other bits should be 0)
'(e.g. if this is 6, then the bits used (x) in last byte are: xxxxxx00)
'0x05 - BYTE Pause after this block (ms.)
'0x07 N BYTE[3] Length of following data
'0x0A - BYTE[N] Data as in .TAP files

' ID 15 - Direct recording block
'0x00 - WORD Number of T-states per sample (bit of data)
'0x02 - WORD Pause after this block in milliseconds (ms.)
'0x04 - BYTE Used bits (samples) in last byte of data (1-8)
'(e.g. if this is 2, only first two samples of the last byte will be played)
'0x05 N BYTE[3] Length of samples' data
'0x08 - BYTE[N] Samples data. Each bit represents a state on the EAR port (i.e. one sample).
'MSb is played first.

' ID 16 - C64 ROM type data block
'0x00 - DWORD Length of the WHOLE block including the data (extension rule)
'0x04 - WORD PILOT TONE pulse length [616]
'0x06 - WORD Number of waves in PILOT TONE
'0x08 - WORD SYNC first wave pulse length [1176]
'0x0A - WORD SYNC 2nd wave pulse length [896]
'0x0C - WORD ZERO bit 1st wave pulse length [616]
'0x0E - WORD ZERO bit 2nd wave pulse length [896]
'0x10 - WORD ONE bit 1st wave pulse length [896]
'0x12 - WORD ONE bit 2nd wave pulse length [616]
'0x14 - BYTE XOR Checksum bit for each Data byte: [1]
'    00 - Start XOR checksum with value 0
'    01 - Start XOR checksum with value 1
'    FF - No checksum bit
'0x15 - WORD FINISH BYTE 1st wave pulse length [1176]
'0x17 - WORD FINISH BYTE 2nd wave pulse length [896]
'0x19 - WORD FINISH DATA 1st wave pulse length [1176]
'0x1B - WORD FINISH DATA 2nd wave pulse length [616]
'0x1D - WORD TRAILING TONE pulse length [616]
'0x1F - WORD Number of waves in TRAILING TONE
'0x21 - BYTE Used bits in last byte (other bits should be 0)
'(e.g. if this is 6, then the bits used (x) in last byte are: xxxxxx00)
'0x22 - BYTE General Purpose, bit-mapped: [1]
'    bit 0 - Data Endianess: 0=LSb first, 1=MSb first
'0x23 - WORD Pause after this block in milliseconds (ms.)
'0x25 N BYTE[3] Length of following data
'0x28 - BYTE[N] Data as in .TAP files

' ID 17 - C64 turbo tape data block
'0x00 - DWORD Length of the WHOLE block including the data (extension rule)
'0x04 - WORD ZERO bit pulse
'0x06 - WORD ONE bit pulse
'0x08 - BYTE Additional bits in bytes (bit-mapped)
'    bits 0-1: number of bits (0-3)
'    bit 2: play additional bit(s) BEFORE (0) or AFTER (hTZXfile) the byte
'    bit 3: value of additional bit(s)
'0x09 - WORD Number of lead-in bytes
'0x0B - BYTE Lead-in byte
'0x0C - BYTE Used bits in last byte (other bits should be 0) [8]
'(e.g. if this is 6, then the bits used (x) in last byte are: xxxxxx00)
'0x0D - BYTE General purpose (bit-mapped) [0]
'    bit 0: little (0) or big (hTZXfile) endian format
'0x0E - WORD Number of trailing bytes
'0x10 - BYTE Trailing byte
'0x11 - WORD Pause after this block in milliseconds (ms.)
'0x13 N BYTE[3] Length of following data
'0x16 - BYTE[N] Data as in .TAP files

Type eC64TurboTape
WholeLen As Integer
End Type


'ID 18 - CSW Recording

'length: [00,01,02,03] 4
'Offset Value Type Description
'0x00 10+N DWORD Block length (without these four bytes)
'0x04 - WORD Pause after this block (in ms).
'0x06 - BYTE[3] Sampling rate
'0x09 - BYTE Compression type
'0 x01: RLE
'0 x02: Z -RLE
'0x0A - DWORD Number of stored pulses (after decompression, for validation purposes)
'0x0E - BYTE[N] CSW data, encoded according to the CSW file format specification


' ID 19 - General

'0x00 - DWORD Block length (without these four bytes)
'0x04 - WORD Pause after this block (ms)
'0x06 TOTP DWORD Total number of symbols in pilot/sync block (can be 0)
'0x0A NPP BYTE Maximum number of pulses per pilot/sync symbol
 
'0x0B ASP BYTE Number of pilot/sync symbols in the alphabet table (0=256)
 
'0x0C TOTD DWORD Total number of symbols in data stream (can be 0)
'0x10 NPD BYTE Maximum number of pulses per data symbol
 
'0x11 ASD BYTE Number of data symbols in the alphabet table (0=256)
 
 Type eGeneral1
        Bloc_Len As Long ' Total block length
        Pause As Integer ' Pause after this block (ms.)
        TOTP As Long ' Total number of records in pilot/sync block
        NPP As Byte ' Max pulses per symbol
        ASP As Byte ' Number of symbols in pilot/sync alphabet
        TOTD As Long ' Total number of records in data stream
        NPD As Byte ' Max pulses per data symbol
        ASD As Byte ' Number of symbols in data alphabet
 End Type

'0x12
'            - SYMDEF[ASP] Pilot and sync symbols definition table
'            This field is present only if TOTP>0
'
'0x12+(2*NPP+1)*ASP
'            - PRLE[TOTP] Pilot and sync data stream
'           This field is present only if TOTP>0
'
'0x12+(TOTP>0)*((2*NPP+1)*ASP)+TOTP*3
'            - SYMDEF[ASD] Data symbols definition table
'           This field is present only if TOTD>0
'
'0x12+(TOTP>0)*((2*NPP+1)*ASP)+TOTP*3+(2*NPD+1)*ASD
'            - BYTE[DS] Data stream
'           This field is present only if TOTD>0

' SYMDEF structure format
'0x00 - BYTE Symbol flags
'b0-b1: starting symbol polarity
'     00: opposite to the current level (make an edge, as usual) - default
'     01: same as the current level (no edge - prolongs the previous pulse)
'     10: force low level
'     11: force high level
 
'0x01 - WORD[MAXP] Array of pulse lengths.

'PRLE structure format
'0x00 - BYTE Symbol to be represented
'0x01 - WORD Number of repetitions




' ID 20 - Pause (silence) or 'Stop the tape' command
'0x00 - WORD Pause duration (ms.)
' Direct as integer

' ID 21 - Group start
'0x00 L BYTE Length of the group name string
'0x01 - CHAR[L] Group name in ASCII format (please keep it under 30 characters long)

Type eGroupeStart
GtextLen As Byte
text As String * 30
End Type

' ID 22 - Group end


' ID 23 - Jump to block
'0x00 - WORD Relative jump value
Type eJumpToBloc
Jump As Integer
End Type

' ID 24 - Loop start
'0x00 - WORD Number of repetitions (greater than 1)
Type eLoopStart
Repeat As Integer
End Type

' ID 25 - Loop end

' ID 26 - Call sequence
'0x00 N WORD Number of calls to be made
'0x02 - WORD[N] Array of call block numbers (relative-signed offsets)

Type eCallSequence
Calls As Integer
Array As Integer
End Type


' ID 27 - Return from sequence


' ID 28 - Select block

'0x00 - WORD Length of the whole block (without these two bytes)
'0x02 N BYTE Number of selections
'0x03 - SELECT[N] List of selections

Type eSelectBloc
Len As Integer
Nb As Byte
End Type

' Id 29 - SELECT structure format

'0x00 - WORD Relative Offset
'0x02 L BYTE Length of description text
'0x03 - CHAR[L] Description text (max. 30 chars)

Type eSelectFormat
Offset As Integer
Len As Byte
text As String * 30
End Type

' ID 2A - Stop tape if in 48K mode
'0x00 - DWORD Length of the whole block (0)

Type TZX_StopTape
LenWhole As Long
End Type


' ID 2B - Set signal level
' length: 05
' Offset Value Type Description
'0x00 1 DWORD Block length (without these four bytes)
'0x04 - BYTE Signal level (0=low, 1=high)
Type TZX_SetSignalLevel
BlockLen As Long
Signal_Level As Byte
End Type

' ID 30 - Text description
'0x00 N BYTE Length of the text description
'0x01 - CHAR[N] Text description in ASCII format

Type TZX_TextDescription
LoT As Byte
End Type

' ID 31 - Message block
'0x00 - BYTE Time (in seconds) for which the message should be displayed
'0x01 N BYTE Length of the text message
'0x02 - CHAR[N] Message that should be displayed in ASCII format

' Read infos
Type TZX_MessageBlocR
Time As Byte
LoT As Byte
Message As String * 255 '(Lenght=LoT)
End Type

' Just retrieve tempo. and the trimed message.
Type TZX_MessageBloc
tempo As Byte
Message As String
End Type

' ID 32 - Archive info
'0x00 - WORD Length of the whole block (without these two bytes)
'0x02 N BYTE Number of text strings
'0x03 - TEXT[N] List of text strings

Type TZX_ArchiveInfoHeader
length As Integer
NBMessage As Byte
End Type
'0x00 - BYTE ID_ARCHIVE
'0x01 L BYTE Length of text string
'0x02 - CHAR[L] Text string in ASCII format

Type TZX_ArchiveInfoR
IDArchive As Byte
length As Byte
Message As String * 255
End Type

Type TZX_ArchiveInfo
IDArchive As Byte
Message As String
End Type

Type Spec_F_type
Tag1 As Byte
Tag2 As Byte
End Type


' ID 33 - Hardware type
'''0x00 N BYTE Number of machines and hardware types for which info is supplied
'''0x01 - HWINFO[N] List of machines and hardware

'0x00 - BYTE Hardware type (Computer)
'0x01 - BYTE Hardware ID   (Zx81)
'0x02 - BYTE Hardware information: (Run on/use device/Run but dont use this hardware/Dont Run)

Type TZX_HardWareInfo
Hardware_Type As Byte 'Computers, cards...
Hardware_ID As Byte ' Name of...
Hardware_Info As Byte
End Type

'Hardware Information:
'    00 - The tape RUNS on this machine or with this hardware,
'          but may or may not use the hardware or special features of the machine.
'    01 - The tape USES the hardware or special features of the machine,
'          such as extra memory or a sound chip.
'    02 - The tape RUNS but it DOESN'T use the hardware
'          or special features of the machine.
'    03 - The tape DOESN'T RUN on this machine or with this hardware.

Const eInf_H00 = "RO" 'Run ON.
Const eInf_H01 = "UF" 'Use feature of
Const eInf_H02 = "NS" 'Not Spec.
Const eInf_H03 = "DR" 'Don't run.


' ID 34 - Emulation info
'0x00 - WORD General emulation flags:
'    bit 0 : R-register emulation [1]
'    bit 1 : LDIR emulation [1]
'    bit 2 : high resolution colour emulation with true interrupt freq. [1]
'    bit 3,4 : video synchronisation : 1=high, 3=low, 0,2=normal [0]
'    bit 5 : fast loading when ROM load routine is used [1]
'    bit 6 : border emulation [1]
'    bit 7 : screen refresh mode (1: ON, 0: OFF) [1]
'    bit 8 : start playing the tape immediately [0]
'           If this is 0 then the emulator should only load the info blocks
'           and WAIT when it encounters first DATA block
'    bit 9 : auto type LOAD"" or press ENTER when in 128k mode [0]
'0x02 - BYTE Screen refresh delay : 1 - 255 (interrupts between refreshes) [1]
'(used when screen refresh mode is ON)
'0x03 - WORD Interrupt Frequency : 0 - 999 Hz
'0x05 - BYTE[3] Reserved for future expansion
'

' ID 35 - Custom info block ******************************************


'0x00 - CHAR[10] Identification string (in ASCII)
'    0x10 L DWORD Length of the custom info
'    0x14 - BYTE[L] Custom info
'
' ------------------- Pokes -------------------
'
'               0x00 - CHAR[10] "POKEs" + 11 spaces (custom block ID)
'               0x10 - DWORD Length of following data
'               0x14 L BYTE General description length
'               0x15 - CHAR[L] General description in ASCII format
'               0x16 N BYTE Number of trainers
'               0x17 - TRAINER[N] Trainer definitions
'
'                ------------------- Trainer -------------------
'
'                 0x00 L BYTE Trainer description length
'                 0x01 - CHAR[L] Trainer description in ASCII format
'                 0x01+L M BYTE General description length
'                 0x02+L - CHAR[M] General description in ASCII format
'                 0x02+L+M N BYTE Number of pokes in this trainer
'                 0x03+L+M - POKE[N] POKEs definitions
'
'                ------------------- Poke Structure -------------------
'
'               0x00 - BYTE POKE type:
'                   bit 0-2 : memory page number
'                   bit 3 : ignore memory page number?
'                   bit 4 : user inserts the POKE value?
'                   bit 5 : unknown original value?
'               0x01 - WORD POKE address
'               0x03 - BYTE POKE value (leave 0 if 'user inserts' bit set)
'               0x04 - BYTE POKE original value (leave 0 if 'unknown' bit set)
'
' ------------------- Instructions block -------------------

'               0x00 - CHAR[10] "Instructions" + 5 spaces (custom block ID)
'               0x10 L DWORD Length of the following data
'               0x14 - BYTE[L] Instructions text in ASCII format
'
'
' ------------------- Spectrum screen block -------------------
'               0x00 - CHAR[10] "Spectrum Screen" + 1 spaces (custom block ID)
'               0x10 - DWORD Length of the following data
'               0x14 L BYTE Description length (if this is 0 then handle it as 'Loading Screen')
'               0x15 - CHAR[L] Description of the picture in ASCII format
'               0x15+L - BYTE BORDER Colour in Spectrum colour format (0=black, 1=blue, ...)
'               0x16+L - BYTE[6912] Screen in standard Spectrum video format

'
' ------------------- ZX-Edit document block -------------------
'               0x00 - CHAR[10] "ZX-Edit document" (custom block ID)
'               0x10 - DWORD Length of the following data
'               0x14 L BYTE Description length (if this is 0 then handle it as 'Instructions')
'               0x15 - CHAR[L] Description of the document in ASCII format
'               0x15+L - BYTE The ZX-Editor document (.ZED file)

'
' ------------------- Picture block -------------------
'               0x00 - CHAR[10] "Picture" + 9 spaces (custom block ID)
'               0x10 - DWORD Length of the following data
'               0x14 - BYTE Picture format:
'                   00 - GIF
'                   01 - JPEG
'               0x14 L BYTE Description length (if this is 0 then handle it as 'Inlay Card')
'               0x15 - CHAR[L] Description of the document in ASCII format
'               0x15+L - BYTE The picture itself

' END OF ID35 BLOC DEF. **********************************************

' ID 40 - Snapshot block

'0x00 - BYTE Snapshot type:
'                           0:      .Z80 Format
'                           1:      .SNA Format
 
'               0x01 L BYTE[3] Snapshot length
'               0x04 - BYTE[L] Snapshot itself

' ID 5A - "Glue" block (90 dec, ASCII Letter 'Z')
'(0x00 - BYTE[9] Value: { "XTape!",0x1A,MajR,MinR }
'Just skip these 9 bytes and you will end up on the next ID.

'Public Val_3Bytes(2) As Byte

Enum eID_Name
TZX_BLOCK_Unknown = 0 '"Unknown"
TZX_BLOCK_ZC = &HC ' ?
TZX_BLOCK_ROM = &H10    '"Standard speed data block"
TZX_BLOCK_TURBO = &H11  '"Turbo speed data block"
TZX_BLOCK_TONE = &H12   '"Pure tone"
TZX_BLOCK_PULSE = &H13  '"Sequence of pulses of different lengths"
TZX_BLOCK_DATA = &H14 '"Pure data block"
TZX_BLOCK_DREC = &H15 '"Direct recording block"
TZX_BLOCK_RDB = &H16 '"C64 ROM type data block"
TZX_BLOCK_TDB = &H17 '"C64 turbo tape data block"
TZX_BLOCK_CSW = &H18 '"CSW recording bloc"
TZX_BLOCK_GENERAL = &H19 '"Generalized data block"
TZX_BLOCK_PAUSE = &H20 '"Pause (silence) or 'Stop the tape' command"
TZX_BLOCK_GSTART = &H21 '"Group start"
TZX_BLOCK_GEND = &H22 '"Group end"
TZX_BLOCK_JUMP = &H23 '"Jump to block"
TZX_BLOCK_LSTART = &H24 '"Loop start"
TZX_BLOCK_LEND = &H25 '"Loop end"
TZX_BLOCK_CS = &H26 '"Call sequence"
TZX_BLOCK_RFS = &H27 '"Return from sequence"
TZX_BLOCK_SBLOCK = &H28 '"Select block"
TZX_BLOCK_STOP48K = &H2A '"Stop tape if in 48K mode"
TZX_BLOCK_SETLEVEL = &H2B '"Set signal level"
TZX_BLOCK_TEXT = &H30 '"Text description"
TZX_BLOCK_MESSAGE = &H31 '"Message block"
TZX_BLOCK_ARCHIVE = &H32 '"Archive info"
TZX_BLOCK_HWTYPE = &H33 '"Hardware type"
TZX_BLOCK_EINFO = &H34 '"Emulation info"
TZX_BLOCK_CUSTOM = &H35 '"Custom info block"
TZX_BLOCK_SNAP = &H40  '"Snapshot block"
TZX_BLOCK_GLUE = &H5A '"Glue block" ' (90 dec, ASCII Letter 'Z')
TZX_BLOCK_EOT = &HFE ' "End of tape"
End Enum
 
'COMP - Computers [46 items]
Const COMP_00 = "ZX Spectrum 16k"
Const COMP_01 = "ZX Spectrum 48k, Plus"
Const COMP_02 = "ZX Spectrum 48k ISSUE 1"
Const COMP_03 = "ZX Spectrum 128k +(Sinclair)"
Const COMP_04 = "ZX Spectrum 128k +2 (grey case)"
Const COMP_05 = "ZX Spectrum 128k +2A, +3"
Const COMP_06 = "Timex Sinclair TC-2048"
Const COMP_07 = "Timex Sinclair TS-2068"
Const COMP_08 = "Pentagon 128"
Const COMP_09 = "Sam Coupe"
Const COMP_0A = "Didaktik M"
Const COMP_0B = "Didaktik Gama"
Const COMP_0C = "ZX-81"
Const COMP_0D = "ZX-80"
Const COMP_0E = "ZX Spectrum 128k, Spanish version"
Const COMP_0F = "ZX Spectrum, Arabic version"
Const COMP_10 = "TK 90-X"
Const COMP_11 = "TK 95"
Const COMP_12 = "Byte"
Const COMP_13 = "Elwro"
Const COMP_14 = "ZS Scorpion"
Const COMP_15 = "Amstrad CPC 464"
Const COMP_16 = "Amstrad CPC 664"
Const COMP_17 = "Amstrad CPC 6128"
Const COMP_18 = "Amstrad CPC 464+"
Const COMP_19 = "Amstrad CPC 6128+"
Const COMP_1A = "Jupiter ACE"
Const COMP_1B = "Enterprise"
Const COMP_1C = "Commodore 64"
Const COMP_1D = "Commodore 128"
Const COMP_1E = "Inves Spectrum+"
Const COMP_1F = "Profi"
Const COMP_20 = "GrandRomMax"
Const COMP_21 = "Kay 1024"
Const COMP_22 = "Ice Felix HC 91"
Const COMP_23 = "Ice Felix HC 2000"
Const COMP_24 = "Amaterske RADIO Mistrum"
Const COMP_25 = "Quorum 128"
Const COMP_26 = "MicroART ATM"
Const COMP_27 = "MicroART ATM Turbo 2"
Const COMP_28 = "Chrome"
Const COMP_29 = "ZX Badaloc"
Const COMP_2A = "TS-1500"
Const COMP_2B = "Lambda"
Const COMP_2C = "TK-65"
Const COMP_2D = "ZX-97"
 
'00 - Computers [46 items]
Enum eComputers
CPM_00 = 0 '"ZX Spectrum 16k"
CPM_01 = 1 '"ZX Spectrum 48k, Plus"
CPM_02 = 2 '"ZX Spectrum 48k ISSUE 1"
CPM_03 = 3 '"ZX Spectrum 128k +(Sinclair)"
CPM_04 = 4 '"ZX Spectrum 128k +2 (grey case)"
CPM_05 = 5 '"ZX Spectrum 128k +2A, +3"
CPM_06 = 6 '"Timex Sinclair TC-2048"
CPM_07 = 7 '"Timex Sinclair TS-2068"
CPM_08 = 8 '"Pentagon 128"
CPM_09 = 9 '"Sam Coupe"
CPM_0A = &HA '"Didaktik M"
CPM_0B = &HB '"Didaktik Gama"
CPM_0C = &HC '"ZX-80"
CPM_0D = &HD '"ZX-81"
CPM_0E = &HE '"ZX Spectrum 128k, Spanish version"
CPM_0F = &HF '"ZX Spectrum, Arabic version"
CPM_10 = &H10 '"TK 90-X"
CPM_11 = &H11 '"TK 95"
CPM_12 = &H12 '"Byte"
CPM_13 = &H13 '"Elwro 800-3"
CPM_14 = &H14 '"ZS Scorpion 256"
CPM_15 = &H15 '"Amstrad CPC 464"
CPM_16 = &H16 '"Amstrad CPC 664"
CPM_17 = &H17 '"Amstrad CPC 6128"
CPM_18 = &H18 '"Amstrad CPC 464+"
CPM_19 = &H19 '"Amstrad CPC 6128+"
CPM_1A = &H1A '"Jupiter ACE"
CPM_1B = &H1B '"Enterprise"
CPM_1C = &H1C '"///Commodore 64///"
CPM_1D = &H1D '"///Commodore 128///"
CPM_1E = &H1E '"Inves Spectrum+"
CPM_1F = &H1F '"Profi"
CPM_20 = &H20 '"GrandRomMax"
CPM_21 = &H21 '"Kay 1024"
CPM_22 = &H22 '"Ice Felix HC 91"
CPM_23 = &H23 '"Ice Felix HC 2000"
CPM_24 = &H24 '"Amaterske RADIO Mistrum"
CPM_25 = &H25 '"Quorum 128"
CPM_26 = &H26 '"MicroART ATM"
CPM_27 = &H27 '"MicroART ATM Turbo 2"
CPM_28 = &H28 '"Chrome"
CPM_29 = &H29 '"ZX Badaloc"
CPM_2A = &H2A '"TS-1500"
CPM_2B = &H2B '"Lambda"
CPM_2C = &H2C '"TK-65"
CPM_2D = &H2D '"ZX-97"

End Enum


' Storage.[24 items]
Const ExStore_00 = "ZX Microdrive"
Const ExStore_01 = "Opus Discovery"
Const ExStore_02 = "MGT Disciple"
Const ExStore_03 = "MGT Plus-D"
Const ExStore_04 = "Rotronics Wafadrive"
Const ExStore_05 = "TR-DOS(BetaDisk)"
Const ExStore_06 = "Byte Drive"
Const ExStore_07 = "Watsford"
Const ExStore_08 = "FIZ"
Const ExStore_09 = "Radofin"
Const ExStore_0A = "Didaktik disk drives"
Const ExStore_0B = "BS - DOS(MB - 2)"
Const ExStore_0C = "ZX Spectrum +3 disk drive"
Const ExStore_0D = "JLO (Oliger) disk interface"
Const ExStore_0E = "Timex FDD3000"
Const ExStore_0F = "Zebra disk drive"
Const ExStore_10 = "Ramex Millenia"
Const ExStore_11 = "Larken"
Const ExStore_12 = "Kempston disk interface"
Const ExStore_13 = "Sandy"
Const ExStore_14 = "ZX Spectrum +3e hard disk"
Const ExStore_15 = "ZXATASP"
Const ExStore_16 = "DivIDE"
Const ExStore_17 = "ZXCF"

'01 - External storage [24 items]
Enum eExternal_Stor
ExSt_00 = 0 '"ZX Microdrive"
ExSt_01 = 1 '"Opus Discovery"
ExSt_02 = 2 '"MGT Disciple"
ExSt_03 = 3 '"MGT Plus-D"
ExSt_04 = 4 '"Rotronics Wafadrive"
ExSt_05 = 5 '"TR-DOS(BetaDisk)"
ExSt_06 = 6 '"Byte Drive"
ExSt_07 = 7 '"Watsford"
ExSt_08 = 8 '"FIZ"
ExSt_09 = 9 '"Radofin"
ExSt_0A = &HA '"Didaktik disk drives"
ExSt_0B = &HB '"BS - DOS(MB - 2)"
ExSt_0C = &HC '"ZX Spectrum +3 disk drive"
ExSt_0D = &HD '"JLO (Oliger) disk interface"
ExSt_0E = &HE '"Timex FDD3000"
ExSt_0F = &HF '"Zebra disk drive"
ExSt_10 = &H10 '"Ramex Millenia"
ExSt_11 = &H11 '"Larken"
ExSt_12 = &H12 '"Kempston disk interface"
ExSt_13 = &H13 '"Sandy"
ExSt_14 = &H14 '"ZX Spectrum +3e hard disk"
ExSt_15 = &H15 '"ZXATASP"
ExSt_16 = &H16 '"DivIDE"
ExSt_17 = &H17 '"ZXCF"
End Enum

' Add-Ons [11 items]
Const Add_00 = "Sam Ram"
Const Add_01 = "Multiface ONE"
Const Add_02 = "Multiface 128k"
Const Add_03 = "Multiface + 3"
Const Add_04 = "MultiPrint"
Const Add_05 = "MB-02 ROM/RAM expansion"
Const Add_06 = "SoftRom"
Const Add_07 = "1K RAM"
Const Add_08 = "16K RAM PACK"
Const Add_09 = "48K RAM PACK"
Const Add_0A = "Memory in 8-16k used"

'02 - ROM/RAM type add-ons [11 items]
Enum eRom
Rom_00 = 0 '"Sam Ram"
Rom_01 = 1 '"Multiface ONE"
Rom_02 = 2 '"Multiface 128k"
Rom_03 = 3 '"Multiface + 3"
Rom_04 = 4 '"MultiPrint"
Rom_05 = 5 '"MB-02 ROM/RAM expansion"
Rom_06 = 6 '"SoftRom"
Rom_07 = 7 '"1K RAM"
Rom_08 = 8 '"16K RAM PACK"
Rom_09 = 9 '"48K RAM PACK"
Rom_0A = 10 '"Memory in 8-16k used"
End Enum

' Sound Card.[14 items]
Const Sound_00 = "Classic AY hardware (compatible with 128k ZXs)"
Const Sound_01 = "Fuller Box AY sound hardware"
Const Sound_02 = "Currah microSpeech"
Const Sound_03 = "SpecDrum"
Const Sound_04 = "AY ACB stereo (A+C=left, B+C=right); Melodik"
Const Sound_05 = "AY ABC stereo (A+B=left, B+C=right)"
Const Sound_06 = "RAM Music Machine"
Const Sound_07 = "Covox"
Const Sound_08 = "General Sound"
Const Sound_09 = "Intec Electronics Digital Interface B8001"
Const Sound_0A = "Zon-X AY"
Const Sound_0B = "QuickSilva AY"
Const Sound_0C = "Jupiter Ace"
Const Sound_0D = "Beeper"


'03 - Sound devices [14 items]
Enum eSound
Snd_00 = 0 '"Classic AY hardware (compatible with 128k ZXs)"
Snd_01 = 1 '"Fuller Box AY sound hardware"
Snd_02 = 2 '"Currah microSpeech"
Snd_03 = 3 '"SpecDrum"
Snd_04 = 4 '"AY ACB stereo (A+C=left, B+C=right); Melodik"
Snd_05 = 5 '"AY ABC stereo (A+B=left, B+C=right)"
Snd_06 = 6 '"RAM Music Machine"
Snd_07 = 7 '"Covox"
Snd_08 = 8 '"General Sound"
Snd_09 = 9 '"Intec Electronics Digital Interface B8001"
Snd_0A = 10 '"Zon-X AY"
Snd_0B = 11 '"QuickSilva AY"
Snd_0C = 12 '"Jupiter Ace"
Snd_0D = 13 '"Beeper"
End Enum

' Joystick.[5 items]
Const Jk_00 = "Kempston"
Const Jk_01 = "Cursor, Protek, AGF"
Const Jk_02 = "Sinclair 2 Left (12345)"
Const Jk_03 = "Sinclair 1 Right (67890)"
Const Jk_04 = "Fuller"

'04 -Joysticks[5 items]
Enum eJoystick
Joy_00 = 0 '"Kempston"
Joy_01 = 1 '"Cursor, Protek, AGF"
Joy_02 = 2 '"Sinclair 2 Left (12345)"
Joy_03 = 3 '"Sinclair 1 Right (67890)"
Joy_04 = 4 '"Fuller"
End Enum

'Mice [2 items]
Const Mouse_00 = "AMX mouse"
Const Mouse_01 = "Kempston mouse"

'05 - Mice [2 items]
Enum eMice
Mice_00 = 0 '"AMX mouse"
Mice_01 = 1 '"Kempston mouse"
End Enum
 
'Other controlers [4 items]
Const OCtrl_00 = "Trickstick"
Const OCtrl_01 = "ZX Light Gun"
Const OCtrl_02 = "Zebra Graphics Tablet"
Const OCtrl_03 = "Defender Light Gun"

'06 - Other controllers [4 items]
Enum eOther_Controllers
OCnt_00 = 0 '"Trickstick"
OCnt_01 = 1 '"ZX Light Gun"
OCnt_02 = 2 '"Zebra Graphics Tablet"
OCnt_03 = 3 '"Defender Light Gun"
End Enum

'Serial [2 items]
Const SP_00 = "ZX Interface 1"
Const SP_01 = "ZX Spectrum 128k"

'07 - Serial ports [2 items]
Enum eSerial_Ports
RSp_00 = 0 '"ZX Interface 1"
RSp_01 = 1 '"ZX Spectrum 128k"
End Enum

'Paralele Port. [11 items]
Const PP_00 = "Kempston S"
Const PP_01 = "Kempston E"
Const PP_02 = "ZX Spectrum +3"
Const PP_03 = "Tasman"
Const PP_04 = "DK 'Tronics"
Const PP_05 = "Hilderbay"
Const PP_06 = "INES Printerface"
Const PP_07 = "ZX LPrint Interface 3"
Const PP_08 = "MultiPrint"
Const PP_09 = "Opus Discovery"
Const PP_0A = "Standard 8255 chip with ports 31,63,95"

'08 - Parallel ports [11 items]
Enum eParallel_Ports
CEp_00 = 0 '"Kempston S"
CEp_01 = 1 '"Kempston E"
CEp_02 = 2 '"ZX Spectrum +3"
CEp_03 = 3 '"Tasman"
CEp_04 = 4 '"DK 'Tronics"
CEp_05 = 5 '"Hilderbay"
CEp_06 = 6 '"INES Printerface"
CEp_07 = 7 '"ZX LPrint Interface 3"
CEp_08 = 8 '"MultiPrint"
CEp_09 = 9 '"Opus Discovery"
CEp_0A = 10 '"Standard 8255 chip with ports 31,63,95"
End Enum

'Printers. [3 items]
Const Printer_00 = "ZX Printer, Alphacom 32 & compatibles"
Const Printer_01 = "Generic printer"
Const Printer_02 = "EPSON compatible"

'09 - Printers [3 items]
Enum ePrinters
Prt_00 = 0 '"ZX Printer, Alphacom 32 & compatibles"
Prt_01 = 1 '"Generic printer"
Prt_02 = 2 '"EPSON compatible"
End Enum


'Modems [2 items]
Const Modem_00 = "Prism VTX 5000"
Const Modem_01 = "T/S 2050 or Westridge 2050"

'0A -Modems [2 items]
Enum eModems
Mod_00 = 0 '"Prism VTX 5000"
Mod_01 = 1 '"T/S 2050 or Westridge 2050"
End Enum

' Digit. [4 items]
Const Dg_00 = "RD Digital Tracer"
Const Dg_01 = "DK 'Tronics Light Pen"
Const Dg_02 = "British MicroGraph Pad"
Const Dg_03 = "Romantic Robot Videoface"

'0B -Digitizers [4 items]
Enum eDigitizers
Dig_00 = 0 '"RD Digital Tracer"
Dig_01 = 1 '"DK 'Tronics Light Pen"
Dig_02 = 2 '"British MicroGraph Pad"
Dig_03 = 3 '"Romantic Robot Videoface"
End Enum
 
'Net [1 item]
Const Network_00 = "ZX Interface 1"

'0C - Network adapters [1 item]
Enum eNetwork
Net_00 = 0 '"ZX Interface 1"
End Enum

'KeyBoard [1 item]
Const Key_00 = "Keypad for ZX Spectrum 128k"

'0D - Keyboards & keypads [1 item]
Enum eKeyboards
Kb_00 = 0 '"Keypad for ZX Spectrum 128k"
End Enum

'AD/DA [2 items]
Const ADDAc_00 = "Harley Systems ADC 8.2"
Const ADDAc_01 = "Blackboard Electronics"

'0E - AD/DA converters [2 items]
Enum eAD_DA
ADDA_00 = 0 '"Harley Systems ADC 8.2"
ADDA_01 = 1 '"Blackboard Electronics"
End Enum

'Eprom [1 item]
Const Eprom_00 = "Orme Electronics"

'0F - EPROM programmers [1 items]
Enum eEprom_Prog
EP_00 = 0 '"Orme Electronics"
End Enum

'10 - Graphic Card. [5 items]
Const GrCard_00 = "WRX Hi-Res"
Const GrCard_01 = "G007"
Const GrCard_02 = "Memotech"
Const GrCard_03 = "Lambda Color"
Const GrCard_04 = ""


'Text identification byte:

Enum ID_ARCHIVE
Title = 0      '- Full title
Publisher = 1  '- Software house/publisher
Author = 2     '- Author(s)
Year = 3       '- Year of publication
Language = 4   '- Language
Game = 5       '- Game/utility type
Price = 6      '- Price
Protection = 7 '- Protection scheme/loader
Origin = 8     '- Origin
Comment = 255  '- Comment(s)
End Enum

Public Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (lpvDest As Any, lpvSource As Any, ByVal cbCopy As Long)



Function OpenTZX() As Boolean
OpenTZX = True
If TZXname = "" Then OpenTZX = False: Exit Function
bPicture = False
bLoadBloc = False
bSaveP = False
ZX81name = ""
hTZXfile = FreeFile
Open TZXname For Binary Lock Read As #hTZXfile
Exit Function
Err_load:
OpenTZX = False
End Function

Sub Dispose()
 Close #hTZXfile
bPicture = False
bLoadBloc = False
bSaveP = False
End Sub
' *********************************************************************

Sub Enum_TZX(Bloc() As Blocinfo, Optional LoadPic As Boolean = False _
, Optional LoadBloc As Boolean = False _
, Optional SaveP As Boolean = False)
Nb_Files = 0
Dim IDBlock As Byte
Dim Header As TZX_header
Dim bRet As Boolean
Dim counter As Integer

 Block_ID = 0 'Block counter
 ReDim Block_START(0)
 ReDim Block_End(0)
 
If LoadPic Then bPicture = True
If LoadBloc Then bLoadBloc = True
If SaveP Then bSaveP = True

        ReDim Preserve Block_START(Block_ID)
        Block_START(Block_ID) = 1

        counter = 0
        Get #hTZXfile, 1, Header
        
        ReDim Preserve Block_End(Block_ID)
        Block_End(Block_ID) = Seek(hTZXfile) - 1
        Block_ID = Block_ID + 1
        
        TZX_Corrupt = False
        If UCase(Header.Signature) <> "ZXTAPE!" Then
        Nb_Files = 0
        TZX_Corrupt = True 'Close #hTZXfile:
        Exit Sub
        End If
ReScan:
         
        If EOF(hTZXfile) Then GoTo Close_tr
        ReDim Preserve Block_START(Block_ID)
        Block_START(Block_ID) = Seek(hTZXfile)
         
        Get #hTZXfile, , IDBlock
        ReDim Preserve Bloc(counter)
        
        Bloc(counter).BlocID = IDBlock
        
Select Case IDBlock
' &h10
                Case TZX_BLOCK_ROM
                Bloc(counter).BlocType = "Standard speed data block."
                Bloc(counter).Description = Load_STANDARD
                Nb_Files = 0
                Tzx_Spectrum = True
                
' &h11
                Case TZX_BLOCK_TURBO
                Bloc(counter).BlocType = "Turbo loading data block."
                Bloc(counter).Description = ""
' &h12
                Case TZX_BLOCK_TONE
                Bloc(counter).BlocType = "Pure tone."
                Bloc(counter).Description = ""
' &h13
                Case TZX_BLOCK_PULSE
                Bloc(counter).BlocType = "Sequence of pulses of different lengths."
                Bloc(counter).Description = ""
' &h14
                Case TZX_BLOCK_DATA
                Bloc(counter).BlocType = "Pure data block."
                Bloc(counter).Description = ""
' &h15
                Case TZX_BLOCK_DREC
                Bloc(counter).BlocType = "Direct recording block."
                Bloc(counter).Description = ""
' &h16
                Case TZX_BLOCK_RDB
                Bloc(counter).BlocType = "C64 ROM type data block."
                Bloc(counter).Description = ""
' &h17
                Case TZX_BLOCK_TDB
                Bloc(counter).BlocType = "C64 turbo tape data block."
                Bloc(counter).Description = ""
' &h18
                Case TZX_BLOCK_CSW
                Bloc(counter).BlocType = "CSW."
                Bloc(counter).Description = ""
' &h19
                Case TZX_BLOCK_GENERAL
                Bloc(counter).BlocType = "Data block."
                Bloc(counter).Description = Load_GENERAL
                Dim DatasGen() As String
                ret = GetArray(Bloc(counter).Description, DatasGen)
                Bloc(counter).Tag = DatasGen(0)
                Bloc(counter).Len = val(DatasGen(1))
                Bloc(counter).Offset = val(DatasGen(2))
'                Bloc(counter).Tag = Trim(Left(Bloc(counter).Description, InStr(1, Bloc(counter).Description, "~") - 1))
                Nb_Files = Nb_Files + 1
' &h20
                Case TZX_BLOCK_PAUSE
                Bloc(counter).BlocType = "Pause (silence) or 'Stop the tape' command."
                Bloc(counter).Description = Load_Pause
                
' &h21
                Case TZX_BLOCK_GSTART
                Bloc(counter).BlocType = "Group start."
                Bloc(counter).Description = Load_TEXT_GS
                Debug.Print
' &h22
                Case TZX_BLOCK_GEND
                Bloc(counter).BlocType = "Group end."
                ' Bloc(Counter).Description = Load_TEXT_GE
' &h23
                Case TZX_BLOCK_JUMP
                Bloc(counter).BlocType = "Jump to block."
                Bloc(counter).Description = ""
' &h24
                Case TZX_BLOCK_LSTART
                Bloc(counter).BlocType = "Loop start."
                Bloc(counter).Description = ""
' &h25
                Case TZX_BLOCK_LEND
                Bloc(counter).BlocType = "Loop end."
                Bloc(counter).Description = ""
' &h26
                Case TZX_BLOCK_CS
                Bloc(counter).BlocType = "Call sequence."
                Bloc(counter).Description = ""
' &h27
                Case TZX_BLOCK_RFS
                Bloc(counter).BlocType = "Return from sequence."
                Bloc(counter).Description = ""
' &h28
                Case TZX_BLOCK_SBLOCK
                Bloc(counter).BlocType = "Select block."
                Bloc(counter).Description = ""
' &h2A
                Case TZX_BLOCK_STOP48K
                Bloc(counter).BlocType = "Stop tape if in 48K mode."
                Bloc(counter).Description = ""
' &H2B
                Case TZX_BLOCK_SETLEVEL
                Bloc(counter).BlocType = "Set Level block."
                Bloc(counter).Description = ""
' &H30
                Case TZX_BLOCK_TEXT
                Bloc(counter).BlocType = "Text description."
                Bloc(counter).Description = Load_TEXT_Description
                
' &H31
                Case TZX_BLOCK_MESSAGE
                Bloc(counter).BlocType = "Message block."
                Dim Mess_infos As TZX_MessageBloc
                Mess_infos = Load_TEXT_Message
                Bloc(counter).Description = "Msg.: " & Mess_infos.Message

' &H32
                Case TZX_BLOCK_ARCHIVE
                Bloc(counter).BlocType = "Archive info."
                Dim ArchInfo() As TZX_ArchiveInfo
                bRet = Load_Archive_info(ArchInfo())
              
                For a = 0 To UBound(ArchInfo)
                Bloc(counter).Description = Bloc(counter).Description & Trim(ArchInfo(a).IDArchive) & ": " & ArchInfo(a).Message & "|"
                Next a
                Debug.Print Bloc(counter).Description
' &H33
                Case TZX_BLOCK_HWTYPE
                Bloc(counter).BlocType = "Hardware type."
                Dim info() As TZX_HardWareInfo
                bRet = Load_HWTYPE(info())
                ReDim Preserve Hard_Info(UBound(info))
                For a = 0 To UBound(info) '- 1
                Hard_Info(a).Description = Get_ExtHard(info(a).Hardware_Type, info(a).Hardware_ID)
                Hard_Info(a).StatID = info(a).Hardware_Info
                Hard_Info(a).TypeID = info(a).Hardware_Type
                Hard_Info(a).NameID = info(a).Hardware_ID
                Bloc(counter).Description = Bloc(counter).Description & Hard_Info(a).Description & info(a).Hardware_Info & "|"
                Next a
' &H34
                Case TZX_BLOCK_EINFO
                Bloc(counter).BlocType = "Emulation info."
                Bloc(counter).Description = ""
' &H35
                Case TZX_BLOCK_CUSTOM
                Bloc(counter).BlocType = "Custom info block."
                Bloc(counter).Description = Load_Custom
' &H40
                Case TZX_BLOCK_SNAP
                Bloc(counter).BlocType = "Snapshot block."
                Bloc(counter).Description = ""
' &H5A
                Case TZX_BLOCK_GLUE
                Bloc(counter).BlocType = "Glue block."
                Bloc(counter).Description = ""
                Seek #hTZXfile, Seek(hTZXfile) + 10
        
' &HF0
                Case &HF0
                Bloc(counter).BlocType = "F0"
                Bloc(counter).Description = "???"
' &HFE
                Case &HFE
                Bloc(counter).BlocType = "End of tape."
                Bloc(counter).Description = ""
                
' &H00
                Case 0
                Bloc(counter).BlocType = ""
                Bloc(counter).Description = ""
' &H??
                Case Else
                Bloc(counter).BlocType = "Unknown Block."
                Bloc(counter).Description = "!!! " & Loc(hTZXfile) & " !!!"
               
                LoadUnknownBlock
End Select
                ReDim Preserve Block_End(Block_ID)
                Block_End(Block_ID) = Seek(hTZXfile) - 1
                Block_ID = Block_ID + 1
                
                If EOF(hTZXfile) Then GoTo Close_tr
                counter = counter + 1
                GoTo ReScan
Close_tr:
                ReDim Preserve Block_End(Block_ID)
                Block_End(Block_ID) = Seek(hTZXfile) - 1

For z = 0 To UBound(Block_START) - 1
Debug.Print Block_START(z); ";"; Block_End(z)
Next

End Sub
' *********************************************************************

Sub LoadUnknownBlock()

End Sub
' *********************************************************************

Function Load_TEXT_Description() As String
Dim LoT As Byte
Dim Str_Tmp() As Byte
Dim Var_tmp As Byte
Dim Description As String

Get #hTZXfile, , LoT
ReDim Str_Tmp(LoT - 1)
Get #hTZXfile, , Str_Tmp

For Var_tmp = 0 To LoT - 1
Description = Description + Chr(Str_Tmp(Var_tmp))
Next

Load_TEXT_Description = Description

End Function


Function Load_HWTYPE(info() As TZX_HardWareInfo) As Boolean

Dim Nb_COMP As Byte

Dim Str_Tmp() As Byte
Dim Var_tmp As Byte
Dim Description As String

Get #hTZXfile, , Nb_COMP

ReDim info(Nb_COMP - 1)
For Var_tmp = 0 To Nb_COMP - 1
'Debug.Print Loc(hTZXfile)
Get #hTZXfile, , info(Var_tmp)
'Debug.Print "/"; Get_ExtHard(info(Var_tmp).Hardware_Type, info(Var_tmp).Hardware_ID); "/"; Get_Hard_Inf(info(Var_tmp).Hardware_Info)
Next



End Function


' *********************************************************************

Function Load_GENERAL() As String
Dim Np As Byte ' MaxPulsesBit
Dim Assy As Byte ' NoSymbols
Dim At() As Integer ' Alphabet
Dim Usedbits As Byte 'FinalBits
Dim Pause As Integer
Dim Data_Len As Long
Dim Sp() As Integer
Dim data() As Byte
Dim cmp As Long
Dim HeaderLen As Long
Dim Get_General As eGeneral1
Dim CRC32 As Long
Dim test(63) As Byte
Dim Old_EO_SPEC As Boolean

cmp = Seek(hTZXfile)

      Get #hTZXfile, , Get_General
'      Get #hTZXfile, 14, test

    

If Get_General.TOTD > 0 And Get_General.Pause = 1 Then Old_EO_SPEC = True

'CRC32 = 0
'For a = 0 To 63
'CRC32 = CRC32 + CLng(test(a))
'Next

'Debug.Print "*" & CRC32 & "*"
'If CRC32 = 2805 Then Old = True Else Old = False ' crc32=3053


If Get_General.TOTD > 0 Then
'**************************************************************************************

'0x12
'            - SYMDEF[ASP] Pilot and sync symbols definition table
'            This field is present only if TOTP>0
' SYMDEF structure format
'           0x00 - BYTE Symbol flags
'           b0-b1: starting symbol polarity
'               00: opposite to the current level (make an edge, as usual) - default
'               01: same as the current level (no edge - prolongs the previous pulse)
'               10: force low level
'               11: force high level
'           0x01 - WORD[MAXP] Array of pulse lengths.


'**************************************************************************************
'0x12+(2*NPP+1)*ASP
'            - PRLE[TOTP] Pilot and sync data stream
'           This field is present only if TOTP>0
'
'PRLE structure format
'             0x00 - BYTE Symbol to be represented
'             0x01 - WORD Number of repetitions
'**************************************************************************************

'0x12+(TOTP>0)*((2*NPP+1)*ASP)+TOTP*3
'            - SYMDEF[ASD] Data symbols definition table
'           This field is present only if TOTD>0
'**************************************************************************************
'
'0x12+(TOTP>0)*((2*NPP+1)*ASP)+TOTP*3+(2*NPD+1)*ASD
'            - BYTE[DS] Data stream
'           This field is present only if TOTD>0


End If

If Old_EO_SPEC Then
'/// Old EO Specs. (TZX 1)
Data_Len = Get_General.Bloc_Len - 75
HeaderLen = 77 + 2 ' Specs + 2 bytes lenght
Else
'/// New EO Specs. (TZX 1.2)
HeaderLen = 18 + Get_General.ASD * ((Get_General.NPD * 2) + 1)
 Data_Len = Get_General.Bloc_Len - HeaderLen + 4  ' (4bytes = length)
End If

On Error Resume Next
        ReDim data(Data_Len - 1)
        
        Get #hTZXfile, cmp + HeaderLen, data

For a = 0 To 30
Load_GENERAL = Load_GENERAL & ZXCharToASCII(data(a))
If data(a) And &H80 Then Exit For
Next

If bLoadBloc Or bSaveP Then
If ZX81name = Load_GENERAL Then
a = a + 1
ReDim TZXBuffer(Data_Len - a - 1)
CopyMemory TZXBuffer(0), data(a), Data_Len - a

        If bLoadBloc Then
        CopyMemory MEM(&H4009), TZXBuffer(0), UBound(TZXBuffer)
        bLoadBloc = False
        End If
        
        If bSaveP Then
        hFile = FreeFile
        Open ZX81name + ".P" For Binary As hFile
        Put #hFile, 1, TZXBuffer
        Close #hFile
        bSaveP = False
        End If
End If
End If

Debug.Print Seek(hTZXfile)
Load_GENERAL = Load_GENERAL & "|" & Data_Len & "|" & (cmp + HeaderLen + a) & "|"

End Function
' *********************************************************************


Function Load_TEXT_Message() As TZX_MessageBloc
'Notes:
'stick to a maximum of 30 chars per line;
'use single 0x0D (13 decimal) to separate lines;
'stick to a maximum of 8 lines.

cmp = Seek(hTZXfile)
Dim Tmp_Load_TEXT_Message As TZX_MessageBlocR
Get #hTZXfile, , Tmp_Load_TEXT_Message
Seek #hTZXfile, cmp + Tmp_Load_TEXT_Message.LoT + 2
Load_TEXT_Message.tempo = Tmp_Load_TEXT_Message.Time
Load_TEXT_Message.Message = Left(Tmp_Load_TEXT_Message.Message, Tmp_Load_TEXT_Message.LoT)
Msg_Text = Load_TEXT_Message.Message
Msg_Tempo = Load_TEXT_Message.tempo
End Function

' *********************************************************************
Function Load_Pause() As Integer
Get #hTZXfile, , Load_Pause
End Function

' *********************************************************************
Function Load_TEXT_GS() As String
Dim LoT As Byte
Dim Str_Tmp() As Byte
Dim Var_tmp As Byte
Dim Description As String

Get #hTZXfile, , LoT
ReDim Str_Tmp(LoT - 1)
Get #hTZXfile, , Str_Tmp

For Var_tmp = 0 To LoT - 1
Description = Description + Chr(Str_Tmp(Var_tmp))
Next

Load_TEXT_GS = Description
End Function
' *********************************************************************
Function Load_TEXT_GE() As String
Dim LoT As Byte
Dim Str_Tmp() As Byte
Dim Var_tmp As Byte
Dim Description As String

Get #hTZXfile, , LoT
ReDim Str_Tmp(LoT - 1)
Get #hTZXfile, , Str_Tmp

For Var_tmp = 0 To LoT - 1
Description = Description + Chr(Str_Tmp(Var_tmp))
Next

Load_TEXT_GE = Description
End Function
' *********************************************************************
Function Load_Archive_info(ArchiveInfo() As TZX_ArchiveInfo) As Boolean

cmp = Seek(hTZXfile)
Dim Tmp_Header As TZX_ArchiveInfoHeader
Get #hTZXfile, , Tmp_Header

ReDim ArchiveInfo(Tmp_Header.NBMessage - 1) As TZX_ArchiveInfo
Dim Tmp_info As TZX_ArchiveInfoR
cmp2 = Seek(hTZXfile)

For a = 0 To Tmp_Header.NBMessage - 1
Get #hTZXfile, , Tmp_info
ArchiveInfo(a).IDArchive = Tmp_info.IDArchive
ArchiveInfo(a).Message = Left(Tmp_info.Message, Tmp_info.length)
cmp2 = cmp2 + Tmp_info.length + 2
Seek #hTZXfile, cmp2
Next

'Seek #hTZXfile, Cmp + Tmp_Header.length '+ 2
End Function

' *********************************************************************

Function Load_Custom() As String
Dim Header As String * 16
Get #hTZXfile, , Header
Load_Custom = Trim(Header) & " : "

Select Case UCase(Header)
Case "POKES           "
'Not implemented.
Case "SPECTRUM SCREEN "
'Not implemented.
Case "INSTRUCTIONS    "
Load_Custom = Load_Custom & Load_Instructions
Case "ZX-EDIT DOCUMENT"
'Not implemented.
Case "PICTURE         "
Load_Custom = Load_Custom & Load_Picture
End Select

End Function
' *********************************************************************

Function Load_Instructions() As String
Dim lenght As Long
Dim Inst_STR As String
Dim Inst_Arr() As Byte

Get #hTZXfile, , lenght
ReDim Inst_Arr(lenght - 1)
Get #hTZXfile, , Inst_Arr

For a = 0 To lenght - 1
Inst_STR = Inst_STR + Inst_Arr(a)
Next a

Load_Instructions = Inst_STR

End Function
' *********************************************************************


Function Load_Picture() As String
Dim PicInfo As String
Dim lenght As Long
Dim DLenght As Byte
Dim Format As Byte
Dim Dstr() As Byte
Dim Inst_STR As String
Dim Inst_Arr() As Byte
cmp = Seek(hTZXfile)
Get #hTZXfile, , lenght
cmp = cmp + lenght
Get #hTZXfile, , Format
Get #hTZXfile, , DLenght
If DLenght = 0 Then GoTo iCard
ReDim Dstr(DLenght - 1)
Get #hTZXfile, , Dstr

For a = 0 To UBound(Dstr)
PicInfo = PicInfo + Chr(Dstr(a))
Next a
Load_Picture = PicInfo

iCard:
If bPicture Then
ReDim Inst_Arr(lenght - DLenght - 3)
Get #hTZXfile, , Inst_Arr
Save_Picture Inst_Arr(), Format
bPicture = False
End If

Seek 1, cmp + 4

End Function


Sub Save_Picture(PicArray() As Byte, Format As Byte)

Dim hFile As Integer
Dim Name As String
Select Case Format
Case 0
Name = "Tmp.gif"
Case 1
Name = "Tmp.jpg"
Case 2
Name = "Tmp.bmp"
End Select

Name = "Tmp.pic"

hFile = FreeFile
Open Name For Binary As hFile
Put #hFile, 1, PicArray
Close #hFile
End Sub
' *********************************************************************


Function Get_ExtHard(IdHard As Byte, IdType As Byte) As String
Select Case IdHard
Case 0
Get_ExtHard = Get_Comp(IdType)
Case 1
Get_ExtHard = Get_Store(IdType)
Case 2
Get_ExtHard = Get_AddOns(IdType)
Case 3
Get_ExtHard = Get_Sound(IdType)
Case 4
Get_ExtHard = Get_Joy(IdType)
Case 5
Get_ExtHard = Get_Mouse(IdType)
Case 6
Get_ExtHard = Get_OControler(IdType)
Case 7
Get_ExtHard = Get_SPort(IdType)
Case 8
Get_ExtHard = Get_PPort(IdType)
Case 9
Get_ExtHard = Get_Printer(IdType)
Case 10
Get_ExtHard = Get_Modem(IdType)
Case 11
Get_ExtHard = Get_Digit(IdType)
Case 12
Get_ExtHard = Get_Network(IdType)
Case 13
Get_ExtHard = Get_Keyboard(IdType)
Case 14
Get_ExtHard = Get_ADDA(IdType)
Case 15
Get_ExtHard = Get_Eprom(IdType)
Case 16
Get_ExtHard = Get_GrCard(IdType)
End Select

End Function

Function Get_Comp(cmp As Byte) As String
On Error Resume Next
Get_Comp = Choose(cmp + 1, COMP_00, COMP_01, COMP_02, COMP_03, COMP_04, COMP_05, COMP_06, COMP_07, COMP_08, COMP_09 _
, COMP_0A, COMP_0B, COMP_0C, COMP_0D, COMP_0E, COMP_0F, COMP_10, COMP_11, COMP_12, COMP_13, COMP_14, COMP_15, _
COMP_16, COMP_17, COMP_18, COMP_19, COMP_1A, COMP_1B, COMP_1C, COMP_1D, COMP_1E, COMP_1F, COMP_20, COMP_21, COMP_22 _
, COMP_23, COMP_24, COMP_25, COMP_26, COMP_27, COMP_28, COMP_29, COMP_2A, COMP_2B, COMP_2C, COMP_2D, COMP_2E)
End Function


Function Get_Store(cmp As Byte) As String
On Error Resume Next
Get_Store = Choose(cmp + 1, ExStore_00, ExStore_01, ExStore_02, ExStore_03, ExStore_04, ExStore_05, ExStore_06, ExStore_07, ExStore_08, _
ExStore_09, ExStore_0A, ExStore_0B, ExStore_0C, ExStore_0D, ExStore_0E, ExStore_0F, ExStore_10, ExStore_11, ExStore_12, ExStore_13, _
ExStore_14, ExStore_15, ExStore_16, ExStore_17)
End Function

Function Get_Hard_Inf(cmp As Byte) As String
On Error Resume Next
Get_Hard_Inf = Choose(cmp + 1, eInf_H00, eInf_H01, eInf_H02, eInf_H03)
End Function

Function Get_AddOns(cmp As Byte) As String
On Error Resume Next
Get_AddOns = Choose(cmp + 1, Add_00, Add_01, Add_02, Add_03, Add_04, Add_05, Add_06, Add_07, Add_08, Add_09, Add_0A)
End Function

Function Get_Sound(cmp As Byte) As String
On Error Resume Next
Get_Sound = Choose(cmp + 1, Sound_00, Sound_01, Sound_02, Sound_03, Sound_04, Sound_05, Sound_06, Sound_07, Sound_08, Sound_09, Sound_0A, Sound_0B, Sound_0C, Sound_0D)
End Function

Function Get_Joy(cmp As Byte) As String
On Error Resume Next
Get_Joy = Choose(cmp + 1, Jk_00, Jk_01, Jk_02, Jk_03, Jk_04)
End Function

Function Get_Mouse(cmp As Byte) As String
On Error Resume Next
Get_Mouse = Choose(cmp + 1, Mouse_00, Mouse_01)
End Function

Function Get_OControler(cmp As Byte) As String
On Error Resume Next
Get_OControler = Choose(cmp + 1, OCtrl_00, OCtrl_01, OCtrl_02, OCtrl_03)
End Function

Function Get_SPort(cmp As Byte) As String
On Error Resume Next
Get_SPort = Choose(cmp + 1, SP_00, SP_01)
End Function

Function Get_PPort(cmp As Byte) As String
On Error Resume Next
Get_PPort = Choose(cmp + 1, PP_00, PP_01, PP_02, PP_03, PP_04, PP_05, PP_06, PP_07, PP_08, PP_09, PP_0A)
End Function

Function Get_Printer(cmp As Byte) As String
On Error Resume Next
Get_Printer = Choose(cmp + 1, Printer_00, Printer_01, Printer_02)
End Function

Function Get_Modem(cmp As Byte) As String
On Error Resume Next
Get_Modem = Choose(cmp + 1, Modem_00, Modem_01)
End Function

Function Get_Digit(cmp As Byte) As String
On Error Resume Next
Get_Digit = Choose(cmp + 1, Dg_00, Dg_01, Dg_02, Dg_03)
End Function

Function Get_Network(cmp As Byte) As String
On Error Resume Next
Get_Network = Choose(cmp + 1, Network_00)
End Function

Function Get_Keyboard(cmp As Byte) As String
On Error Resume Next
Get_Keyboard = Choose(cmp + 1, Key_00)
End Function

Function Get_ADDA(cmp As Byte) As String
On Error Resume Next
Get_ADDA = Choose(cmp + 1, ADDAc_00, ADDAc_01)
End Function

Function Get_Eprom(cmp As Byte) As String
On Error Resume Next
Get_Eprom = Choose(cmp + 1, Eprom_00)
End Function

Function Get_GrCard(cmp As Byte) As String
On Error Resume Next
Get_GrCard = Choose(cmp + 1, GrCard_00, GrCard_01, GrCard_02, GrCard_03, GrCard_04)
End Function


' *********************************************************************
' In case of 3 bytes lenght variable,
' We can't get this value within exsting there variables declarations.
' We had to retrive 3bytes (BYTE[3] or STRING[3]) and
' convert it to an LONG variable.(Get_3b sub-routine)

' Type t3Bytes_lenght
' Values(3) As Byte
' End Type
'(...)
' dim 3Bytes_lenght as t3Bytes_lenght
' get #1,,3Bytes_lenght
' Local_value= Get_3b(3Bytes_lenght.Values())

Function Get_3b(bArray() As Byte) As Long
       Get_3b = bArray(0) + bArray(hTZXfile) * 256 + bArray(2) * 65536
End Function

Function GetArray(TxtStr As String, datas() As String) As Integer
' Convert a string : "1|2|3"
' to a string array Datas(0)="1" , Datas(1)="2" and Datas(2)="3".
Dim cmp As Integer
cmp = 0
On Error Resume Next
Do
ReDim Preserve datas(cmp)
datas(cmp) = Left(TxtStr, InStr(1, TxtStr, "|") - 1)
TxtStr = Right(TxtStr, Len(TxtStr) - Len(datas(cmp)) - 1)
If InStr(1, TxtStr, "|") = 0 Then Exit Do
cmp = cmp + 1
Loop
'If Trim(TxtStr) <> "" Then Datas(Cmp) = Right(TxtStr, Len(TxtStr) - Len(Datas(Cmp)) - 1): Cmp = Cmp + 1
GetArray = cmp

End Function

Function GetTZXIcon(BlocID As Byte) As String

Select Case BlocID
Case 0 '"Unknown"
Case &H10  '"Standard speed data block"
GetTZXIcon = "I19"
Case &H11  '"Turbo loading data block"
GetTZXIcon = "I19"
Case &H12  '"Pure tone"
GetTZXIcon = "I15"
Case &H13  '"Sequence of pulses of different lengths"
GetTZXIcon = "I15"
Case &H14  '"Pure data block"
GetTZXIcon = "I15"
Case &H15  '"Direct recording block"
GetTZXIcon = "I15"
Case &H16  '"C64 ROM type data block"
GetTZXIcon = "I16"
Case &H17  '"C64 turbo tape data block"
GetTZXIcon = "I15"
Case &H18  '"CSW"
GetTZXIcon = "I0"
Case &H19  '"Data block"
GetTZXIcon = "I19"
Case &H20  '"Pause (silence) or 'Stop the tape' command"
GetTZXIcon = "I20"
Case &H21  '"Group start"
GetTZXIcon = "I0"
Case &H22  '"Group end"
GetTZXIcon = "I0"
Case &H23  '"Jump to block"
GetTZXIcon = "I0"
Case &H24  '"Loop start"
GetTZXIcon = "I0"
Case &H25  '"Loop end"
GetTZXIcon = "I0"
Case &H26  '"Call sequence"
GetTZXIcon = "I0"
Case &H27  '"Return from sequence"
GetTZXIcon = "I0"
Case &H28  '"Select block"
GetTZXIcon = "I0"
Case &H2A  '"Stop tape if in 48K mode"
GetTZXIcon = "I20"
Case &H30  '"Text description"
GetTZXIcon = "I30"
Case &H31  '"Message block"
GetTZXIcon = "I31"
Case &H32  '"Archive info"
GetTZXIcon = "I32"
Case &H33  '"Hardware type"
GetTZXIcon = "I33"
Case &H34  '"Emulation info"
GetTZXIcon = "I34"
Case &H35  '"Custom info block"
GetTZXIcon = "IPIC"
Case &H40  '"Snapshot block"
GetTZXIcon = "I40"
Case &H5A  '"Glue block" ' (90 dec, ASCII Letter 'Z')
GetTZXIcon = "I0"
Case Else
GetTZXIcon = "I0"
End Select
End Function




' General bloc info.

Sub CreateNewBloc()
Dim Sample_Variant
Dim Sample_Byte() As Byte

 Sample_Variant = Array(&H2, &H12, &H2, &H8, &H2, &H12, &H2, &H8, &H2, &H12, &H2, &H8, &H2, &H12, &H12, &H51, _
                &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0, &H0, _
                &H2, &H12, &H2, &H8, &H2, &H12, &H2, &H8, &H2, &H12, &H2, &H8, &H2, &H12, &H2, &H8, _
                &H2, &H12, &H2, &H8, &H2, &H12, &H2, &H8, &H2, &H12, &H2, &H8, &H2, &H12, &H12, &H51)

' Convert Variant to Bytes ... to Write it in the binary file. (Only 1 byte per value)
' The copy memory can't work with a different type of variable.
ReDim Sample_Byte(UBound(Sample_Variant)) As Byte
For a = 0 To UBound(Sample_Variant)
Sample_Byte(a) = CByte(Sample_Variant(a))
Next

' Kill Sample_Variant!!! hi hi hi ...
Erase Sample_Variant


'Dim nFile As Integer
'nFile = FreeFile
'Open "EXEmple.bin" For Binary Access Write Lock Write As #nFile
'Put #nFile, 1, test
'Close #nFile



End Sub
Sub Cut_Tzx()
Dim Bfr() As Byte
Dim hFileTMP As Integer

For z = 0 To UBound(Block_START) - 1
ReDim Bfr(Block_End(z) - Block_START(z))
Get #hTZXfile, Block_START(z), Bfr
hFileTMP = FreeFile
Open App.path & "\blocks\B" & Format(z, "000") & ".tmp" For Binary Access Write As #hFileTMP
Put #hFileTMP, 1, Bfr
Close #hFileTMP

Next z

End Sub

Sub Save_PictureBlk(Picture_Name As String, ID_Nm As String, Comments As String)
Dim Pic_Buffer()  As Byte
Dim nFile As Integer
Dim Bloc_ID As Byte
Dim Header As String * 16
Dim lenght As Long 'Bloc len
Dim DLenght As Byte 'Comments len
Dim Pic_Type As Byte
Dim Dstr() As Byte '
 
DLenght = Len(Comments)
If DLenght = 0 Then GoTo iCard
ReDim Dstr(DLenght - 1)
For a = 0 To UBound(Dstr)
Dstr(a) = Asc(Mid(Comments, a + 1, 1))
Next a
iCard:

Bloc_ID = &H35
Header = "PICTURE         "

Select Case UCase(Right(Picture_Name, 4))
Case ".GIF"
Pic_Type = 0
Case ".JPG", "JPEG"
Pic_Type = 1
Case ".BMP"
Pic_Type = 2
Case Else
Pic_Type = 0
End Select

nFile = FreeFile
Open Picture_Name For Binary Access Read As #nFile
ReDim Pic_Buffer(LOF(nFile) - 1)
Get #nFile, 1, Pic_Buffer
Close #nFile

lenght = 2 + DLenght + UBound(Pic_Buffer) + 1
nFile = FreeFile
Open App.path & "\blocks\" & ID_Nm For Binary Access Write As #nFile
Put #nFile, 1, Bloc_ID ' [&h1]
Put #nFile, 2, Header '[&h10]
Put #nFile, , lenght '1+1+len(comments)+Pic_Buffer =>[4]
Put #nFile, , Pic_Type '[&h1]
Put #nFile, , DLenght '[&h1]
If DLenght <> 0 Then Put #nFile, , Dstr '[DLenght]
Put #nFile, , Pic_Buffer
Close #nFile

End Sub

Function Load_STANDARD() As String
Dim Blk_info As eStand_DB
Dim data() As Byte
Dim cmp As Long
Dim HeaderLen As Long
Dim SpecName As String

cmp = Seek(hTZXfile)

      Get #hTZXfile, , Blk_info
      

        ReDim data(Blk_info.Len_of_Block - 1)
        
        Get #hTZXfile, cmp + Len(Blk_info), data

For a = 2 To 11
SpecName = SpecName & Chr(data(a))
Next

If bLoadBloc Or bSaveP Then
If ZX81name = Load_STANDARD Then

        If bLoadBloc Then
        bLoadBloc = False
        End If
        
        If bSaveP Then
        hFile = FreeFile
        Open ZX81name + ".BIN" For Binary As hFile
        Put #hFile, 1, data
        Close #hFile
        bSaveP = False
        End If
End If
End If

Debug.Print Seek(hTZXfile)

Dim Ftype As Spec_F_type
Ftype.Tag1 = data(1)
Ftype.Tag2 = data(0)

Debug.Print Hex(Ftype.Tag1); "|"; Hex(Ftype.Tag2)


Select Case Ftype.Tag2
Case &H0
Load_STANDARD = Load_STANDARD & SpecName & " "
Case Else
'Load_STANDARD = Load_STANDARD & Hex(Ftype.Tag2) & "]"
End Select

If Ftype.Tag2 <> 255 Then

Select Case ((Ftype.Tag1 And 63) + 1)

Case 1 '     1  - ZX Basic
             Load_STANDARD = Load_STANDARD & "[Basic "
Case 2 '     2  - ZX NumArr
             Load_STANDARD = Load_STANDARD & "[NumArr "
Case 3 '     3  - ZX StrArr
             Load_STANDARD = Load_STANDARD & "[StrArr "
Case 4 '     4  - ZX Code
             Load_STANDARD = Load_STANDARD & "[Codes "
Case 5 '     5  - ZX Snapshot file  SNP 48k
             Load_STANDARD = Load_STANDARD & "[Snap 48K "
Case 6 '     6  - Microdriv
             Load_STANDARD = Load_STANDARD & "[MicroDRV "
Case 7 '     7  - ZX Screen
             Load_STANDARD = Load_STANDARD & "[Screen "
Case 8 '     8  - Special
             Load_STANDARD = Load_STANDARD & "[Special "
Case 9 '     9  - Snap 128k
             Load_STANDARD = Load_STANDARD & "[Snap 128K "
Case 10 '     10 - Opentype
             Load_STANDARD = Load_STANDARD & "[OpenType "
Case 11 '     11 - ZXExecute
             Load_STANDARD = Load_STANDARD & "[Execute "
Case 12 '     12 - ? (12)
             Load_STANDARD = Load_STANDARD & "[ ? "
Case 13 '     13 - ? (13)
             Load_STANDARD = Load_STANDARD & "[ ? "
Case 14 '     14 - ? (14)
             Load_STANDARD = Load_STANDARD & "[ ? "
Case 15 '     15 - ? (15)
             Load_STANDARD = Load_STANDARD & "[ ? "
Case 16 '  - SAM BASIC program BAS
             Load_STANDARD = Load_STANDARD & "[ SAM BASIC "
Case 17 '  - Numeric array     D ARRAY
             Load_STANDARD = Load_STANDARD & "[ D Array "
Case 18 '  - String array      $ ARRAY
             Load_STANDARD = Load_STANDARD & "[ $ Array "
Case 19 '  - Code file     C
             Load_STANDARD = Load_STANDARD & "[ C Code "
Case 20 '     20  - Screen file       SCREEN$
'                   Mode 1 (Spectrum)
'                   Mode 2 (Linear)
'                   Mode 3 (512x384/4)
'                   Mode 4 (256x192/16)
             Load_STANDARD = Load_STANDARD & "[ Screen$ "
Case 21 '     21 - Subdir (MDos spec.)
             Load_STANDARD = Load_STANDARD & "[ Mdos Dir "
Case 22 '     21 - Def. list
             Load_STANDARD = Load_STANDARD & "[ Def List "
Case Else '  - ???????? -
Load_STANDARD = Load_STANDARD & "[" & Hex(Ftype.Tag1) & "|"
End Select

Select Case Ftype.Tag2
Case &H0
Load_STANDARD = Load_STANDARD & "Header]"
Case Else
Load_STANDARD = Load_STANDARD & Hex(Ftype.Tag2) & "]"
End Select

Else

Select Case Ftype.Tag1
Case 0
Load_STANDARD = Load_STANDARD & "[Basic prg]"
Case Else
Load_STANDARD = Load_STANDARD & "[Block]"
End Select
End If


Load_STANDARD = Load_STANDARD & " + " & (Blk_info.Pause / 1000) & "s. " & "~" & Blk_info.Len_of_Block & " b."

End Function




' ID 20 - Pause (silence) or 'Stop the tape' command
Sub Save_Si_Blk(ID_Nm As String, duration As Integer)
Dim Bloc_ID As Byte
Bloc_ID = &H20

nFile = FreeFile
Open App.path & "\blocks\" & ID_Nm For Binary Access Write As #nFile
Put #nFile, 1, Bloc_ID ' [&h25]
Put #nFile, 2, duration ' ms.
Close #nFile

End Sub

' ID 21 - Group start
Sub Save_Gs_Blk(ID_Nm As String, Gr_Name As String)
Dim GRName As String * 29
Dim lenght As Byte
Dim Bloc_ID As Byte
Bloc_ID = &H21
GRName = Gr_Name
lenght = 29
nFile = FreeFile
Open App.path & "\blocks\" & ID_Nm For Binary Access Write As #nFile
Put #nFile, 1, Bloc_ID ' [&h21]
Put #nFile, 2, lenght
Put #nFile, 3, GRName
Close #nFile
End Sub

' ID 22 - Group end
Sub Save_Ge_Blk(ID_Nm As String)
Dim Bloc_ID As Byte
Bloc_ID = &H22

nFile = FreeFile
Open App.path & "\blocks\" & ID_Nm For Binary Access Write As #nFile
Put #nFile, 1, Bloc_ID ' [&h22]
Close #nFile
End Sub

' ID 25 - Loop end.
Sub Save_LE_Blk(ID_Nm As String)
Dim Bloc_ID As Byte

Bloc_ID = &H25

nFile = FreeFile
Open App.path & "\blocks\" & ID_Nm For Binary Access Write As #nFile
Put #nFile, 1, Bloc_ID ' [&h25]
Close #nFile

End Sub

' ID 27 - Return Sequence.
Sub Save_RS_Blk(ID_Nm As String)
Dim Bloc_ID As Byte

Bloc_ID = &H27

nFile = FreeFile
Open App.path & "\blocks\" & ID_Nm For Binary Access Write As #nFile
Put #nFile, 1, Bloc_ID ' [&h27]
Close #nFile

End Sub

' ID 28 - Select block
Sub Save_SB_Blk(ID_Nm As String, eSelectBloc As eSelectBloc, selection() As String)
Dim Bloc_ID As Byte

Bloc_ID = &H28

nFile = FreeFile
Open App.path & "\blocks\" & ID_Nm For Binary Access Write As #nFile
Put #nFile, 1, Bloc_ID ' [&h28]
Close #nFile

'0x00 - WORD Length of the whole block (without these two bytes)
'0x02 N BYTE Number of selections
'0x03 - SELECT[N] List of selections

'Type eSelectBloc
'Len As Integer
'Nb As Byte
'End Type

End Sub

'ID FE - End Of Tape.
Sub Save_EOT_Blk(ID_Nm As String)
Dim Bloc_ID As Byte

Bloc_ID = &HFE

nFile = FreeFile
Open App.path & "\blocks\" & ID_Nm For Binary Access Write As #nFile
Put #nFile, 1, Bloc_ID ' [&hFE]
Close #nFile

End Sub

