Attribute VB_Name = "modZX81"

' /*******************************************************************************
'   modZX81.bas within vb81.vbp
'
'   Routines for emulating the ZX81 hardware; displaying the
'   video memory (reads DFILE directory), reading the keyboard (port
'   &hFE), and loading and saving from 'tape' (*.P files)
'
'   Author: Chris Cowley <ccowley@grok.co.uk>
'
'   Copyright (C)2000-2002  Grok Developments Ltd.
'   http://www.grok.co.uk/
'
'   This program is free software; you can redistribute it and/or
'   modify it under the terms of the GNU General Public License
'   as published by the Free Software Foundation; either version 2
'   of the License, or (at your option) any later version.
'   This program is distributed in the hope that it will be useful,
'   but WITHOUT ANY WARRANTY; without even the implied warranty of
'   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'   GNU General Public License for more details.
'
'   You should have received a copy of the GNU General Public License
'   along with this program; if not, write to the Free Software
'   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
'
' *******************************************************************************
'  VB81 XuR updated by XavSnap. (2007-2010)
'  http://zx81.VB81.free.fr
'  xavsnap@free.fr
' *******************************************************************************

Option Base 0

' Private Constants:

Private Declare Function SetBitmapBits Lib "gdi32" (ByVal hBitmap As Long, ByVal dwCount As Long, lpBits As Any) As Long

    Private Const SRCCOPY = &HCC0020
    Private Const SRCINVERT = &H660046
    Private Const BLACKNESS = &H42
    Private Const WHITENESS = &HFF0062
    Private Const SRCAND = &H8800C6
    Private Const SRCERASE = &H440328
    Private Const SRCPAINT = &HEE0086

Option Explicit

Function doKey(down As Boolean, Ascii As Integer, mods As Integer) As Boolean
Debug.Print "key:"; down, Ascii, mods

' CTRL Key pressed - Menu Short-Cut.
If mods = 2 Then doKey = False: Exit Function
    
    Dim CAPS As Boolean
   
    CAPS = mods And 1

    If CAPS Then keyCAPS_V = (keyCAPS_V And (Not 1&)) Else keyCAPS_V = (keyCAPS_V Or 1&)
    
    Select Case Ascii
    Case 8 ' Backspace
        If down Then
            key6_0 = (key6_0 And (Not 1&))
            keyCAPS_V = (keyCAPS_V And (Not 1&))
        Else
            key6_0 = (key6_0 Or 1&)
            If Not CAPS Then
                keyCAPS_V = (keyCAPS_V Or 1&)
            End If
        End If
    Case 16  ' SHIFT
        If down Then keyCAPS_V = (keyCAPS_V And (Not 1&)) Else keyCAPS_V = (keyCAPS_V Or 1&)
    Case 65 ' A
        If down Then keyA_G = (keyA_G And (Not 1&)) Else keyA_G = (keyA_G Or 1&)
    Case 66 ' B
        If down Then keyB_SPC = (keyB_SPC And (Not 16&)) Else keyB_SPC = (keyB_SPC Or 16&)
    Case 67 ' C
        If down Then keyCAPS_V = (keyCAPS_V And (Not 8&)) Else keyCAPS_V = (keyCAPS_V Or 8&)
    Case 68 ' D
        If down Then keyA_G = (keyA_G And (Not 4&)) Else keyA_G = (keyA_G Or 4&)
    Case 69 ' E
        If down Then keyQ_T = (keyQ_T And (Not 4&)) Else keyQ_T = (keyQ_T Or 4&)
    Case 70 ' F
        If down Then keyA_G = (keyA_G And (Not 8&)) Else keyA_G = (keyA_G Or 8&)
    Case 71 ' G
        If down Then keyA_G = (keyA_G And (Not 16&)) Else keyA_G = (keyA_G Or 16&)
    Case 72 ' H
        If down Then keyH_ENT = (keyH_ENT And (Not 16&)) Else keyH_ENT = (keyH_ENT Or 16&)
    Case 73 ' I
        If down Then keyY_P = (keyY_P And (Not 4&)) Else keyY_P = (keyH_ENT Or 4&)
    Case 74 ' J
        If down Then keyH_ENT = (keyH_ENT And (Not 8&)) Else keyH_ENT = (keyH_ENT Or 8&)
    Case 75 ' K
        If down Then keyH_ENT = (keyH_ENT And (Not 4&)) Else keyH_ENT = (keyH_ENT Or 4&)
    Case 76 ' L
        If down Then keyH_ENT = (keyH_ENT And (Not 2&)) Else keyH_ENT = (keyH_ENT Or 2&)
    Case 77 ' M
        If down Then keyB_SPC = (keyB_SPC And (Not 4&)) Else keyB_SPC = (keyB_SPC Or 4&)
    Case 78 ' N
        If down Then keyB_SPC = (keyB_SPC And (Not 8&)) Else keyB_SPC = (keyB_SPC Or 8&)
    Case 79 ' O
        If down Then keyY_P = (keyY_P And (Not 2&)) Else keyY_P = (keyY_P Or 2&)
    Case 80 ' P
        If down Then keyY_P = (keyY_P And (Not 1&)) Else keyY_P = (keyY_P Or 1&)
    Case 81 ' Q
        If down Then keyQ_T = (keyQ_T And (Not 1&)) Else keyQ_T = (keyQ_T Or 1&)
    Case 82 ' R
        If down Then keyQ_T = (keyQ_T And (Not 8&)) Else keyQ_T = (keyQ_T Or 8&)
    Case 83 ' S
        If down Then keyA_G = (keyA_G And (Not 2&)) Else keyA_G = (keyA_G Or 2&)
    Case 84 ' T
        If down Then keyQ_T = (keyQ_T And (Not 16&)) Else keyQ_T = (keyQ_T Or 16&)
    Case 85 ' U
        If down Then keyY_P = (keyY_P And (Not 8&)) Else keyY_P = (keyY_P Or 8&)
    Case 86 ' V
        If down Then keyCAPS_V = (keyCAPS_V And (Not 16&)) Else keyCAPS_V = (keyCAPS_V Or 16&)
    Case 87 ' W
        If down Then keyQ_T = (keyQ_T And (Not 2&)) Else keyQ_T = (keyQ_T Or 2&)
    Case 88 ' X
        If down Then keyCAPS_V = (keyCAPS_V And (Not 4&)) Else keyCAPS_V = (keyCAPS_V Or 4&)
    Case 89 ' Y
        If down Then keyY_P = (keyY_P And (Not 16&)) Else keyY_P = (keyY_P Or 16&)
    Case 90 ' Z
        If down Then keyCAPS_V = (keyCAPS_V And (Not 2&)) Else keyCAPS_V = (keyCAPS_V Or 2&)
    Case 48 ' 0
        If down Then key6_0 = (key6_0 And (Not 1&)) Else key6_0 = (key6_0 Or 1&)
    Case 49 ' 1
        If down Then key1_5 = (key1_5 And (Not 1&)) Else key1_5 = (key1_5 Or 1&)
    Case 50 ' 2
        If down Then key1_5 = (key1_5 And (Not 2&)) Else key1_5 = (key1_5 Or 2&)
    Case 51 ' 3
        If down Then key1_5 = (key1_5 And (Not 4&)) Else key1_5 = (key1_5 Or 4&)
    Case 52 ' 4
        If down Then key1_5 = (key1_5 And (Not 8&)) Else key1_5 = (key1_5 Or 8&)
    Case 53 ' 5
        If down Then key1_5 = (key1_5 And (Not 16&)) Else key1_5 = (key1_5 Or 16&)
    Case 54 ' 6
        If down Then key6_0 = (key6_0 And (Not 16&)) Else key6_0 = (key6_0 Or 16&)
    Case 55 ' 7
        If down Then key6_0 = (key6_0 And (Not 8&)) Else key6_0 = (key6_0 Or 8&)
    Case 56 ' 8
        If down Then key6_0 = (key6_0 And (Not 4&)) Else key6_0 = (key6_0 Or 4&)
    Case 57 ' 9
        If down Then key6_0 = (key6_0 And (Not 2&)) Else key6_0 = (key6_0 Or 2&)
    Case 96 ' Keypad 0
        If down Then key6_0 = (key6_0 And (Not 1&)) Else key6_0 = (key6_0 Or 1&)
    Case 97 ' Keypad 1
        If down Then key1_5 = (key1_5 And (Not 1&)) Else key1_5 = (key1_5 Or 1&)
    Case 98 ' Keypad 2
        If down Then key1_5 = (key1_5 And (Not 2&)) Else key1_5 = (key1_5 Or 2&)
    Case 99 ' Keypad 3
        If down Then key1_5 = (key1_5 And (Not 4&)) Else key1_5 = (key1_5 Or 4&)
    Case 100 ' Keypad 4
        If down Then key1_5 = (key1_5 And (Not 8&)) Else key1_5 = (key1_5 Or 8&)
    Case 101 ' Keypad 5
        If down Then key1_5 = (key1_5 And (Not 16&)) Else key1_5 = (key1_5 Or 16&)
    Case 102 ' Keypad 6
        If down Then key6_0 = (key6_0 And (Not 16&)) Else key6_0 = (key6_0 Or 16&)
    Case 103 ' Keypad 7
        If down Then key6_0 = (key6_0 And (Not 8&)) Else key6_0 = (key6_0 Or 8&)
    Case 104 ' Keypad 8
        If down Then key6_0 = (key6_0 And (Not 4&)) Else key6_0 = (key6_0 Or 4&)
    Case 105 ' Keypad 9
        If down Then key6_0 = (key6_0 And (Not 2&)) Else key6_0 = (key6_0 Or 2&)
    Case 106 ' Keypad *
        If down Then
            keyB_SPC = (keyB_SPC And (Not 16&))
            keyCAPS_V = (keyCAPS_V And (Not 1&))
        Else
            keyB_SPC = (keyB_SPC Or 16&)
            If Not CAPS Then
                keyCAPS_V = (keyCAPS_V Or 1&)
            End If
        End If
    Case 107 ' Keypad +
        If down Then
            keyH_ENT = (keyH_ENT And (Not 4&))
            keyCAPS_V = (keyCAPS_V And (Not 1&))
        Else
            keyH_ENT = (keyH_ENT Or 4&)
            If Not CAPS Then
                keyCAPS_V = (keyCAPS_V Or 1&)
            End If
        End If
    Case 109 ' Keypad -
        If down Then
            keyH_ENT = (keyH_ENT And (Not 8&))
            keyCAPS_V = (keyCAPS_V And (Not 1&))
        Else
            keyH_ENT = (keyH_ENT Or 8&)
            If Not CAPS Then
                keyCAPS_V = (keyCAPS_V Or 1&)
            End If
        End If
    Case 110 ' Keypad .
        If down Then keyB_SPC = (keyB_SPC And (Not 2&)) Else keyB_SPC = (keyB_SPC Or 2&)
    Case 111 ' Keypad /
        If down Then
            keyCAPS_V = (keyCAPS_V And (Not 17&))
        Else
            If Not CAPS Then
                keyCAPS_V = (keyCAPS_V Or 17&)
            Else
                keyCAPS_V = (keyCAPS_V Or 16&)
            End If
        End If
        
    Case 37 ' Left "5"
                Get_UserKeys down, 5

    Case 38 ' Up "7"
                Get_UserKeys down, 7
    
    Case 39 ' Right "8"
                Get_UserKeys down, 8
    
    Case 40 ' Down "6"
                Get_UserKeys down, 6


    Case 13 ' RETURN
        If down Then keyH_ENT = (keyH_ENT And (Not 1&)) Else keyH_ENT = (keyH_ENT Or 1&)
    Case 32 ' SPACE BAR
        If down Then keyB_SPC = (keyB_SPC And (Not 1&)) Else keyB_SPC = (keyB_SPC Or 1&)
    Case 190 ' .
        If down Then keyB_SPC = (keyB_SPC And (Not 2&)) Else keyB_SPC = (keyB_SPC Or 2&)
    Case Else
        doKey = False
    End Select
        
    'French Keyboard assignement.
   ' If bKbd = Fr Then
   ' Select Case Ascii
   ' Case 188 ',
   ' If mods = 1 Then
  '  If down Then keyCAPS_V = (keyCAPS_V And (Not 8&)) Else keyCAPS_V = (keyCAPS_V Or 8&)
  '  Else
  '  If Not down Then keyB_SPC = (keyB_SPC And (Not 2&)) Else keyB_SPC = keyB_SPC Or 2&
  '  End If
  '  End Select
  '  End If
                'Debug.Print "*"; eCursors.K5.Row, eCursors.K5.Col, down
                Debug.Print key1_5
                Debug.Print key6_0
                Debug.Print keyQ_T
                Debug.Print keyY_P
                Debug.Print keyA_G
                Debug.Print keyH_ENT
                Debug.Print keyCAPS_V
                Debug.Print keyB_SPC
                
 Update_Kybd
      
    doKey = True
End Function



Public Sub HiresScreenPaint()
    Dim x As Integer, y As Integer
    Dim c As Long
    Dim IntR_ As Long
    Dim IntI_ As Long
               
    Dim lTopMost As Long, lBottomMost As Long, lLeftMost As Long, lRightMost As Long
    Dim bUpdated As Boolean
    
    lTopMost = 191
    lBottomMost = 0
    lLeftMost = 31 ' Screen res 34x24?
    lRightMost = 0
    
      
    Select Case Display_WRX

    Case 3
' -----------------------------------------------
' Mode 3 :
' -----------------------------------------------
    
    CopyMemory gcBufferBits(0), MEM(8192), 6144
    
    Case 1
' -----------------------------------------------
' Mode 1 : Rock-Crush ; Micro-Mouse ; Rocket-man.
' -----------------------------------------------
    For y = 0 To 191
        For x = 0 To 31
        
               c = MEM(lHiresLoc + x + y * 33) 'And &HBF& 'And &HBF&    ' strip bit 6

            If (c <> cLastHiResScreen(y, x) Or bForceDisplay) Then
                If y < lTopMost Then lTopMost = y
                If y > lBottomMost Then lBottomMost = y
                If x < lLeftMost Then lLeftMost = x
                If x > lRightMost Then lRightMost = x
                bUpdated = True
                
                     
                    ' // Inverse video (B7=1) ;   Normal video (B7=0)
                    gcBufferBits(y * 32 + x) = MEM((intI * 256) + (c And 127) * 8) Xor (IIf((c And 128) <> 0, 255, 0)) Xor IIf((c And 64) <> 0, 40, 0)
                   ' gcBufferBits(Y * 32 + X) = (MEM((intI * 256) + IIf((c And 128) <> 0, c And 63, c) * 8)) Xor (IIf((c And 128) <> 0, 255, 0))
                    
                    If (c And 128) <> 0 Then
                    ' // Inverse video
                    gcBufferBits(y * 32 + x) = MEM((intI * 256) + (c And 63) * 8) Xor 255
                    
                Else
                    ' // Normal video
                    gcBufferBits(y * 32 + x) = MEM(((intI * 256) + c * 8))
                    End If
                    
                    IntR_ = (lHiresLoc + x + y * 33) And 255!
                    IntI_ = (lHiresLoc + x + y * 33) And 65280!
                End If
             
                cLastHiResScreen(y, x) = c
            
        Next x
    Next y
    
    lLeftMost = lLeftMost * 8
    lRightMost = lRightMost * 8

    If bUpdated Then gpicDisplay.Visible = False: StretchDIBits gpicDisplay.hDC, lLeftMost * glDisplayXMultiplier, (lBottomMost + 1) * glDisplayYMultiplier, (lRightMost - lLeftMost + 8) * glDisplayXMultiplier, -(lBottomMost - lTopMost + 1) * glDisplayYMultiplier, lLeftMost, lTopMost, (lRightMost - lLeftMost) + 8, lBottomMost - lTopMost + 1, gcBufferBits(0), bmiBufferHR, DIB_RGB_COLORS, SRCCOPY
    gpicDisplay.Visible = True
    
   Case 2
' -----------------------------------------------
' Mode 2 :
' -----------------------------------------------
    For y = 0 To 191
        For x = 0 To 31
        
               c = MEM(lHiresLoc + x + y * 33) ' And &HBF&) ' strip bit 6
               'Dim IntR_
              ' Dim IntI_
             '  IntR_ = (lHiresLoc + X + y * 33) 'And 255
               
            If (c <> cLastHiResScreen(y, x) Or bForceDisplay) Then
                If y < lTopMost Then lTopMost = y
                If y > lBottomMost Then lBottomMost = y
                If x < lLeftMost Then lLeftMost = x
                If x > lRightMost Then lRightMost = x
                bUpdated = True
                
                If (G_Card = eHiResWRX) Then
                
                    ' // WRX video
                    gcBufferBits(y * 32 + x) = (MEM((intI * 256) + (c And 63) * 8) Xor (IIf((c And 128) <> 0, 255, 0)))
                    
                      Else
                      
                    ' // Inverse video (B7=1) ;   Normal video (B7=0)
                    gcBufferBits(y * 32 + x) = MEM((intI * 256) + (c And 127) * 8) Xor (IIf((c And 128) <> 0, 255, 0)) Xor IIf((c And 64) <> 0, 40, 0)

                    
                      '           gcBufferBits(y * 32 + X) = MEM((intI * 256) + (c And 63) * 8) Xor (IIf((c And 128) <> 0, 255, 0))
                    IntR_ = (lHiresLoc + x + y * 33) And 255
                    IntI_ = (lHiresLoc + x + y * 33) And 65280
                     ' data=readbyte((z80.i<<8) | (z80.r7 & 128) | ((z80.r-1) & 127));
                    'gcBufferBits(y * 32 + X) = MEM((intI * 256) + (intR And 128) + ((intR - 1) And 127)) ' Xor 255
                End If
                
                cLastHiResScreen(y, x) = c
            End If
        Next x
    Next y
    
    lLeftMost = lLeftMost * 8
    lRightMost = lRightMost * 8

    If bUpdated Then gpicDisplay.Visible = False: StretchDIBits gpicDisplay.hDC, lLeftMost * glDisplayXMultiplier, (lBottomMost + 1) * glDisplayYMultiplier, (lRightMost - lLeftMost + 8) * glDisplayXMultiplier, -(lBottomMost - lTopMost + 1) * glDisplayYMultiplier, lLeftMost, lTopMost, (lRightMost - lLeftMost) + 8, lBottomMost - lTopMost + 1, gcBufferBits(0), bmiBufferHR, DIB_RGB_COLORS, SRCCOPY
    gpicDisplay.Visible = True
End Select


End Sub


Function inb(ByVal port As Long) As Long
' IO port list:
'-----------------------------------------------------------
'&1Fh/31 : Kempston joystick (Default:)
'&2Fh/47 : Memotech Centronics IF card. (\ACK : Send a BIT)
'&3Fh/63 : VTR SPEECH ENGINE CARD ( Data ).
'&3Fh/63 : 8 E/S Card ( Port)
'&3Fh/63 : Memotech Centronics IF card. ( a byte is sent get error?)
'&41h/65 : Larken Disk Drive ( Data ).
'&43h/67 : Larken Disk Drive ( Head Stat ).
'&45h/69 : Larken Disk Drive.( Port in Adrress : 12300 / 300Ch )
'&5Fh/95 : Memotech HRG card.
'&7Dh/125: Mageco/VTR Light-Pen(X datas).
'&7Fh/127: Mageco/VTR Light-Pen(Y datas).
'&DFh/223: QSound/ZonX.
'&DFh/223: AGB JOYSTICK II ( Read Datas ).
'&EFh/239: AGB JOYSTICK II ( Read Datas ).
'&FBh/251: Printer read Info.
'&FEh/254: Keyboard read Info.
'-----------------------------------------------------------

'Debug.Print "Input Port:"; Hex(port)
Dim ret As Long
Dim res As Long
Dim JPort_ROW As Byte
Dim JPort_COL As Byte
Dim Port_ID As Byte
Dim Port_DATA As Byte
Dim lTopMost As Long, lBottomMost As Long, lLeftMost As Long, lRightMost As Long


Port_ID = CByte(port And &HFF)
Port_DATA = CByte(glMemAddrDiv256(port And &HFF00&))


 '// 3Fh/63 - VTR SPEECH ENGINE CARD ( Data ).
 '//
If Port_ID = &H3F And S_Card = esSPO256 Then
 Debug.Print "VTR SPEECH ENGINE CARD (63): "; Hex(port)
 ' IN ($3F) = 254 (BUSY) = 255 (FREE)
        If bSPO Then inb = 254 Else inb = 255
        Exit Function
 End If
 
'// 4Fh/79 - Parrot SPEECH ENGINE CARD ( Data ).
 '//
If Port_ID = &H4F And S_Card = esParrotSPO256 Then
 Debug.Print "Parrot SPEECH ENGINE CARD (79): "; Hex(port)
 ' IN ($4F) = 254 (BUSY) = 255 (FREE)
        If bSPO Then inb = 254 Else inb = 255
        Exit Function
 End If
 
 
             '  case 73h :TS2050 D8251readDATA sound card.
             '  case 77h :TS2050 D8251readCTRLDATA sound card.
             '  case DDh :Jupiter ACE sound card.
                       
'// 8 E/S Card (3Fh Port)  [bI8O8 = True]
'If Port_ID = &H3F Then
'Debug.Print " 8 I/O Card: input="; Port_DATA ' You can assign res= &hff if all Input are set to hight level.
'End If

 '// 2Fh/47 - Memotech Centronics IF.
 '//
If Port_ID = &H2F And PrinterCfg = 1 Then

        Static fH As Integer
        On Error GoTo Nxtstep
            fH = FreeFile
            Open App.path & "\Printer_out\Printer_CIF.txt" For Binary As fH
            Put #fH, LOF(fH) + 1, CByte(Port_DATA)
            Close #fH
Nxtstep:
            reset
            If Frm_MemotechCIF.Visible Then Frm_MemotechCIF.Display_Centro_Datas Port_DATA
            
            Dim MyPRTchar As String
            Select Case Port_DATA
            Case &HA
            MyPRTchar = vbCrLf

            Case 0
            MyPRTchar = ""
            Case Is < 32
            MyPRTchar = "[" & Port_DATA & "]"
            Case Else
            MyPRTchar = Chr(Port_DATA)
            End Select
            
            FrmPrinter.ASCII_PRT.Enabled = False
            FrmPrinter.ASCII_PRT.text = FrmPrinter.ASCII_PRT.text + MyPRTchar
            FrmPrinter.ASCII_PRT.SelStart = Len(FrmPrinter.ASCII_PRT)
            FrmPrinter.ASCII_PRT.Enabled = True
            
                    inb = Port_DATA
        Exit Function
End If


 '// 3Fh/63 - Memotech Centronics IF.
 '//
If Port_ID = &H3F And PrinterCfg = 1 Then
        Debug.Print "Centronics_update:" & Port_DATA
        inb = 130
        Exit Function
End If




'QSound/ZonX: &HDF
If Port_ID = &HDF Then
       inb = AYPSG.Regs(glSoundRegister)
       Exit Function
End If


'  // LARKEN DISK DRIVE CONTROLER.
' // Port 41h : Datas
' // Port 43h :
' // Port 45h :

'// 41h - Larken Disk Drive ( Data ).
If Port_ID = &H41 Then
 Debug.Print "Input Port Larken (65): "; Hex(port)

        inb = &HFF - 1
        Exit Function

 End If
  
'// 43h - 67 : Larken Disk Drive ( Head Stat ).
If Port_ID = &H43 Then
 Debug.Print "Input Port Larken (43h): "; Hex(port)

'                                                0x0001 : B0
' /* set if disc is write protected - also set if drive is present but no disc in drive */
'#define FLOPPY_DRIVE_DISK_WRITE_PROTECTED       0x0002 : B1
'/* set if drive is connected and head is positioned over track 0 */
'#define FLOPPY_DRIVE_HEAD_AT_TRACK_0            0x0004 : B2

'Next track                                      0x0008 : B3

'/* set if drive is ready */
'#define FLOPPY_DRIVE_READY                      0x0010 : B4
'/* set if index has just occured */
'#define FLOPPY_DRIVE_INDEX                      0x0020 : B5
''/* motor state */
'#define FLOPPY_DRIVE_                   0x0040 : B6
'                                                0x0080 : B7

Motor_On = ((Motor_On Xor &HFF) And &H1E)

        inb = Motor_On
        Exit Function

 End If
 
'// 45h - Larken Disk Drive ( 45h... Port in Adrress : 12300 / 300Ch ).
 If Port_ID = &H45& Then
 Debug.Print "Input Port Larken: "; Hex(port)
        inb = &H26 'Disk_Data And &HFF
        Exit Function
' "CRC ER"
 End If
 
 
'// 5Fh - Memotech HRG card.
 If Port_ID = &H5F& And G_Card = eHiResMEMOTECH Then
 
 
' Debug.Print "Input Port MEMOTECH: "; Hex(port)

   If Mem_Display = 0 Then
               ' Clear display buffer.
               ClrMemory sLastScreen(0), 768
               
               ' Clear display buffer.
               ClrMemory gcBufferBits(0), 6528
               
               ' Clear Basic display.
                gpicDisplay.Cls
  End If
       
    
    Mem_Display = CByte(Int(port \ 256)) ' 0 for Basic Display, 2 for HRG buffer and 3 for Inverted display.
    If Mem_Display = 0 Then FrmMainWnd.DispHRGon Else FrmMainWnd.DispHRGoff
'    Bit0= 0=> Normal Basic Mode / 1=> Inverted Display.
'    Bit1= 0=> Rom (Set RomCs'=1) / 1=> Memotech buffer actived.(0h to 400h)

  Exit Function

 End If
 
 
 If Port_ID = &HFB And PrinterCfg = 2 Then
     '// BFh - AGB Centronics IF.
     '//
     inb = CByte(gCentronic)
     gCentronic = Not gCentronic
     Exit Function
 End If
 
'// Printer read Info. (FBh Port)

' read
' D0   encoder:  0=between   1=pixel
' D2
' D3
' D4
' D5
' D6   printer:  0=present   1=no printer
' D7   stylus:     0=off paper   1=on paper

If Port_ID = &HFB& And PrinterCfg = 0 Then

    If G_Card = eHiResQS Then
    inb = (regA Xor 1) + 128
    Exit Function
    End If

'Print #hFile_PRT, "IN:"; ConvBin(CDbl(Port_DATA), 8)

      If Port_DATA = &HFF& Then
         GoTo Fast_Scan
       ' 0    Data Request  (0=Busy, 1=Ready/DRQ)
        If (Port_DATA And 1) = 0 Then Debug.Print "Busy" Else Debug.Print "Ready/DRQ"
       ' 1    Not used
        If (Port_DATA And 2) = 0 Then Debug.Print " A1: 0" Else Debug.Print " A1: 1"
       ' 2    Not used
        If (Port_DATA And 4) = 0 Then Debug.Print " A2: 0" Else Debug.Print " A2: 1"
       ' 3    Not used
        If (Port_DATA And 8) = 0 Then Debug.Print " A3: 0" Else Debug.Print " A3: 1"
       ' 4    Not used
        If (Port_DATA And 16) = 0 Then Debug.Print " A4: 0" Else Debug.Print " A4: 1"
       ' 5    Not used
        If (Port_DATA And 32) = 0 Then Debug.Print " A5: 0" Else Debug.Print " A5: 1"

       ' 6    Printer Detect(0 = Okay, 1 = None)
        If (Port_DATA And 64) = 0 Then Debug.Print "Okay" Else Debug.Print "None"
       ' 7    Printer Online(0 = Nope, 1 = Ready / Online)
        If (Port_DATA And 128) = 0 Then Debug.Print "Nope" Else Debug.Print "Ready / Online"
        
Fast_Scan:

       ' INVADER "LPRINT" BUGG.
       If bQSChargen Then
       inb = IIf(bQS_ON, 127, 128)
       Else
       inb = IIf(cPrinter.Get_Paper, 128, 0) + 0 ' 0=ZxPrinter Present
       End If
       Exit Function
     End If
     
    End If   'End ZX Printer
       
       
       
       
' CZ1500 - CBM1541 disk interface
'
'+---+---+---+---+---+---+---+---+
'| 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |   Port 0BFH
'+---+---+---+---+---+---+---+---+
'  |   |   |   |   |   |       |
'  |   |   |   |   |   |       +--> R/O Busy status line from EEPROM active low
'  |   |   |   |   |   |
'  |   |   |   |   |   +
'  |   |   |   |   +
'  |   |   |   +
'  |   |   +
'  |   +--------------------------> R/O Serial clock input           active low
'  +------------------------------> R/O Serial data input            active low

'Bits 2-7 are used in the same way as in the Commodore 64.
'Bit 1 selects the 16k ram bank at address 8000H-BFFFH 0=user 1=system.
'Bit 0 returns the state of the EEPROM busy line, used during EEPROM programming
'    0=memory busy 1=ready to accept data.
      
      
  If Port_ID = &HBF& Then
  inb = &H0 'FF 'Xor &HFF
  'inb = &H0 Xor &H40 Xor &H80 + 4
  'CBM1541_CLK = CBM1541_CLK + 1
  Debug.Print "clk:" & CBM1541_CLK
  'EEprom active
  
  '       inb = CBM1541_CLK Xor &HC1
  '       Exit Function
  End If

'Mageco/VTR Light-Pen X columns(&H7D)
If Port_ID = &H7D And bLpen Then
    inb = LpenX
    Exit Function
End If
'Mageco/VTR Light-Pen Y raws(&H7F)
If Port_ID = &H7F And bLpen Then
    inb = LpenY
    Exit Function
End If


'//  DFh + EFh - AGB JOYSTICK II ( Read Datas ).
'PORT A
If Port_ID = &HDF Then
    inb = 0
    JPort_ROW = MEM(16421) Xor 255
    JPort_COL = MEM(16422) Xor 255
    If JPort_ROW = 0 Or JPort_COL = 0 Then Exit Function
    
    ' Cursors keys or 5/6/7/8 action:N/L
    If (JPort_ROW And 8) = 8 Then If (JPort_COL And 32) = 32 Or (JPort_COL And 33) = 33 Then inb = inb + 2 'Left
    
    If (JPort_ROW And 16) = 16 Then
    If (JPort_COL And 32) = 32 Or (JPort_COL And 33) = 33 Then inb = inb + 4 'Down
    If (JPort_COL And 16) = 16 Or (JPort_COL And 17) = 17 Then inb = inb + 8 'UP
    If (JPort_COL And 8) = 8 Or (JPort_COL And 9) = 9 Then inb = inb + 1 'Right
    End If

    If (JPort_ROW And 64) = 64 Then If (JPort_COL And 2) = 2 Or (JPort_COL And 3) = 3 Then inb = inb + 16 ' Fire
    Exit Function
 End If
 
 'PORT B
 If Port_ID = &HEF Then
    inb = 0
    JPort_ROW = MEM(16421) Xor 255
    JPort_COL = MEM(16422) Xor 255
    If JPort_ROW = 0 Or JPort_COL = 0 Then Exit Function
    
    ' A,S,W,D action:F
    If (JPort_ROW And 4) = 4 Then If (JPort_COL And 4) = 4 Then inb = inb + 8 'UP
    
    If (JPort_ROW And 2) = 2 Then
    If (JPort_COL And 2) = 2 Then inb = inb + 2 'Left
    If (JPort_COL And 4) = 4 Then inb = inb + 4 'Down
    If (JPort_COL And 8) = 8 Then inb = inb + 1  'Right
    If (JPort_COL And 16) = 16 Then inb = inb + 16 ' Fire
    End If
    Exit Function
 End If

'// Kempston joystick (Default:&1F)
If Port_ID = Kempston.IO Then
        inb = Kempston.Stat
        Kempston.Stat = 0
        Exit Function
End If



'// Keyboard read Info. (FEh Port)
'     Input from Port FEh (or any other port with A0 zero)
'     Reading from this port also initiates the Vertical Retrace period,
'     and resets the LINECNTR register to zero, LINECNTR remains stopped/zero until
'     user terminates retrace - In the ZX81, all of the above happens only if NMIs are disabled.
'
'  Bit  Expl.
'  0-4  Keyboard column bits (0=Pressed)
'  5    Not used             (1)
'  6    Display Refresh Rate (0=60Hz, 1=50Hz)
'  7      Cassette Input(0 = Normal, 1 = Pulse)
'
 If (port And &HFF) = &HFE Then
     '  bInputWait = False
 End If
 
' // Input port A0 =1 and PORT=FEh

 If (port And 1) = 1 Then
    
        res = &HFF
        port = glMemAddrDiv256(port And &HFF00&)
        
        GoTo fast_scan2
        
        If (port And 1) = 0 Then Debug.Print " Kb column 0: Pressed"
        If (port And 2) = 0 Then Debug.Print " Kb column 1: Pressed"
        If (port And 4) = 0 Then Debug.Print " Kb column 2: Pressed"
        If (port And 8) = 0 Then Debug.Print " Kb column 3: Pressed"
        If (port And 16) = 0 Then Debug.Print " Kb column 4: Pressed"
        If (port And 32) = 0 Then Debug.Print " Not Used : 1"
        If (port And 64) = 0 Then Debug.Print " Refresh Rate: 60Hz" Else Debug.Print " Refresh Rate: 50Hz"
        If (port And 128) = 1 Then Debug.Print " Tape Signal:Pulse" Else Debug.Print " Tape Signal: normal"
        
fast_scan2:
        
        If Not bEAR_input Then res = res Or &H128
        res = res And &HFF
        inb = res
        Exit Function
End If


'When reading from the keyboard, one of the upper bits (A8-A15) of the I/O address must be "0" to select
' the desired keyboard row (0-7).
'(When using IN A,(nn), the old value of the A register is output as upper address bits and <nn>
' as lower bits. Otherwise, ie. when using IN r,(C) or INI or IND, the BC register is output
' to the address bus.)

' // Input port A0 =0 and PORT=FEh
If (port And 1) = 0 Then
        res = &HFF
        port = glMemAddrDiv256(port And &HFF00&)
        If (port And 1) = 0 Then res = res And keyCAPS_V
        If (port And 2) = 0 Then res = res And keyA_G
        If (port And 4) = 0 Then res = res And keyQ_T
        If (port And 8) = 0 Then res = res And key1_5
        If (port And 16) = 0 Then res = res And key6_0
        If (port And 32) = 0 Then res = res And keyY_P
        If (port And 64) = 0 Then res = res And keyH_ENT
        If (port And 128) = 0 Then res = res And keyB_SPC
        ' // Bit7 of the port FE is always 0 on the zx81 (or so it appears)
        bAutoType = False
        res = res And 127
        inb = res
End If

End Function

Sub outb(port As Long, outbyte As Long, Optional tstates As Long = 0)

' IO port list:
'-----------------------------------------------------------
'0  :\ Reserved.
'1  :/ Internal vb81 hack.
'2  : AGB Sound card.
'0Fh/15  : VTR/ZON X SOUND CARD-Datas. ('// 0Fh - Zon-X sound Card.)
'2Fh/3Fh : Memotech Centronics IF Card.(Out:2Fh/In:3Fh)
'3Fh/63 : Fuller Master Box SOUND CARD(Command Register).
'3Fh/63 : VTR SPEECH ENGINE CARD ( Data ).
'41h : LARKEN DISK DRIVE CONTROLER-Datas
'43h : LARKEN DISK DRIVE CONTROLER-NC
'45h : LARKEN DISK DRIVE CONTROLER-Commands
'57h : LARKEN DISK DRIVE CONTROLER-Card control (Drive0/Drive1)
'5Fh : Fuller Master Box SOUND CARD(Datas).
'73h : D8251 for TS2050 Sound Card.
'77h : D8251 for TS2050 Sound Card (write countrol)
'C7h : D8255 Sound Card (Port A)
'CFh : D8255 Sound Card (Port B)
'D7h : D8255 Sound Card (Port C)
'DDh : Ace Jupiter sound card.
'DFh/223 : VTR/ZON X SOUND CARD-Command Register (Jupiter Ace Sound Card & Zon-X sound Card.)
'FBh/251 : ZX Printer Port.
'FDh/253 : ZX81 display (Fast/Slow mode)
'FEh/254 : NMI generator output.
'FFh/255 : TV synchr./ Tape ouput. ( ULA - PIN16)
'-----------------------------------------------------------

 Dim SPOtime As Integer
 Dim MyPRTchar As String
            
'Debug.Print "Output port : "; Hex(port), Hex(outbyte)
 
 ' sound card Test.
 If S_Card = esNo_Card Then
 Select Case port And 255
 'VTR-ZONx
 Case 15, 223
 FrmMainWnd!Sound_detect.Visible = True
 FrmMainWnd!Snd_Cap1.Caption = "VTR": FrmMainWnd!Snd_Cap2.Caption = "ZON-x": FrmMainWnd!Sound_infos.Enabled = True
 Case 2
 FrmMainWnd!Sound_detect.Visible = True
 FrmMainWnd!Snd_Cap1.Caption = "": FrmMainWnd!Snd_Cap2.Caption = "AGB": FrmMainWnd!Sound_infos.Enabled = True
 Case 63
 FrmMainWnd!Sound_detect.Visible = True
 FrmMainWnd!Snd_Cap1.Caption = "VTR": FrmMainWnd!Snd_Cap2.Caption = "SPO256": FrmMainWnd!Sound_infos.Enabled = True
 End Select
 End If

' To avoid "Pause" Blinking this sub is ignored.
  If (MEM(16443) And 64) = 64 And port = &HFF And outbyte = 0 Then

       regAF_ = regAF_ Or 32768
       MEM(16443) = MEM(16443) And &HC0 '(BF+128)
       If peekw(16443) = 0 Then
       Debug.Print regHL
       End If
       bInputWait = True
       ShowDisplay True, True
    ' Exit Sub
   End If
     
   
   If port = &HFF And outbyte = 0 And MEM(16443) = 0 Then
      regAF_ = regAF_ Or 32768
      MEM(16443) = MEM(16443) And &HC0 '(BF+128)
      bInputWait = True
      ShowDisplay True, True
     'Exit Sub
   End If
   
' SOUND>>>----------------------------------------------------------------------
 
 '// AGB SOUND CARD ( Data ).
 '   ------------------------
 If S_Card = esAGB Then
     ' >>> REGISTERS
     If (port And &HFF) = 0 Then
        glSoundRegister = outbyte And &HF
        
      '>>> DATAS
     ElseIf (port And &HFF) = 2 Then
        AYWriteReg glSoundRegister, outbyte
        GoTo Dips_infos
     End If
 End If
 
' // Fuller Master Box SOUND CARD.
' // Port 3Fh : Command Register.
' // Port 5Fh : Datas.
 '   ------------------------
  If S_Card = esFuller Then
     ' >>> REGISTERS
     If (port And &HFF) = 63 Then
        glSoundRegister = outbyte And &HF
        
      '>>> DATAS
     ElseIf (port And &HFF) = 95 Then
        AYWriteReg glSoundRegister, outbyte
        GoTo Dips_infos
     End If
 End If
 
  '// User SOUND CARD.
 '   ------------------------
 If S_Card = esUSER Then
     ' >>> REGISTERS
     If (port And &HFF) = AY_User.CommandReg Then
        glSoundRegister = outbyte And &HF
        
      '>>> DATAS
     ElseIf (port And &HFF) = AY_User.DatasReg Then
        AYWriteReg glSoundRegister, outbyte
        GoTo Dips_infos
     End If
 End If
 
  '   ------------------------
 '// 3Fh/63 - VTR SPEECH ENGINE CARD ( Data ).
 '// NOT CODED.
  If S_Card = esSPO256 Or esParrotSPO256 Then
If (port And &HFF) = &H3F Or (port And &HFF) = &H4F Then
    If outbyte And 128 Then
    ' Reset the SPO256-AL2 CHIPSET.
        Dim ret As Variant
        ret = sndPlaySound(0&, 0&)
        FrmMainWnd.SPOTimer.Enabled = False
        bSPO = False
    Else
     Debug.Print "VTR/Parrot SPEECH ENGINE CARD (63/79): "; Hex(outbyte)
If Not bSPO Then
    ' Init BUSY timer, and play Allophone.
    Select Case (outbyte And 63)
        Case 0
        SPOtime = 10
        Case 1
        SPOtime = 30
        Case 61
        SPOtime = 40
        Case 2, 63
        SPOtime = 50
        Case 7, 12, 15, 21
        SPOtime = 70
        Case 28, 36
        SPOtime = 80
        Case 55
        SPOtime = 90
        Case 3, 17, 22, 23, 24, 30
        SPOtime = 100
        Case 45
        SPOtime = 110
        Case 8, 26, 39
        SPOtime = 120
        Case 27, 49
        SPOtime = 130
        Case 10, 11, 13, 34
        SPOtime = 140
        Case 40
        SPOtime = 150
        Case 33, 37, 42, 51
        SPOtime = 160
        Case 14
        SPOtime = 170
        Case 16, 25, 29, 46, 57
        SPOtime = 180
        Case 35, 38, 41, 50, 56, 62
        SPOtime = 190
        Case 4, 48
        SPOtime = 200
        Case 9, 43
        SPOtime = 210
        Case 44
        SPOtime = 220
        Case 53, 54
        SPOtime = 240
        Case 19
        SPOtime = 250
        Case 6, 31
        SPOtime = 260
        Case 20
        SPOtime = 280
        Case 18, 59
        SPOtime = 290
        Case 52
        SPOtime = 300
        Case 58
        SPOtime = 330
        Case 60
        SPOtime = 350
        Case 47
        SPOtime = 360
        Case 32
        SPOtime = 370
        Case 5
        SPOtime = 420
    End Select
    
     FrmMainWnd.SPOTimer.Interval = SPOtime
     
     BeginPlaySound outbyte And 63, True
     End If
     Exit Sub
     End If
 End If
'   ------------------------
End If
 
' // VTR/ZON X SOUND CARD.
' // Port DFh : Command Register (// DFh - Jupiter Ace Sound Card & Zon-X sound Card.)
' // Port 0Fh : Datas. ('// 0Fh - Zon-X sound Card.)
'
If S_Card = esVtr Then

' // ZON-X / VTR
'   ------------------------
If (port And &HFF) = &HDF Then
'// Command Register.
'// -----------------
'// Reg. #0 : A Chanel. Hight resolution step. (0-255)
'// Reg. #1 : A Chanel. Low   resolution step. (0-255)x255
'// Reg. #2 : B Chanel. Hight resolution step. (0-255)
'// Reg. #3 : B Chanel. Low   resolution step. (0-255)x255
'// Reg. #4 : C Chanel. Hight resolution step. (0-255)
'// Reg. #5 : C Chanel. Low   resolution step. (0-255)x255
'// Reg. #6 : A,B,C Chanels. 0 loodness - 31 Stricky! (enveloppe)
'// Reg. #7 : chanel trigger. A(8-1) B(16-2) C(32-4)
'// Reg. #8 : A Chanel Volume.(0-15) >16 Set by #13 register
'// Reg. #9 : B Chanel Volume.(0-15) >16 Set by #13 register
'// Reg. #10: C Chanel Volume.(0-15) >16 Set by #13 register
'// Reg. #11: #13 register Speed. Hight resolution step. (0-255)
'// Reg. #12: #13 register Speed. Low   resolution step. (0-255)x255
'// Reg. #13: Parametric sound volume.

 glSoundRegister = outbyte And &HF
 

ElseIf (port And &HFF) = &HF Then

'// Datas.
AYWriteReg glSoundRegister, outbyte

Dips_infos:
If FrmSndKitDebugger.SND_OUT.ListItems.Count > 1000 Or Not bSND_Display Then Exit Sub
'Load FrmSndKitDebugger.
If Not FrmSndKitDebugger.Visible Then FrmSndKitDebugger.Visible = True

If FrmSndKitDebugger.SND_OUT.ListItems.Count = 1000 Then FrmSndKitDebugger.SND_OUT.ListItems.Clear
FrmSndKitDebugger.SND_OUT.ListItems.Add CLng(FrmSndKitDebugger.SND_OUT.ListItems.Count + 1), , "Register" + IIf(glSoundRegister < 10, "_", "") + Str(glSoundRegister) + "=" + IIf(outbyte < 100, IIf(outbyte < 10, "  ", " "), "") + Str(outbyte) + " / h" + IIf(outbyte < 16, " ", "") + Hex(outbyte)
FrmSndKitDebugger.SND_OUT.ListItems.Item(FrmSndKitDebugger.SND_OUT.ListItems.Count).EnsureVisible
Exit Sub
End If
End If
' SOUND<<<----------------------------------------------------------------------

' // LARKEN DISK DRIVE CONTROLER.
' // Port 41h : Datas
' // Port 43h : NC
' // Port 45h : Commands
' // Port 57h : Card control (Drive0/Drive1)

'// 41h - Larken Disk Drive
   If (port And &HFF) = &H41 Then
   Debug.Print "Larken ouput (41h):"; outbyte
'   format=      3     &     14     &     20
'           "00000011" & "00001110" & "00001110"

     
'// 45h - Larken Disk Drive
   ElseIf (port And &HFF) = &H45 Then
   
'   format= 18 (step+Wr) & 8 () & A (+seldir) &   20 (Side_Sel+seldir)
'           "000010010"  &  "00001000"  &  "00001010"         &  "00010100"
'
'   format= 9 "00001001" (  + Stepper pulse )
 
  '// Bit 0 : Stepper pulse.
  '// Bit 1 : Stepper direction(Set to "0" for dirve 1 and "1" for Drive 2).
  '// Bit 2 : Side select.
  '// Bit 3 : Motor on.
  '// Bit 4 : Write anabled.
  
  If (outbyte And 16) = 16 Then Debug.Print "Write_Enable = 1" ' Write_Enable
  If (outbyte And 8) = 8 Then Debug.Print " = 1" '
  If (outbyte And 4) = 4 Then Debug.Print "Side_Sel = 1" ' Side_Sel
  If (outbyte And 2) = 2 Then Debug.Print "Drive 2 connected." Else Debug.Print "Drive 1 connected." ' Stepper_Dir
  If (outbyte And 1) = 1 Then Debug.Print "Stepper_Pulse = 1" ' Stepper_Pulse

'// 57h - Larken Disk Drive (87)
    ElseIf (port And &HFF) = &H57 Then
    Debug.Print "Larken ouput (57h):"; outbyte
    
    If outbyte = 0 Then
        If Disk_Stat(0) Then
         Active_Drive0
        Else
         MsgBox "Drive0 not installed! ", vbCritical, "I/O error."
        End If
    End If
    
    If outbyte = 2 Then
         If Disk_Stat(0) Then
          Active_Drive1
         Else
          MsgBox "Drive1 not installed! ", vbCritical, "I/O error."
         End If
    End If


   ElseIf (port And &HFF) = &H69 Then
'// 69h - Larken Disk Drive

    Debug.Print "Larken ouput (69h):"; outbyte

'Port FDh Write (ZX81 only)
'Writing any data to this port disables the NMI generator.


'// PORT FDh
'/////////////////////////////////////////////////////////////////////////////
    ElseIf port = &HFD Then
    
        ' SLOW Mode
        regAF_ = regAF_ Or 32768
        

'// PORT FEh - NMI generator output.
'/////////////////////////////////////////////////////////////////////////////

    ElseIf port = &HFE Then
  
        ' FAST Mode
        regAF_ = regAF_ And (Not 32768)
        
End If ' Else if end

'// PORT 0
'/////////////////////////////////////////////////////////////////////////////
If port = 0 Then
        'Rom Hack feed back : "InstallROMHacks" in sub ModMain.
        ' Rom is waiting for keyboard input!
        ' This is an obselete Hack, but it's used to fill the memory
        ' while Auto-launch feature! "vb81 example.p" in Windows command line.
        bInputWait = True
        bBooting = False
        ShowDisplay True, True
        
        ' Shift+6 and Shift+7 in Basic Editor.
        ' Release Rom untile our NMI emulation is sets bit 0 of CDFLAGS (16443/403B)
        ' The Hack is now waiting for a change in HL address...
        ' We had to force the keyboard refresh...
        ' and refresh Debounce variable.
                
        If MEM(&H4027) = 255 Then MEM(&H4027) = outbyte
        

      'Don't allow to select a file while boot!
      If Not FrmMainWnd.mnuFile(1).Enabled Then FrmMainWnd.mnuFile(1).Enabled = True
      
' AUTOSTART
' vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
' Direct files laucher...

If AutoStart Then OpenCommand

' End of AUTO-START
' ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

End If ' port = 0


'// PORT 1
'/////////////////////////////////////////////////////////////////////////////
If port = 1 Then
        'Rom Hack feed back : "InstallROMHacks" in sub ModMain.
        ' Rom asck to refresh the screen in fast mode!
        bInputWait = False
        bBooting = False
        If MEM(16443) And 128 Then
        ShowDisplay True, True
        Else
        ShowDisplay False, False
        End If

        'Exit Sub

  
    
'// PORT 73h - D8251 for TS2050 Sound Card
'/////////////////////////////////////////////////////////////////////////////
    ElseIf port = &H73 Then Debug.Print " D8251 for TS2050 Sound Card "
    Exit Sub
    
    
'// PORT 77h - D8251 for TS2050 Sound Card (write countrol)
'/////////////////////////////////////////////////////////////////////////////
    ElseIf port = &H77 Then Debug.Print " D8251 for TS2050 Sound Card "
    Exit Sub
    
    
'// PORT C7h - D8255 Sound Card (Port A)
'/////////////////////////////////////////////////////////////////////////////
    ElseIf port = &HC7 Then Debug.Print " D8255 Sound Card - Port A"
    Exit Sub
    
    
'// PORT CFh - D8255 Sound Card (Port B)
'/////////////////////////////////////////////////////////////////////////////
    ElseIf port = &HCF Then Debug.Print " D8255 Sound Card - Port B"
    Exit Sub
    
'// PORT D7h - D8255 Sound Card (Port C)
'/////////////////////////////////////////////////////////////////////////////
    ElseIf port = &HD7 Then Debug.Print " D8255 Sound Card - Port C"
    Exit Sub
    
'// PORT DDh - Ace Jupiter sound card.
'/////////////////////////////////////////////////////////////////////////////
    ElseIf port = &HDD Then Debug.Print " Jupiter ACE Sound Card "
    Exit Sub
    
    
' ///////////////////////////////////////////////////////////
' /////// CENTRONICS PRINTERS MEMOTECH IF / MAGECO //////////
' ///////////////////////////////////////////////////////////


     '   ElseIf port = &H2F And PrinterCfg = 1 Then
     '// 2Fh - Memotech Centronics IF.
     '//
   
     ' Usually used on the IN IORQ Port.
     

    ElseIf port = &HFB And PrinterCfg = 2 Then
     '// BFh - AGB Centronics IF.
     '//
          Static fH2 As Integer
         On Error GoTo Nxtstep2
            fH2 = FreeFile
            Open App.path & "\Printer_out\Printer_Cen.txt" For Binary As fH2
            Put #fH2, LOF(fH2) + 1, CByte(outbyte)
            Close #fH2
Nxtstep2:
            gCentronic = False
            reset
            If Frm_MemotechCIF.Visible Then Frm_MemotechCIF.Display_Centro_Datas CByte(outbyte)

            Select Case outbyte
            Case &HA
            MyPRTchar = vbCrLf
            Case 0
            MyPRTchar = ""
            Case Is < 32
            MyPRTchar = "[" & outbyte & "]"
            Case Else
            MyPRTchar = Chr(outbyte)
            End Select
            
            FrmPrinter.ASCII_PRT.text = FrmPrinter.ASCII_PRT.text + MyPRTchar
            FrmPrinter.ASCII_PRT.SelStart = Len(FrmPrinter.ASCII_PRT)
   

        Exit Sub
    
'// PORT FBh - ZX Printer Port.
'/////////////////////////////////////////////////////////////////////////////
    ElseIf port = &HFB And PrinterCfg = 0 Then
'
' write
' D0
' D1   speed:     0=fast    1=slow
' D2   motor:     0=run      1=stop
' D3
' D4
' D5
' D6
' D7   stylust:  0=current off   1=burn pixel
    
    
    ' Printer function:
    ' B01: High level: Slow mode. (this line is overridden by D2 being high)
    ' B02: High level: means stop the motor.
    ' B07: High level: applies power to the stylus (print a dot).
      
GoTo Fast_Scan
  If (outbyte And 128) = 128 Then Debug.Print "Ploted(1)"; Else Debug.Print "Ploted(0)";
  If (outbyte And 64) = 64 Then Debug.Print "1"; Else Debug.Print "0";
  If (outbyte And 32) = 32 Then Debug.Print "1"; Else Debug.Print "0";
  If (outbyte And 16) = 16 Then Debug.Print "1"; Else Debug.Print "0";
  If (outbyte And 8) = 8 Then Debug.Print "1"; Else Debug.Print "0";
  If (outbyte And 4) = 4 Then Debug.Print "1"; Else Debug.Print "0";
  If (outbyte And 2) = 2 Then Debug.Print "1"; Else Debug.Print "0";
  If outbyte <> 0 And outbyte <> &H80 Then
  Debug.Print
  
  End If
  
Fast_Scan:

  If (outbyte And 2) Then cPrinter.SPEED (True) Else cPrinter.SPEED (False)

  ' Motor printer ON/OFF.
  If (outbyte And 4) And Not cPrinter.Get_Motor Then
  'Refresh_Leds 101
  Stylus_X = -1
  cPrinter.Motor_On True
  'Exit Sub:
  Else
  cPrinter.Motor_On False
  End If
  '
  If (outbyte And 1) And (outbyte And 4) Then Stylus_X = Stylus_X + 1: Exit Sub 'Stylus_X = Stylus_X + 1 : Exit Sub
  
  'Motor On / Motor Off
  If (outbyte And 2) And (outbyte And 4) Then Exit Sub

 
  ' Plot a dot.
  cPrinter.PlotToPrinter outbyte And 128
  
  Exit Sub
  
'// PORT BFh
'/////////////////////////////////////////////////////////////////////////////
  
  ' CZ1500 - CBM1541 disk interface
'
' +---+---+---+---+---+---+---+---+
'| 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |   Port 0BFH
'+---+---+---+---+---+---+---+---+
'  |   |   |   |   |   |   |   |
'  |   |   |   |   |   |   |   +--> R/O Busy status line from EEPROM active low
'  |   |   |   |   |   |   +------> W/O High ram bank select
'  |   |   |   |   |   +----------> W/O Serial reset                 active high
'  |   |   |   |   +--------------> W/O Serial attention output      active high
'  |   |   |   +------------------> W/O Serial clock output          active high
'  |   |   +----------------------> W/O Serial data  output          active high
'  |   +-------------------------->
'  +------------------------------>

'Bits 2-7 are used in the same way as in the Commodore 64.
'Bit 1 selects the 16k ram bank at address 8000H-BFFFH 0=user 1=system.
'Bit 0 returns the state of the EEPROM busy line, used during EEPROM programming
'    0=memory busy 1=ready to accept data.
   ElseIf port = &HBF Then
   Debug.Print "CBM1541:";
    If (outbyte And 128) = 128 Then Debug.Print "1"; Else Debug.Print "0";
  If (outbyte And 64) = 64 Then Debug.Print "1"; Else Debug.Print "0";
  If (outbyte And 32) = 32 Then Debug.Print "1"; Else Debug.Print "0";
  If (outbyte And 16) = 16 Then Debug.Print "1"; Else Debug.Print "0";
  If (outbyte And 8) = 8 Then Debug.Print "1"; Else Debug.Print "0";
  If (outbyte And 4) = 4 Then Debug.Print "1"; Else Debug.Print "0";
  If (outbyte And 2) = 2 Then Debug.Print "1"; Else Debug.Print "0";
  If (outbyte And 2) = 1 Then Debug.Print "1"; Else Debug.Print "0"

If outbyte = 2 Then Debug.Print "RESET"
If outbyte = 10 Then Debug.Print "DATA RQ"
If outbyte = 26 Then Debug.Print "CALL DATA RQ"

'// PORT FFh - TV synchr./ Tape ouput. ( ULA - PIN16)
'/////////////////////////////////////////////////////////////////////////////
    ElseIf port = &HFF Then   'And (outbyte And &H80)

   If (MEM(16443) And 64) = 64 Then '/ TV synchr. ( ULA - PIN16)
          ' FAST Mode
        regAF_ = regAF_ And (Not 32768)
        bInputWait = False
        bBooting = False
        ShowDisplay True, True
  End If
  
   If (MEM(16443) And 128) = 0 Then '/ Tape ouput. ( ULA - PIN16)
        bInputWait = False
        bBooting = False
        If (outbyte And 16&) Then
            glBeeperVal = 200&
        Else
            glBeeperVal = 128&
        End If

      Select Case (outbyte)
        Case 4, 9, 255
             Wav_Top = Wav_Top + 1
            '// Save "4" or "9" Datas in Tape Buffer.
             ReDim Preserve Wav_Buffer(0 To Wav_Top)
             Wav_Buffer(Wav_Top - 1) = CByte(outbyte)
 
        Case 15
         Debug.Print "End of File ."; Hex(MEM(&H4035))
       
        Exit Sub
         End Select
           
          '// Send byte to Sound monitor.
           FrmMainWnd.Put_Pitch (CByte(outbyte))
        Exit Sub
    

    End If
End If ' Global ElseIf End
 
'Output to Port FFh (or ANY other port)
'Writing any data to any port terminates the Vertical Retrace period, and restarts the LINECNTR counter.
'End If

End Sub


Public Sub ReInitHiresScreen()
    Dim y As Long, x As Long
    
    For y = 0 To 191
        For x = 0 To 31
            cLastHiResScreen(y, x) = 0
            gcBufferBits(y * 32 + x) = 0
        Next x
    Next y
End Sub


Sub ScreenPaint(Optional Brefresh As Boolean = False)
    Dim ptr As Long, x As Long, y As Long
    Dim c As Long
    Dim lChar As Long, lRow As Long
    
    Dim lTopMost As Long, lBottomMost As Long, lLeftMost As Long, lRightMost As Long
    Dim bUpdated As Boolean
    
    Dim Get_width As Long ' Implemented Value to allow 34 columnes features.
    Dim Get_height As Long ' Implemented Value to allow 24 lines features.
    Dim LocAdd As Long
    Dim LocByte As Byte
    Dim x2 As Long, y2 As Long
    
    lTopMost = 191
    lBottomMost = 0
    lLeftMost = 31
    lRightMost = 0
    
    ' //ZX-tricator
    If intHRG <> 0 Then
     ptr = intHRG
    
    lTopMost = 191
    lBottomMost = 0
    lLeftMost = 0
    
    lRightMost = 272 ' 247 old buffer size.
    
    ZxTric_refresh ptr
    gpicDisplay.Visible = False
    StretchDIBits gpicDisplay.hDC, 0, 0, (lRightMost + 1) * glDisplayXMultiplier, (lTopMost + 1) * glDisplayYMultiplier, 0, lTopMost + 2, lRightMost + 9, -lTopMost - 1, gcBufferBits(0), bmiBuffer, DIB_RGB_COLORS, SRCCOPY
    gpicDisplay.Visible = True
    
    Exit Sub
    End If
    
' ********************    G R A P H I C S     C A R D S    *********************

     ' // Load True graphic card Eprom.
   Select Case G_Card
   Case 0, 5, 6
   ' No card.
    ptr = peekw(16396) + 1 ' // D_FILE (+1 to skip the initial HALT)

   Case 1
    ' HRG Memotech
       If Mem_Display = 0 Or intI = &H1E Then  ' intI <> &H1E
       ptr = peekw(16396) + 1 ' // D_FILE (+1 to skip the initial HALT)
       Else
       HrgMemo_PageOff = peekw(&H407B)  ' GET V value of new hight resolution d_file buffered in &h407B.
       ptr = HrgMemo_PageOff
       
       lTopMost = 191
       lBottomMost = 0
       lLeftMost = 0
       lRightMost = 247  'old buffer size.
     
       'lLeftMost = 0
       'lRightMost = 248 * 8
    
       Memo_refresh ptr
       If Mem_Display = 3 Then InvertScreen
       gpicDisplay.Visible = False
       StretchDIBits gpicDisplay.hDC, 0, 0, (lRightMost + 1) * glDisplayXMultiplier, (lTopMost + 1) * glDisplayYMultiplier, 0, lTopMost + 2, lRightMost + 9, -lTopMost - 1, gcBufferBits(0), bmiBufferHR, DIB_RGB_COLORS, SRCCOPY
       gpicDisplay.Visible = True
       
       Exit Sub
       End If
    
      Case 2
    ' HRG G007
        If G007_Display = 0 Or intI = &H1E Then
         ptr = peekw(16396) + 1 ' // D_FILE (+1 to skip the initial HALT)
           Else
        lTopMost = 192
        lBottomMost = 0
        lLeftMost = 0
        lRightMost = 248  ' old buffer size.
     
        lLeftMost = lLeftMost * 8
        lRightMost = lRightMost * 8
         
        ptr = peekw(&H2308)  ' // D_FILE (+1 to skip the initial HALT)
         
        G007_refresh ptr
    
        gpicDisplay.Visible = False
        StretchDIBits gpicDisplay.hDC, lLeftMost * glDisplayXMultiplier, (lBottomMost) * glDisplayYMultiplier - 1, (lRightMost - lLeftMost + 8) * glDisplayXMultiplier, -(lBottomMost - lTopMost + 1) * glDisplayYMultiplier, lLeftMost, lTopMost, (lRightMost - lLeftMost) + 8, lBottomMost - lTopMost + 1, gcBufferBits(0), bmiBufferHR, DIB_RGB_COLORS, SRCCOPY
        gpicDisplay.Visible = True
            
        Exit Sub
        End If
    
    Case 3
    ' HRG QSHRG
       If QSHRG_Display = 0 Then
        ptr = peekw(16396) + 1 ' // D_FILE (+1 to skip the initial HALT)
          Else
       lTopMost = 192
       lBottomMost = 0
       lLeftMost = 0
       lRightMost = 249 * 8
    
       CopyMemory gcBufferBits(0), MEM(40960), 6913
       gpicDisplay.Visible = False
       StretchDIBits gpicDisplay.hDC, lLeftMost * glDisplayXMultiplier, (lBottomMost) * glDisplayYMultiplier, (lRightMost - lLeftMost) * glDisplayXMultiplier, -(lBottomMost - lTopMost) * glDisplayYMultiplier, lLeftMost, lTopMost + 1, (lRightMost - lLeftMost), lBottomMost - lTopMost, gcBufferBits(0), bmiBufferHR, DIB_RGB_COLORS, SRCCOPY
       gpicDisplay.Visible = True
           
       Exit Sub
       End If
    
' REMMED 2/2009  - Wrx card deleted and assumed like a normal ZX81 feature.
' Intecepted Whith IR,IX and IM machine codes instructions. (modZ80)
' Case 4
    ' HRG WRX
'    If Display_WRX = 0 Then
'            If G_Card = eHiResWRX Then
'            ptr = peekw(16396) + 1 ' // D_FILE (+1 to skip the initial HALT)
'            Else
'            HiresScreenPaint
'            End If
'       Else
     
'     HiresScreenPaint
'     Exit Sub
     
'    lTopMost = 192
'    lBottomMost = 0
'    lLeftMost = 0
'    lRightMost = 249 * 8
  
     
    'ptr = peekw(&H2308)  ' // D_FILE (+1 to skip the initial HALT)
     
   ' WRX_refresh ' ptr

'    gpicDisplay.Visible = False
'    StretchDIBits gpicDisplay.hdc, lLeftMost * glDisplayXMultiplier, (lBottomMost) * glDisplayYMultiplier, (lRightMost - lLeftMost) * glDisplayXMultiplier, -(lBottomMost - lTopMost) * glDisplayYMultiplier, lLeftMost, lTopMost + 1, (lRightMost - lLeftMost), lBottomMost - lTopMost + 1, gcBufferBits(0), bmiBufferHR, DIB_RGB_COLORS, SRCCOPY
'    gpicDisplay.Visible = True
        
'    Exit Sub
'    End If

   End Select
 
 ' ********************    G R A P H I C S     C A R D S    *********************
    


    On Error Resume Next
    
    ' // don't bother if D_FILE is uninitialised
    If gpicDisplay Is Nothing Or ptr = 0 Then Exit Sub

    
    'S_POSN  &H4039 : X-Coordinate for PRINT max 34
    '        &H403A : Y-Coordinate for PRINT max 24

    Get_width = 34
    Get_height = 24
    
 
    lTopMost = 191
    lBottomMost = 0
    lLeftMost = Get_width - 1
    lRightMost = 0
    
    Dim bEOL As Boolean ' End of line reached blank char after a 76$ char.
   'If ptr > &HFFFF Then Exit Sub
   On Error Resume Next
    ' // Build the video memory map in sScreen
    For y = 0 To Get_height - 1 '23
   
        For x = 0 To Get_width - 1 '31-32-33
        lChar = y * Get_width + x
            
            If MEM(ptr) = &H76& Or MEM(ptr) = &HC9& Then
                bEOL = True
                
                   Else
                c = (MEM(ptr) And &HBF&) ' strip bit 6
            End If
            
            If bEOL Then c = 0  ' // space
            
            If c <> sLastScreen(lChar) Or bForceDisplay Or Brefresh Then
                ' // If this character has changed since we last painted
                ' // the screen, repaint it
                If y * 8 < lTopMost Then lTopMost = y * 8
                If y * 8 + 7 > lBottomMost Then lBottomMost = y * 8 + 7
                If x < lLeftMost Then lLeftMost = x
                If x > lRightMost Then lRightMost = x
                
                bUpdated = True
                
                If G_Card = eCOLOUR16 Or G_Card = ePENTRON16 Then
                        x2 = x * 8
                        y2 = y * 8
                        
                End If
                
                If (c And 128) Then
                On Error Resume Next
                ' Video reversed mode
                    For lRow = 0 To 7
                    If bAGB_ON Then gcBufferBits((y * (Get_width + 2) * 8) + x + (lRow * (Get_width + 2))) = MemUDG(((c And 63) * 8) + lRow) Xor 255: GoTo AGBLoop
                    
                    If bQS_ON Then
                        gcBufferBits((y * (Get_width + 2) * 8) + x + (lRow * (Get_width + 2))) = MemUDG(((c And 63) * 8) + lRow + 512) Xor 255
                    Else
                       If G_Card = eCOLOUR16 Or G_Card = ePENTRON16 Then
                        ' Update COLOR card buffer.

                        LocByte = MEM((intI * 256) + ((c And 63) * 8) + lRow) Xor 255
Debug.Print LastColor
                        ' Mageco 16 couleurs. debugtag
                        If LocByte And 128 Then glBufferBits16(x2, y2 + lRow) = 1 Else glBufferBits16(x2, y2 + lRow) = LastColor
                        If LocByte And 64 Then glBufferBits16(x2 + 1, y2 + lRow) = 1 Else glBufferBits16(x2 + 1, y2 + lRow) = LastColor
                        If LocByte And 32 Then glBufferBits16(x2 + 2, y2 + lRow) = 1 Else glBufferBits16(x2 + 2, y2 + lRow) = LastColor
                        If LocByte And 16 Then glBufferBits16(x2 + 3, y2 + lRow) = 1 Else glBufferBits16(x2 + 3, y2 + lRow) = LastColor
                        If LocByte And 8 Then glBufferBits16(x2 + 4, y2 + lRow) = 1 Else glBufferBits16(x2 + 4, y2 + lRow) = LastColor
                        If LocByte And 4 Then glBufferBits16(x2 + 5, y2 + lRow) = 1 Else glBufferBits16(x2 + 5, y2 + lRow) = LastColor
                        If LocByte And 2 Then glBufferBits16(x2 + 6, y2 + lRow) = 1 Else glBufferBits16(x2 + 6, y2 + lRow) = LastColor
                        If LocByte And 1 Then glBufferBits16(x2 + 7, y2 + lRow) = 1 Else glBufferBits16(x2 + 7, y2 + lRow) = LastColor
                        'If c > 127 Then c = 128
                        
                       Else
                       ' Update binary buffer.
                       gcBufferBits((y * (Get_width + 2) * 8) + x + (lRow * (Get_width + 2))) = MEM((intI * 256) + ((c And 63) * 8) + lRow) Xor 255
                       End If
                    End If
AGBLoop:
                    Next lRow
                Else
                                            
' Standard Video mode
                        On Error Resume Next
                    For lRow = 0 To 7
                    
                    If bQS_ON Or bAGB_ON Then
                        gcBufferBits((y * (Get_width + 2) * 8) + x + (lRow * (Get_width + 2))) = MemUDG((c * 8) + lRow)
                    Else
                    
                       If G_Card = eCOLOUR16 Or G_Card = ePENTRON16 Then
                       ' Update COLOR card buffer.
                        LocByte = MEM((intI * 256) + ((c And 63) * 8) + lRow)
                        If (LocByte And 128) Then glBufferBits16(x2, y2 + lRow) = 1 Else glBufferBits16(x2, y2 + lRow) = LastColor
                        If (LocByte And 64) Then glBufferBits16(x2 + 1, y2 + lRow) = 1 Else glBufferBits16(x2 + 1, y2 + lRow) = LastColor
                        If (LocByte And 32) Then glBufferBits16(x2 + 2, y2 + lRow) = 1 Else glBufferBits16(x2 + 2, y2 + lRow) = LastColor
                        If (LocByte And 16) Then glBufferBits16(x2 + 3, y2 + lRow) = 1 Else glBufferBits16(x2 + 3, y2 + lRow) = LastColor
                        If (LocByte And 8) Then glBufferBits16(x2 + 4, y2 + lRow) = 1 Else glBufferBits16(x2 + 4, y2 + lRow) = LastColor
                        If (LocByte And 4) Then glBufferBits16(x2 + 5, y2 + lRow) = 1 Else glBufferBits16(x2 + 5, y2 + lRow) = LastColor
                        If (LocByte And 2) Then glBufferBits16(x2 + 6, y2 + lRow) = 1 Else glBufferBits16(x2 + 6, y2 + lRow) = LastColor
                        If (LocByte And 1) Then glBufferBits16(x2 + 7, y2 + lRow) = 1 Else glBufferBits16(x2 + 7, y2 + lRow) = LastColor
                     
                        
                               Else
                       ' Update binary buffer.
                        gcBufferBits((y * (Get_width + 2) * 8) + x + (lRow * (Get_width + 2))) = MEM((intI * 256) + (c * 8) + lRow)
                       End If
                       
                    End If
                    
                    Next lRow
                End If
                sLastScreen(lChar) = c
                

     End If

  
             If Not bEOL Then ptr = ptr + 1
        Next x
        If Not bEOL Then ptr = ptr - 1
        ptr = ptr + 1
        bEOL = False
    Next y
    
    lLeftMost = lLeftMost * 8
    lRightMost = lRightMost * 8
    
    If bUpdated Or Brefresh Or bGraphTrig Then
        If (G_Card = eCOLOUR16 Or G_Card = ePENTRON16) And bGraphTrig Then
            gpicDisplay.Visible = False
            StretchDIBits256 gpicDisplay.hDC, lLeftMost * glDisplayXMultiplier, (lBottomMost + 1) * glDisplayYMultiplier - 1, (lRightMost - lLeftMost + 8) * glDisplayXMultiplier, -(lBottomMost - lTopMost + 1) * glDisplayYMultiplier, lLeftMost, lTopMost, (lRightMost - lLeftMost) + 8, lBottomMost - lTopMost + 1, glBufferBits16(0, 0), bmiBuffer16, DIB_RGB_COLORS, SRCCOPY
            gpicDisplay.Visible = True
        Else
            If bInv_Display Then InvertScreen
            gpicDisplay.Visible = False
            StretchDIBits gpicDisplay.hDC, lLeftMost * glDisplayXMultiplier, (lBottomMost + 1) * glDisplayYMultiplier - 1, (lRightMost - lLeftMost + 8) * glDisplayXMultiplier, -(lBottomMost - lTopMost + 1) * glDisplayYMultiplier, lLeftMost, lTopMost, (lRightMost - lLeftMost) + 8, lBottomMost - lTopMost + 1, gcBufferBits(0), bmiBuffer, DIB_RGB_COLORS, SRCCOPY
            gpicDisplay.Visible = True
        End If
        
    End If
        
    bGraphTrig = False
End Sub

Public Function SearchHiresScreen() As Long
    Dim lCounter As Long, v As Byte, FoundHiRes As Boolean
    Dim lCounter2 As Long

      If bBooting And intI = &H1E Then Exit Function
    
    SearchHiresScreen = 0
    Display_WRX = 0
                 
    For lCounter = (32767 - 6336) To 8192 Step -1
        v = MEM(lCounter + 32)
        If (v And 64) <> 0 Then
            FoundHiRes = True
            For lCounter2 = 0 To 191
                If (MEM(lCounter + 33 * lCounter2) And 64) Or (MEM(lCounter + 33 * lCounter2 + 32) <> v) Then
                    FoundHiRes = False
                    Exit For
                End If
            Next lCounter2
        End If
        
        If FoundHiRes Then
            'init Psedo - HRG
            'CreateScreenBuffer_HR
            Display_WRX = 1
            FrmMainWnd.DispHRGoff
            SearchHiresScreen = lCounter
            ClrMemory gcBufferBits(0), 6912
            Exit For
        End If
    Next lCounter
End Function

Public Sub WavLoad(ByVal FileNamePtr As Long, Optional sFileName As String = "")
    Dim Raw_File As Boolean
    Raw_File = False
    
    ' // Keep the frist pointer to retrive empty names.
    Dim FFileNamePtr As Long
    FFileNamePtr = FileNamePtr
    
        ' Load then sFileName file!
        If sFileName <> "" Then
            GoTo Go_Name
        End If
        
        ' // LOAD "filename"
        FileNamePtr = FileNamePtr And 32767
               
        ' // FileNamePtr points to the filename in the ZX81 character set
        If MEM(FileNamePtr) = 227 Then
            End
        End If
        
        Do While MEM(FileNamePtr) < 127
            sFileName = sFileName + ZXCharToASCII(MEM(FileNamePtr))
            FileNamePtr = FileNamePtr + 1
        Loop
        
        sFileName = sFileName & ZXCharToASCII(MEM(FileNamePtr)) & ".wav"
        
        ' Load"" case ... or not found!
        If MEM(FFileNamePtr And 32767) = &HB Then sFileName = "auto_load.wav"
        If Dir(Get_LastPath & "\" & sFileName) = "" Then
        MsgBox "Wav file name not specified or not found." & vbCrLf & Chr(&H22) & sFileName & Chr(&H22), vbCritical, "Bad Wav format!"
 
 Load Dialog_Frame
     With OpenDialog
     .CancelError = True
     .DialogTitle = "Load a ZX81 'WAV' file..."
     .FileName = ""
     .InitDir = Get_LastPath
     .hOwner = Dialog_Frame.hWnd
     .Filter = "All ZX files  (*.wav)|*.wav|Raw sound file.(*.Raw : 22Khz/8bits)|*.raw|Other files (*.*)|*.*"
     .SelDir = Get_LastPath
     .ShowOpen
     End With
 Unload Dialog_Frame
 
  If OpenDialog.FileName = "" Then Exit Sub
        sFileName = OpenDialog.FileName
        End If
        
        If UCase(Right(sFileName, 4)) = ".RAW" Then Raw_File = True
Go_Name:
       
   
Dim Xa As Integer
Dim Ya As Integer
Xa = FrmMainWnd.picDisplay.Left + (FrmMainWnd.picDisplay.width \ 2) - (FrmMainWnd.Tape.width \ 2)
Ya = FrmMainWnd.picDisplay.top + (FrmMainWnd.picDisplay.height \ 2) - (FrmMainWnd.Tape.height \ 2)

FrmMainWnd.Tape.Move Xa, Ya
FrmMainWnd.Tape.Visible = True

If Not Raw_File Then

If Not wHeadInfo(sFileName) Then
MsgBox "The wav's header is corrupt.", vbCritical, "Wav format corrupt!"
Else
FrmSoundIn.Caption = "Input sound: " & OpenDialog.FileName
FrmSoundIn.Mnu_Winfo.Caption = "Info: " & wInfo.Freq & "Hz - " & IIf(wInfo.channels = 1, "mono", "stereo") & " - " & wInfo.Bits & " bits - " & wInfo.wPlaytime & "mn / " & wInfo.kbps & "Kbps."
End If

Else

FrmSoundIn.Caption = "Input sound: " & OpenDialog.FileName
wInfo.Freq = 22050
wInfo.channels = 1
wInfo.Bits = 8
FrmSoundIn.Mnu_Winfo.Caption = "Info: (Raw file)" & wInfo.Freq & "Hz - " & IIf(wInfo.channels = 1, "mono", "stereo") & " - " & wInfo.Bits & " bits."
End If

DoEvents
Get_WAV sFileName
DecodeWav Cap_Line, False, True, False
FrmMainWnd.Tape.Visible = False
End Sub

Public Sub TapeLoad(ByVal FileNamePtr As Long)
    Dim sFileName As String
    Dim RAM_TOP As Long, a As Long
    Dim InfoRam As String
    Dim Bloc() As Blocinfo
    Dim Cnm As String
    Dim result
        
If FileNamePtr >= 32768 Then ' [1]

  ' // LOAD ""
  ' T81/TZX/SBB files :
    If UCase(Right(VT_NameMs, 4)) = ".T81" Or UCase(Right(VT_NameMs, 4)) = ".TZX" Or UCase(Right(VT_NameMs, 4)) = ".SBB" Then ' [2]
    '------------
    If UCase(Right(VT_NameMs, 4)) = ".T81" Then
    Load_T81File VT_NameMs, CLng(VT_Index - 1)
    ZX81name = VT_NameZX(VT_Index - 1).Nom_Fichier
    End If
    
    '------------
    If UCase(Right(VT_NameMs, 4)) = ".SBB" Then
    Load_SBBFile VT_NameMs, CLng(VT_Index - 1)
    ZX81name = VT_NameZX(VT_Index - 1).Nom_Fichier
    End If

    '------------
    If UCase(Right(VT_NameMs, 4)) = ".TZX" Then
    TZXname = VT_NameMs
    
    If Not OpenTZX Then MsgBox "File error", vbCritical ' Open the TZX if True=OK False=File error.
    bLoadBloc = True
    ZX81name = VT_NameZX(VT_Index - 1).Nom_Fichier
    Enum_TZX Bloc()
    Dispose
    End If
    '------------
    Zx_Name = Cut_Name(VT_NameMs) & "-" & ZX81name
    FrmMainWnd.SelectTB (VT_Index)
    FrmMainWnd.Ref_TB (4)
    FrmMainWnd.Caption = "VB81 XuR [" & Zx_Name & "] " & GetMemName

   If FrmDisassembler.Visible Then FrmDisassembler.Mem_Changed
   Exit Sub
    End If ' [2]
  
  ' // LOAD ""
  ' Hard Disk :
  
       
       
       Load Dialog_Frame
       
On Error Resume Next
ChDir (Get_LastPath)

 With OpenDialog
     .CancelError = True
     .DialogTitle = "Load a ZX81 file..."
     .FileName = ""
     .InitDir = Get_LastPath
     .hOwner = Dialog_Frame.hWnd
     .Filter = "All ZX files  (*.P;*.81;*.Z81;*.T81;*.TZX)|*.p;*.81;*.z81;*.T81;*.TZX|Zx81 Tape files (*.P;*.81)|*.p;*.81|Tape files (*.T81;*.TZX)|*.T81;*.TZX|Other files (*.*)|*.*"
     .InitDir = Get_LastPath
     .SelDir = Get_LastPath
     .ShowOpen
 End With
  
 Unload Dialog_Frame
 
  If OpenDialog.FileName = "" Then Exit Sub
   
      SaveSetting "Grok", "vb81", "LastPath", Cut_Path_Name(OpenDialog.FileName)
      
      On Error Resume Next
      ChDir Cut_Path_Name(OpenDialog.FileName)
      
If UCase(Right(OpenDialog.FileName, 4)) = ".T81" Then ' [3]
    
    VT_NameMs = OpenDialog.FileName
    VT_Index = 1
    Enum_T81files VT_NameMs
    FrmMainWnd.SelectTB (VT_Index)
    FrmMainWnd.Ref_TB (4)

    Load_T81File VT_NameMs, 0
    Zx_Name = Cut_Name(VT_NameMs) & "-" & VT_NameZX(0).Nom_Fichier
    FrmMainWnd.Caption = "VB81 XuR [" & Zx_Name & "] " & GetMemName
    On Error Resume Next

    FrmMainWnd.Init_VTape

        bInputWait = True
        ShowDisplay True, False
    Exit Sub
End If ' [3]
   
If UCase(Right(OpenDialog.FileName, 4)) = ".TZX" Then ' [4]
         VT_NameMs = OpenDialog.FileName
         TZXname = VT_NameMs
         VT_Index = 1
         If Not OpenTZX Then MsgBox "File error", vbCritical ' Open the TZX if True=OK False=File error.
         bLoadBloc = True
         Enum_TZX Bloc()
         
  Dim cmp As Long, Vt_Cmp As Long
  
  For cmp = 0 To UBound(Bloc)
    If Bloc(cmp).Tag <> "" Then ' [5]
        ReDim Preserve VT_NameZX(Vt_Cmp)
            VT_NameZX(Vt_Cmp).Nom_Fichier = Bloc(cmp).Tag
            VT_NameZX(Vt_Cmp).Offset = Bloc(cmp).Offset
            VT_NameZX(Vt_Cmp).Taille = Bloc(cmp).Len
            Vt_Cmp = Vt_Cmp + 1
    End If ' [5]
  Next cmp

        ZX81name = VT_NameZX(VT_Index - 1).Nom_Fichier
        TZXname = VT_NameMs
        bLoadBloc = True
        Enum_TZX Bloc()
        FrmMainWnd.Init_VTape
        Dispose
        FrmMainWnd.SelectTB (VT_Index)
        FrmMainWnd.Ref_TB (4)
          
          bInputWait = True
          ShowDisplay True, False
     
          Zx_Name = Cut_Name(VT_NameMs) & "-" & VT_NameZX(0).Nom_Fichier
          FrmMainWnd.Caption = "VB81 XuR [" & Zx_Name & "] " & GetMemName
          Exit Sub
End If ' [4]
   
If UCase(Right(OpenDialog.FileName, 4)) = ".Z81" Then ' [5]
    Snap_CPU OpenDialog.FileName
    Zx_Name = OpenDialog.FileName
    FrmMainWnd.Caption = "VB81 XuR -Memory dump- [" & Cut_Name(Zx_Name) & "] " & GetMemName
    VT_NameMs = ""
   If FrmDisassembler.Visible Then FrmDisassembler.Mem_Changed
   Exit Sub
End If ' [5]

      ' ".P" files
If UCase(Right(OpenDialog.FileName, 2)) = ".P" Or UCase(Right(OpenDialog.FileName, 2)) = "81" Then ' [6]
    VT_Index = 1
    VT_NameMs = OpenDialog.FileName


    RAM_TOP = peekw(16388)
    Load_File_In_Mem OpenDialog.FileName, &H4009

     bInputWait = True
     ShowDisplay True, False
     
     Zx_Name = OpenDialog.FileName
     ReDim VT_NameZX(0)

     Cnm = Cut_Name(VT_NameMs)
     VT_NameZX(0).Nom_Fichier = Left(Cnm, Len(Cnm) - 2)
     FrmMainWnd.Caption = "VB81 XuR [" & Cut_Name(Zx_Name) & "] " & GetMemName
     FrmMainWnd.Init_VTape
     
     If FrmDisassembler.Visible Then FrmDisassembler.Mem_Changed
     
    If (FileLen(OpenDialog.FileName) + &H4009 > RAM_TOP) Then GoTo RamTop_err
     
    Exit Sub
     
 End If ' [6]
    
     Exit Sub
     
Else  '  FileNamePtr < 32768 ' [1]

        ' // LOAD "filename"
        FileNamePtr = FileNamePtr And 32767
        
        
        ' LARKEN :
        ' // if filename in the ZX81 character is set to "=" then launch the LDOS
        If ZXCharToASCII(MEM(FileNamePtr)) = "=" Then
        If bLarken Then regPC = 14336: Exit Sub
        If bMageco Then regPC = 14336: Exit Sub
        Exit Sub
        End If
        
        ' // if filename in the ZX81 character is set to "+" then launch the DIREctory
        If ZXCharToASCII(MEM(FileNamePtr)) = "+" Then
        If bLarken Then regPC = &H3A5B: Exit Sub
        If bMageco Then regPC = &H3A5B: Exit Sub
        Exit Sub
        End If
        
        ' // FileNamePtr points to "STOP" function shift+A then quit VBXur
        If MEM(FileNamePtr) = 227 Then
            End
            ' Exit Sub
        End If
        
        Do While MEM(FileNamePtr) < 127
            sFileName = sFileName + ZXCharToASCII(MEM(FileNamePtr))
            FileNamePtr = FileNamePtr + 1
        Loop
        sFileName = sFileName & ZXCharToASCII(MEM(FileNamePtr))
        
        
        ' Xtender : (To load a binary file)
        ' // if filename in the ZX81 character is set to "*LOAD:C:" then launch the DIREctory
        If Left(sFileName, 1) = "*" Then
        
        If Mid(sFileName, 2, 6) = "LOAD:C" Then
        Dim CodeName As String, MemOffset1 As Long, MemOffset2 As Long
         CodeName = Mid(sFileName, 9, InStr(10, sFileName, ":") - 9) & ".C"
         MemOffset1 = val(InStr(10, sFileName, ":")) + 1
         If Right(sFileName, 1) = ":" Then
         MemOffset2 = val(InStr(MemOffset1, sFileName, ":"))
         Else
         MemOffset2 = Len(sFileName) + 1
         End If
        
         Load_File_In_Mem CodeName, val(Mid(sFileName, MemOffset1, MemOffset2 - MemOffset1))

         RAM_TOP = peekw(16388)
         If (FileLen(CodeName) + &H4009 > RAM_TOP) Then GoTo RamTop_err
          
        End If
        
        Exit Sub
    End If
  
  '*****************************************************************************
  ' Get Virtual tape file.
  '
  ' T81 files :
  If UCase(Right(VT_NameMs, 4)) = ".T81" Or UCase(Right(VT_NameMs, 4)) = ".TZX" Then
    

For a = 0 To UBound(VT_NameZX)
 If VT_NameZX(a).Nom_Fichier = sFileName Then
    VT_Index = a + 1
    If VT_Index >= UBound(VT_NameZX) Then VT_Index = 1: FrmMainWnd.SelectTB (VT_Index) Else FrmMainWnd.SelectTB (VT_Index): FrmMainWnd.Ref_TB (4)
     
 If UCase(Right(VT_NameMs, 4)) = ".T81" Then Load_T81File VT_NameMs, a
    
 If UCase(Right(VT_NameMs, 4)) = ".TZX" Then
    TZXname = VT_NameMs
    If Not OpenTZX Then MsgBox "File error", vbCritical ' Open the TZX if True=OK False=File error.
    bLoadBloc = True
    ZX81name = sFileName
    Enum_TZX Bloc()
    Dispose
 End If
    
    
    Zx_Name = Cut_Name(VT_NameMs) & "-" & sFileName
    FrmMainWnd.Caption = "VB81 XuR [" & Zx_Name & "] " & GetMemName
    GoTo Exit_F
    End If
   Next a
   sFileName = Trim$(sFileName) & ".P"
        If Dir$(sFileName) = "" Then
         FrmSelectDir.Show vbModal, FrmMainWnd
          Else
         VT_Index = 1
         GoTo LoadP
        End If
Exit_F:

   If FrmDisassembler.Visible Then FrmDisassembler.Mem_Changed
   Exit Sub
  End If
  '*****************************************************************************


        sFileName = Trim$(sFileName) & ".P"
    End If
    On Error Resume Next
    If Dir$(sFileName) = "" Then Exit Sub
LoadP:
     VT_NameMs = sFileName
     Load_File_In_Mem sFileName, &H4009
     
     bInputWait = True
     ShowDisplay True, False
     
Ret_Name:
     Zx_Name = sFileName
     
     If OpenDialog.FileName = "" Then OpenDialog.FileName = sFileName Else sFileName = OpenDialog.FileName
     If sFileName = "" Then Exit Sub
     ReDim VT_NameZX(0)
     Cnm = Cut_Name(VT_NameMs)
     VT_NameZX(0).Nom_Fichier = Left(Cnm, Len(Cnm) - 2)
     FrmMainWnd.Init_VTape
     FrmMainWnd.Caption = "VB81 XuR [" & Cut_Name(Zx_Name) & "] " & GetMemName
     If FrmDisassembler.Visible Then FrmDisassembler.Mem_Changed
     RAM_TOP = peekw(16388)
     Debug.Print FileLen(sFileName) + &H4009
     If (FileLen(sFileName) + &H4009 > RAM_TOP) Then GoTo RamTop_err
     Exit Sub
     
RamTop_err:


    If FileLen(OpenDialog.FileName) > (SRamLim + 1 - &H4000) Then
     
    Select Case FileLen(OpenDialog.FileName)
    Case 0 To &H400
    InfoRam = RamPack(0).Name
    
    Case &H401 To &H800
    InfoRam = RamPack(1).Name
    
    Case &H801 To &H4000
    InfoRam = RamPack(2).Name
    
    Case &H4001 To &H8000
    InfoRam = RamPack(3).Name
    
    Case &H8001 To &H10000
    InfoRam = RamPack(4).Name
    End Select

    result = MsgBox("This file is designed to run with more memory." & vbCrLf _
    & " You had to change emulator's setup to run this file" & vbCrLf & "Please select : " & InfoRam & " device." & vbCrLf & "Try anyway (OK) or Reset (Cancel)", vbOKCancel, "Critical Zx81's memory error to load this file!")
    If result = vbCancel Then
    sFileName = ""
    Z80ResetHard
    Exit Sub
    End If

GoTo Ret_Name
    End If

    result = MsgBox("This file is designed to run with more memory." & vbCrLf & " You had to initialize your RAM to suite to upper Sinclair 16kb ram pack." & vbCrLf & " Set the RamTop to the suitable address." & vbCrLf & "-64kb RamPack = POKE 16389, 255: NEW" & vbCrLf & "-48kb RamPack = POKE 16389, 192: NEW" & vbCrLf & "-32kb RamPack = POKE 16389, 128: NEW" & vbCrLf & "-16kb RamPack = POKE 16388, 0:POKE 16389, 80: NEW" & vbCrLf & "-1kb RamPack =POKE 16388, 0: POKE 16389, 68: NEW" & vbCrLf & "(POKE 16388,255 - this is not usually needed)" & vbCrLf & "Try an 64K init. (OK) or Reset (Cancel)", vbOKCancel, "Critical memory error to load this file!")
    OpenDialog.FileName = sFileName
    If result = vbCancel Then
    sFileName = ""
    Z80ResetHard
    Exit Sub
    Else
        Write_Flash_inMemRAMTOP "32K"
    End If

GoTo Ret_Name
End Sub

Sub TapeLoad_Win(sFileName As String, Optional Ram As String = "16K")
    Dim InfoRam As String
    Dim RAM_TOP As Long
    Dim result
    
     bInputWait = True
     ShowDisplay True, False
     bBooting = True
 
 Init_Regs
    
' // Initialize everything
    intIFF1 = True
    intIFF2 = True
    intIM = 0
    initParity
    
' Load file
Load_File_In_Mem sFileName, &H4009



 RAM_TOP = peekw(16388)
     
     
     If FileLen(sFileName) > (SRamLim + 1 - &H4000) Then
    
    
    Select Case FileLen(sFileName)
    Case 0 To &H400
    InfoRam = RamPack(0).Name
    
    Case &H401 To &H800
    InfoRam = RamPack(1).Name
    
    Case &H801 To &H4000
    InfoRam = RamPack(2).Name
    
    Case &H4001 To &H8000
    InfoRam = RamPack(3).Name
    
    Case &H8001 To &H10000
    InfoRam = RamPack(4).Name
    End Select
    
    result = MsgBox("This file is designed to run with more memory." & vbCrLf _
    & " You had to change emulator's setup to run this file" & vbCrLf & "Please select : " & InfoRam & " device." & vbCrLf & "Try anyway (OK) or Reset (Cancel)", vbOKCancel, "Critical Zx81's memory error to load this file!")
    If result = vbCancel Or Ram = "32K" Then
     sFileName = ""
     Z80ResetHard
      Exit Sub
    Else
        Write_Flash_inMemRAMTOP "32K"
    End If

    End If
    
 If (FileLen(sFileName) + &H4009 > RAM_TOP) Then
        result = MsgBox("This file is designed to run with more memory." & vbCrLf & " You had to initialize your RAM to suite to upper Sinclair 16kb ram pack." & vbCrLf & " Set the RamTop to the suitable address." & vbCrLf & "-64kb RamPack = POKE 16389, 255: NEW" & vbCrLf & "-48kb RamPack = POKE 16389, 192: NEW" & vbCrLf & "-32kb RamPack = POKE 16389, 128: NEW" & vbCrLf & "-16kb RamPack = POKE 16388, 0:POKE 16389, 80: NEW" & vbCrLf & "-1kb RamPack =POKE 16388, 0: POKE 16389, 68: NEW" & vbCrLf & "(POKE 16388,255 - this is not usually needed)" & vbCrLf & "Try anyway (OK) or Reset (Cancel)", vbOKCancel, "Critical memory error to load this file!")
    If result = vbCancel Then
    sFileName = ""
    Z80ResetHard
    Exit Sub
    End If
 End If
    
If Not AutoStart Then
' Reinit Basic systems vars
MEM(&H400A) = 0: MEM(&H400B) = 0 ' Line Number of currently selected line [>] Cursor
MEM(&H4023) = 0: MEM(&H4024) = 0 ' Line Number of first line for automatic LISTing
MEM(&H4007) = 0: MEM(&H4008) = 0 ' Line Number of most recently executed BASIC line
MEM(&H402B) = 0: MEM(&H402C) = 0 ' Line Number for CONT
MEM(&H4029) = &H7E: MEM(&H402A) = &H40 ' Address of next BASIC line which is to be executed
End If
   

'Reinit UDG Offset
intI = &H1E


' NMI BUGGGGGGGGG

    ' // PERFORM Z80 hardware functions (mimicking the NMI on a real ZX81)
    If (MEM(16443) And 128) = 128 Then
          ' FAST Mode
        regAF_ = regAF_ And (Not 32768)

        Else

    End If

' Refresh basic program (Magic offset!)
regPC = &H46F
MEM(16443) = 128 + 64

        bBooting = False
        bInputWait = False
        ShowDisplay True, True

   
    ScreenPaint
    ResetKeyboard
  
  
 Dim D_FILE As Long
   D_FILE = ((CLng(MEM(16397)) * 256) + CLng(MEM(16396))) + 1 + (33 * 20)
   
      MEM(D_FILE) = &H97: MEM(D_FILE + 1) = &HAB: MEM(D_FILE + 2) = &HAE: MEM(D_FILE + 3) = &HB1
      MEM(D_FILE + 4) = &HAA: MEM(D_FILE + 5) = &H80: MEM(D_FILE + 6) = &HB1: MEM(D_FILE + 7) = &HB4
      MEM(D_FILE + 8) = &HA6: MEM(D_FILE + 9) = &HA9: MEM(D_FILE + 10) = &HAA: MEM(D_FILE + 11) = &HA9
      MEM(D_FILE + 12) = &H97
  
  D_FILE = D_FILE + 33
   MEM(D_FILE) = 28
   MEM(D_FILE + 1) = 24
   MEM(D_FILE + 2) = 28
   

ReDim VT_NameZX(0)

         Zx_Name = Cut_Name(sFileName)
         Dim TmpStr As String
         TmpStr = GetMemName
         If TmpStr = "" Or InStr(1, TmpStr, "_") > 0 Then
         If UCase(Right(Zx_Name, 2)) = ".P" Then
         VT_NameZX(0).Nom_Fichier = Left(Zx_Name, Len(Zx_Name) - 2)
         Else
         VT_NameZX(0).Nom_Fichier = Left(Zx_Name, Len(Zx_Name) - 3)
         End If
         Else
         VT_NameZX(0).Nom_Fichier = Left(Right(TmpStr, Len(TmpStr) - 6), Len(TmpStr) - InStr(7, TmpStr, Chr(34)) - 2)
         End If
         FrmMainWnd.Caption = "VB81 XuR [" & Zx_Name & "] " & TmpStr
         If FrmDisassembler.Visible Then FrmDisassembler.Mem_Changed


'Tape Info.
VT_Index = 1
VT_NameZX(0).Offset = 1
VT_NameZX(0).Taille = FileLen(sFileName)
VT_NameMs = sFileName

End Sub

Public Sub TapeSave(ByVal FileNamePtr As Long)
    Dim sFileName As String, i As Long, ln As Long
    Dim hFile As Long
    Dim RAM_TOP As Long
    
    RAM_TOP = peekw(16388) + 1
    If FileNamePtr >= 32768 And RAM_TOP < 32768 Then
    
        ' // SAVE ""
        
        ' Save function aborted!
        ' Your memory card isn't initialized.
        Exit Sub
    Else
        ' // SAVE "filename"
        If RAM_TOP < 32768 Then
        FileNamePtr = FileNamePtr And 32767
        Else
        FileNamePtr = FileNamePtr
        End If
        
        ' // FileNamePtr points to the filename in the ZX81 character set
        Do While MEM(FileNamePtr) < 127
            sFileName = sFileName + ZXCharToASCII(MEM(FileNamePtr))
            FileNamePtr = FileNamePtr + 1
        Loop
        
        sFileName = Trim$(sFileName & ZXCharToASCII(MEM(FileNamePtr)))

        
        
        ' // S A V E   I N   F A S T   L O A D  F O R M A T .
            ' // Intercept the text tag.
        If InStr(1, UCase(sFileName), ".FL") <> 0 Then
        If InStr(1, UCase(sFileName), ".WAV") = 0 Then sFileName = sFileName & ".WAV"
        FrmInfoFastLoad.Show
        FrmInfoFastLoad.Caption = "Generate a Wav File in Fast-load format."
          FrmInfoFastLoad.FileInfo = "File name: " & vbCrLf & Get_LastPath & sFileName
        Zx_Name = Left(sFileName, InStr(1, UCase(sFileName), ".FL") - 1)
        Call FrmSound.Save_FastLoad(Get_LastPath & sFileName)
        Exit Sub
        End If
        
        
        ' // S A V E   I N   WAV  F O R M A T .
            ' // Intercept the text tag.
        If InStr(1, UCase(sFileName), ".WAV") <> 0 Then
        
        'FrmInfoFastLoad.Caption = "Generate a Wav File format."
        'FrmInfoFastLoad.FileInfo = "File name: " & vbCrLf & Get_LastPath & sFileName
        Zx_Name = Left(sFileName, InStr(1, UCase(sFileName), ".WAV") - 1)
        Call FrmSound.Save_WAVformat(Get_LastPath & sFileName)
        Exit Sub
        End If
        
        
        
        ' // S A V E   M E M O R Y   I N   T E X T   F O R M A T .
        ' // Intercept the text tag.
        If InStr(1, UCase(sFileName), ".TXT") <> 0 Then
        Zx_Name = Left(sFileName, InStr(1, UCase(sFileName), ".TXT") - 1)
        TConvBasLen = (CDbl(MEM(16397)) * 256 + CDbl(MEM(16396)) - 1) - 16509
       If TConvBasLen <= 0 Then
       MsgBox "Type in your program firt !", vbCritical, "No line in memory!"
       Exit Sub
       End If
       
       ' Case of SAVE".TXT"
       If InStr(1, UCase(sFileName), ".TXT") = 1 Then
       Zx_Name = "MyWork"
       End If
       
       '// In case of "MyGame.TXT:C" , save CBI functions in the text file.
       If InStr(1, UCase(sFileName), ":C") <> 0 Then
       bCBIconv = True
       Else
       bCBIconv = False
       End If
       
       '// In case of "MyGame.TXT:V" , save CBI functions in the text file.
       If InStr(1, UCase(sFileName), ":V") <> 0 Then
       bVars = True
       Else
       bVars = False
       End If
       
       '// In case of "MyGame.TXT:R" , save the text file in regulare ASCII format.
       If InStr(1, UCase(sFileName), ":R") <> 0 Then
       bTxtFormat = True
       Else
       bTxtFormat = False
       End If
       
       '// In case of "MyGame.TXT:D" , save ASM IN DECIMALS values.
       If InStr(1, UCase(sFileName), ":D") <> 0 Then
       bASMhex = False
       Else
       bASMhex = True
       End If

       
        ' Call the Text converter window.
        TextFile = eEditorText
        If bTxtFormat = False Then GRAPHIC_MODE = True Else GRAPHIC_MODE = False
        FrmTextCompil.Show 1, FrmMainWnd
        
        Exit Sub
        
        End If
        
        ' TXT test aborted, continue and save a P file in the current directory.
        sFileName = sFileName & ".P"
    End If
    
    err.Clear
    On Error Resume Next
    
MsTest:
    '/ Illegal MS name
    If InStr(1, sFileName, "/") Or InStr(1, sFileName, ":") Or _
    InStr(1, sFileName, "*") Or InStr(1, sFileName, "?") Or _
    InStr(1, sFileName, Chr(34)) Or InStr(1, sFileName, "<") Or _
    InStr(1, sFileName, ">") Then
    sFileName = InputBox("Please enter another name without / : * ? " & Chr(34) & " < >", "Msdos name error", sFileName)
    GoTo MsTest
    End If
    
    hFile = FreeFile
    
    
    Open sFileName For Output As hFile
    Debug.Print CurDir$ & "\" & sFileName
    If err.number <> 0 Then
        MsgBox "Error: Unable to open output file:" & vbCrLf & vbCrLf & "   " & CurDir$ & "\" & sFileName, vbOKOnly Or vbExclamation, "vb81"
        err.Clear
        Exit Sub
    End If
        
    For i = &H4009 To peekw(16404)
        Print #hFile, Chr$(MEM(i));
    Next i
F_ERR:
    If err.number <> 0 Then
        MsgBox "Error: Unable to write to output file:" & vbCrLf & vbCrLf & "   " & CurDir$ & "\" & sFileName, vbOKOnly Or vbExclamation, "vb81"
        err.Clear
    End If
    
    Close #hFile
    
         Zx_Name = sFileName
         FrmMainWnd.Caption = "VB81 XuR [" & sFileName & "]" & GetMemName
         If FrmDisassembler.Visible Then FrmDisassembler.Mem_Changed

    bInputWait = True
    ShowDisplay True, True
End Sub


Public Function Update_Kybd() As Boolean

        If bBooting Then Exit Function
        
    Dim oldLastK1 As Long, oldLastK2 As Long
    Dim LastK1 As Long, LastK2 As Long
    
    oldLastK1 = MEM(16421)
    oldLastK2 = MEM(16422)
    
    
    ' Boot!
    If oldLastK1 = 0 And oldLastK2 = 0 Then
        MEM(16421) = 255
        MEM(16422) = 255
        Exit Function
    End If
    


    ' ################### A to G Key ######################
    
    ' A
    If (keyA_G And &H1) = 0 Then
        LastK1 = 253
        LastK2 = 253
    End If

    ' S
    If (keyA_G And &H2) = 0 Then
        LastK1 = 253
        LastK2 = 251
    End If

    ' D
    If (keyA_G And &H4) = 0 Then
        LastK1 = 253
        LastK2 = 247
    End If

    ' F
    If (keyA_G And &H8) = 0 Then
        LastK1 = 253
        LastK2 = 239
    End If

    ' G
    If (keyA_G And &H10) = 0 Then
        LastK1 = 253
        LastK2 = 223
    End If
    
    ' ################### H to ENTER Key ######################
    ' H
    If (keyH_ENT And &H10) = 0 Then
        LastK1 = 191
        LastK2 = 223
    End If

    ' J
    If (keyH_ENT And &H8) = 0 Then
        LastK1 = 191
        LastK2 = 239
    End If

    ' K
    If (keyH_ENT And &H4) = 0 Then
        LastK1 = 191
        LastK2 = 247
    End If

    ' L
    If (keyH_ENT And &H2) = 0 Then
        LastK1 = 191
        LastK2 = 251
    End If

    ' ENTER
    If (keyH_ENT And &H1) = 0 Then
        LastK1 = 191
        LastK2 = 253
    End If
    
    ' ################### 1 to 5 Key ######################
    ' 1
    If (key1_5 And &H1) = 0 Then
        LastK1 = 247
        LastK2 = 253
    End If
    ' 2
    If (key1_5 And &H2) = 0 Then
        LastK1 = 247
        LastK2 = 251
    End If
    ' 3
    If (key1_5 And &H4) = 0 Then
        LastK1 = 247
        LastK2 = 247
    End If
    ' 4
    If (key1_5 And &H8) = 0 Then
        LastK1 = 247
        LastK2 = 239
    End If
    ' 5
    If (key1_5 And &H10) = 0 Then
        LastK1 = 247
        LastK2 = 223
    End If
    
    ' ################### 6 to 0 Key ######################
    ' 6
    If (key6_0 And &H10) = 0 Then
        LastK1 = 239
        LastK2 = 223
    End If
    ' 7
    If (key6_0 And &H8) = 0 Then
        LastK1 = 239
        LastK2 = 239
    End If
    ' 8
    If (key6_0 And &H4) = 0 Then
        LastK1 = 239
        LastK2 = 247
    End If
    ' 9
    If (key6_0 And &H2) = 0 Then
        LastK1 = 239
        LastK2 = 251
    End If
    ' 0
    If (key6_0 And &H1) = 0 Then
        LastK1 = 239
        LastK2 = 253
    End If
    
    ' ################### Q to T Key ######################
    ' Q
    If (keyQ_T And &H1) = 0 Then
        LastK1 = 251
        LastK2 = 253
    End If
    ' W
    If (keyQ_T And &H2) = 0 Then
        LastK1 = 251
        LastK2 = 251
    End If
    ' E
    If (keyQ_T And &H4) = 0 Then
        LastK1 = 251
        LastK2 = 247
    End If
    ' R
    If (keyQ_T And &H8) = 0 Then
        LastK1 = 251
        LastK2 = 239
    End If
    ' T
    If (keyQ_T And &H10) = 0 Then
        LastK1 = 251
        LastK2 = 223
    End If
    
    ' ################### Y to P Key ######################
    ' Y
    If (keyY_P And &H10) = 0 Then
        LastK1 = 223
        LastK2 = 223
    End If
    ' U
    If (keyY_P And &H8) = 0 Then
        LastK1 = 223
        LastK2 = 239
    End If
    ' I
    If (keyY_P And &H4) = 0 Then
        LastK1 = 223
        LastK2 = 247
    End If
    ' O
    If (keyY_P And &H2) = 0 Then
        LastK1 = 223
        LastK2 = 251
    End If
    ' P
    If (keyY_P And &H1) = 0 Then
        LastK1 = 223
        LastK2 = 253
    End If
    
    ' ################### Z to V Key ######################
    ' Z
    If (keyCAPS_V And &H2) = 0 Then
        LastK1 = 254
        LastK2 = 251
    End If
    ' X
    If (keyCAPS_V And &H4) = 0 Then
        LastK1 = 254
        LastK2 = 247
    End If
    ' C
    If (keyCAPS_V And &H8) = 0 Then
        LastK1 = 254
        LastK2 = 239
    End If
    ' V
    If (keyCAPS_V And &H10) = 0 Then
        LastK1 = 254
        LastK2 = 223
    End If
    
    ' ################### B to Space Key ######################
    ' B
    If (keyB_SPC And &H10) = 0 Then
        LastK1 = 127
        LastK2 = 223
    End If
    ' N
    If (keyB_SPC And &H8) = 0 Then
        LastK1 = 127
        LastK2 = 239
    End If
    ' M
    If (keyB_SPC And &H4) = 0 Then
        LastK1 = 127
        LastK2 = 247
    End If
    ' .
    If (keyB_SPC And &H2) = 0 Then
        LastK1 = 127
        LastK2 = 251
    End If
    ' SPACE
    If (keyB_SPC And &H1) = 0 Then
        LastK1 = 127
        LastK2 = 253
    End If
    
    ' ################### SHIFT Key ######################
   ' SHIFT
    If (keyCAPS_V And &H1) = 0 Then
        LastK2 = LastK2 And (Not 1&)
    End If
    
    If LastK1 = 0 Then
        MEM(16421) = 255
        If (keyCAPS_V And &H1) = 0 Then
             MEM(16422) = 254
        Else
            MEM(16422) = 255
        End If
        Exit Function
    End If
   

    
    ' ############# SET KEYBOARD VALUES ##################

    MEM(16421) = LastK1
    MEM(16422) = LastK2
    
   ' If MEM(16443) And 1 = 1 Then MEM(16443) = MEM(16443) - 1

   ' Remmed due to a Memocalc freezing or the keyboard trigger is ignored.
   If MEM(&H4027) <> 255 Then bInputWait = True
   
   
   If bInputWait = False Then Exit Function
    
    ' // Set FRAMES
    MEM(16424) = 55
    If MEM(&H4027) = 255 Then
        MEM(&H4027) = 15
    Else
        If ((MEM(16421) <> oldLastK1) Or ((MEM(16422) And 254) <> (oldLastK2 And 254))) And _
            (MEM(16421) <> 255 And MEM(16422) <> 255) Then
            MEM(16443) = MEM(16443) Or 1
                       
        End If
    End If

    
    Update_Kybd = True

End Function




Sub Load_TrueHrg(CARD As Graphic_Card)
'        if (zx81.truehires==HIRESMEMOTECH) memory_load("memohrg.rom", 8192, 2048);
'        if (zx81.truehires==HIRESG007) memory_load("g007hrg.rom",10240,2048);
'        if (zx81.truehires==HIRESQS) memory_load("QShrg.rom",10240,2048);

Dim Name As String
Dim Rom_Lenght As Integer
Dim Rom_offset As Integer

Select Case CARD
'MEMOTECH
Case 1
Name = "memohrg.rom": Rom_Lenght = 2048: Rom_offset = 8192

'G007
Case 2
Name = "g007hrg.rom": Rom_Lenght = 2048: Rom_offset = 10240

'QS-HRG
Case 3
Name = "QSHrg.rom": Rom_Lenght = 2048: Rom_offset = 10240

End Select

On Error Resume Next
       If Dir$(App.path & "\Roms\" & Name) = "" Then
        MsgBox "Error: Could not find required ROM image '" & App.path & "\Roms\" & Name & "'.", vbOKOnly Or vbCritical, "vb81"
        Unload MainWindow
        Exit Sub
    End If
    
UpdateRomAlloc App.path & "\Roms\" & Name, CLng(Rom_offset)
Load_File_In_Mem App.path & "\Roms\" & Name, CLng(Rom_offset)

End Sub

Sub Load_LDos()

On Error Resume Next
       If Dir$(App.path & "\Roms\LDos.ROM") = "" Then
        MsgBox "Error: Could not find required ROM image '" & App.path & "\Roms\LDos.ROM'.", vbOKOnly Or vbCritical, "vb81"
        Unload MainWindow
        Exit Sub
    End If
    
' Load Larken Dos in 3800h - 4000h
Load_File_In_Mem App.path & "\Roms\LDos.ROM", 14336 ' -> 3800h
UpdateRomAlloc App.path & "\Roms\LDos.ROM", 14336
' Patch larken ROM to Load or Save Tracks.
LarkenHack

End Sub

Sub Load_MDos()

On Error Resume Next
       If Dir$(App.path & "\Roms\Mageco" & GetSetting("Grok", "vb81", "MagecoROM", "2.6") & ".rom") = "" Then
        MsgBox "Error: Could not find required ROM image '" & App.path & "\Roms\Mageco" & GetSetting("Grok", "vb81", "MagecoROM", "2.6") & ".rom'.", vbOKOnly Or vbCritical, "vb81"
        Exit Sub
    End If
    
' Load Larken Dos in 2000h - 3000h
Load_File_In_Mem App.path & "\Roms\Mageco" & GetSetting("Grok", "vb81", "MagecoROM", "2.6") & ".rom", 8192 ' -> 2000h
UpdateRomAlloc App.path & "\Roms\Mageco" & GetSetting("Grok", "vb81", "MagecoROM", "2.6") & ".rom", 8192

End Sub


Sub Load_CBM1541()

On Error Resume Next
       If Dir$(App.path & "\Roms\CBM1541.ROM") = "" Then
        MsgBox "Error: Could not find required ROM image '" & App.path & "\Roms\CBM1541.ROM'.", vbOKOnly Or vbCritical, "vb81"
        Unload MainWindow
        Exit Sub
    End If
    
' Load Larken Dos in 3800h - 4000h
Load_File_In_Mem App.path & "\Roms\CBM1541.ROM", 8192 ' -> 2000h
UpdateRomAlloc App.path & "\Roms\CBM1541.ROM", 8192

' Patch CBM ROLM to Load or Save Tracks.
'LarkenHack


End Sub

Sub Load_File_In_Mem(Name As String, Mem_Offset As Long, Optional LOB As Long = 0)
Dim ln As Long
Dim hFile As Integer
Dim Err_File As Integer

hFile = FreeFile
Err_File = 0
On Error GoTo exit_sub

       If Right(Name, 1) = "\" Then Exit Sub
       If Dir$(Name) = "" Or Right(Name, 1) = "\" Then Err_File = 1: GoTo exit_err
       If FileLen(Name) = 0 Then Err_File = 2: GoTo exit_err
       
     Open Name For Binary Access Read Lock Read As #hFile
         ln = LOF(hFile)
         ReDim Newbuffer(0 To ln - 1) As Byte
         Get #hFile, , Newbuffer
    Close #hFile
    
    ' Cut the file, if the buffer is up to the memory top.
    If (Mem_Offset + ln) - 65535 > 0 Then ln = 65535 - Mem_Offset
    CopyMemory MEM(Mem_Offset), Newbuffer(0), IIf(LOB <> 0, LOB, ln)

    Exit Sub
    
exit_sub:

    Close #hFile
    
  
exit_err:

  If Err_File = 1 Then MsgBox "This program attempt to load '" & Name & "'..." & vbCrLf & "But, this file isn't in the current directory !" & vbCrLf & vbCrLf & "If this file have a ~*.C~ extension, it's a Xtender's binary file " & vbCrLf & "loaded using a LOAD~*LOAD:C:myprog:16514~." & vbCrLf & "Have a look to files extensions inclued in your software package ..." & vbCrLf & "or the Basic SAVE line in the main program.", vbOKOnly Or vbCritical, "File access error:"
  If Err_File = 2 Then MsgBox "This file is empty !", vbOKOnly Or vbCritical, "Zero byte File error:"
  
End Sub


Sub Active_Drive0()

If bLarken Or bMageco Then

If bLarken Then Save_Disk

ActiveDrive = Drive0

''// Larken's Ram Buffer memory bloc
If bLarken Then
Load_Disk
End If

FrmMainWnd.active(0).BackColor = vbGreen
FrmMainWnd.active(1).BackColor = vbRed


Else
If GetSetting("Grok", "vb81", "Mageco", "0") = "1" Or GetSetting("Grok", "vb81", "Larken", "0") = "1" Then MsgBox "Larken/Mageco disk controler not installed.", vbCritical, "I/O error"
End If

End Sub


Sub Active_Drive1()

If bLarken Or bMageco Then
If bLarken Then Save_Disk

ActiveDrive = Drive1

'// Larken's Ram Buffer memory bloc
If bLarken Then
Load_Disk
End If

FrmMainWnd.active(0).BackColor = vbRed
FrmMainWnd.active(1).BackColor = vbGreen


Else
If GetSetting("Grok", "vb81", "Mageco", "0") = "1" Or GetSetting("Grok", "vb81", "Larken", "0") = "1" Then MsgBox "Larken/Mageco disk controler not installed.", vbCritical, "I/O error"
End If

End Sub
Sub Halt_CPU()
MsgBox "Halt", vbOKOnly, "..."
     bInputWait = True
     ShowDisplay True, False
End Sub

Public Sub Get_WAV(file_name As String)
Dim I_Count As Long
Dim Wav_Buffer16() As Integer
Dim Wav_Buffer16b() As Byte
Dim ln As Long
Dim hFile As Integer
Dim Tmp_Str As String
Dim y1 As Byte
Dim y2 As Byte
Dim ValY As Integer

Select Case wInfo.Freq
Case "22050"
    If wInfo.Bits = 8 Then
    ' Load The 8bits WAV files (sample 22050Hz).
        hFile = FreeFile
        Open file_name For Binary Access Read As #hFile
         ln = LOF(hFile)
         ReDim Wav_Buffer(0 To ln - 46) As Byte
         Get #hFile, 45, Wav_Buffer
        Close #hFile
    Else
    ' Load The 16bits WAV files (sample 22050Hz).
        hFile = FreeFile
        Open file_name For Binary Access Read As #hFile
         ln = LOF(hFile)
         ReDim Wav_Buffer16(0 To (ln - 46) \ 2)
         Get #hFile, 45, Wav_Buffer16
        Close #hFile
        ReDim Wav_Buffer((ln - 46) / 2)

        For I_Count = 0 To UBound(Wav_Buffer16) - 1
        Wav_Buffer(I_Count) = CByte(128 + ((Wav_Buffer16(I_Count) / 32767) * 128))
        Next
        Erase Wav_Buffer16
   
    End If
Case "44100"
    If wInfo.Bits = 8 Then
    ' Load The 8bits WAV files (sample 44100Hz).
        hFile = FreeFile
        Open file_name For Binary Access Read As #hFile
         ln = LOF(hFile)
         ReDim Wav_Buffer16b(0 To ln - 46)
         Get #hFile, 45, Wav_Buffer16b
        Close #hFile
        
        ReDim Wav_Buffer((ln - 46) / 2)
        For I_Count = 0 To UBound(Wav_Buffer) - 1
        y1 = CByte(Wav_Buffer16b((I_Count * 2)))
        y2 = CByte(Wav_Buffer16b((I_Count * 2) + 1))
        Wav_Buffer(I_Count) = CByte(IIf(y1 > 128, IIf(y1 >= y2, y1, y2), IIf(y1 <= y2, y1, y2)))
        Next
        Erase Wav_Buffer16b

    Else
    ' Load The 16bits WAV files (sample 44100Hz).
        hFile = FreeFile
        Open file_name For Binary Access Read As #hFile
         ln = LOF(hFile)
         ReDim Wav_Buffer16(0 To (ln - 46) / 2)
         Get #hFile, 45, Wav_Buffer16
        Close #hFile
        ReDim Wav_Buffer((ln - 46) / 4)
        For I_Count = 0 To UBound(Wav_Buffer16) - 1 Step 2
        ValY = Wav_Buffer16(I_Count)
        Wav_Buffer(I_Count \ 2) = CByte(128 + ((ValY / 32767) * 128))
        Next
        Erase Wav_Buffer16
    End If
Case Else
    ' Load  WAV files (sample ???).
        hFile = FreeFile
        Open file_name For Binary Access Read As #hFile
         ln = LOF(hFile)
         ReDim Wav_Buffer(0 To ln - 46) As Byte
         Get #hFile, 45, Wav_Buffer
        Close #hFile

End Select

    Exit Sub
    
exit_sub:

    Close #hFile
End Sub

Sub DecodeWav(C_Level As Byte, Save_P As Boolean, Load_mem As Boolean, M_Stop As Boolean, Optional file_name As String = "")

On Error GoTo error_Conv

Dim Cnt As Long
Dim Tmp_Str As String
Dim I_Count As Long
Dim a As Long
Dim hFile As Integer
Dim New_Index As Long
Dim Wav_Z_Count() As Long
Dim I_CountZ As Long
Dim Wav_Buffer_T() As Byte ' Cleaned wav file.(255 hight level/128 low level)
Dim Wav_Buffer2() As Byte ' Silence counter.
Dim Bits_array() As Boolean ' Bits Array (0-1)
FrmSoundIn!progress.Move 0, 0, 5, FrmSoundIn!wait.height
DoEvents
    ' Clean the Wav.
    ' Get
    

Tmp_Str = ""

'// Copy Wav_buffer in Wav_Buffer_T
ReDim Wav_Buffer_T(UBound(Wav_Buffer))

'Init Streamer
  Dim cStream As New cBinaryFileStream
   cStream.Init_Array Wav_Buffer() ' Wav_Buffer is now buffered.

' Data streamer
Dim buffer(0 To 8191) As Byte
Dim readSize As Long
Dim i As Integer
readSize = 8192

Dim Count As Integer

Count = cStream.Read_Array(buffer, Wav_Buffer, readSize)
    
Select Case C_Level
    Case Is > 128
     
     Do While (Count > 0)
      For i = 0 To Count - 1
     If buffer(i) >= C_Level Then buffer(i) = 255 Else buffer(i) = 128
     Next i
     CopyMemory Wav_Buffer_T(cStream.Curr_Array_Offset - Count), buffer(0), Count
     Count = cStream.Read_Array(buffer, Wav_Buffer, readSize)
     Loop
     
     Case Is <= 128
     
     Do While (Count > 0)
      For i = 0 To Count - 1
     If buffer(i) <= C_Level Then buffer(i) = 255 Else buffer(i) = 128
     Next i
     CopyMemory Wav_Buffer_T(cStream.Curr_Array_Offset - Count), buffer(0), Count
     Count = cStream.Read_Array(buffer, Wav_Buffer, readSize)
     Loop
End Select
    
FrmSoundIn!progress.Move 0, 0, (FrmSoundIn!wait.width \ 6), FrmSoundIn!wait.height
DoEvents

    New_Index = 0
    cStream.Init_Array Wav_Buffer_T() ' "Wav_Buffer_T" is now buffered.

    Count = cStream.Read_Array(buffer, Wav_Buffer_T, readSize)
    
     Do While (Count > 1)
     
 For i = 0 To Count - 2
 Select Case buffer(i)
 Case 255
         If buffer(i + 1) = 128 Then
            ReDim Preserve Wav_Buffer2(New_Index)
            Wav_Buffer2(New_Index) = 255
            New_Index = New_Index + 1
            Cnt = 0
         End If
  Case Else
            ReDim Preserve Wav_Buffer2(New_Index)
            Wav_Buffer2(New_Index) = 128
            New_Index = New_Index + 1
  End Select
           
  Next i
           
     Count = cStream.Read_Array(buffer, Wav_Buffer_T, readSize, -1)
     Loop

   ' For I_Count = 0 To UBound(Wav_Buffer_T)
   ' If Wav_Buffer_T(I_Count) = 255 And I_Count + 1 < UBound(Wav_Buffer_T) Then
         
   '     If Wav_Buffer_T(I_Count + 1) = 128 Then
            
   '         ReDim Preserve Wav_Buffer2(New_Index)
   '         Wav_Buffer2(New_Index) = 255
   '         New_Index = New_Index + 1
   '         Cnt = 0
            
    '    End If
        
   ' Else
   '         ReDim Preserve Wav_Buffer2(New_Index)
   ''         Wav_Buffer2(New_Index) = 128
   '         New_Index = New_Index + 1
                
   ' End If
   ' Next
    
FrmSoundIn!progress.Move 0, 0, (FrmSoundIn!wait.width \ 6) * 2, FrmSoundIn!wait.height
DoEvents
    
ReDim Wav_Buffer_T(UBound(Wav_Buffer2))
CopyMemory Wav_Buffer_T(0), Wav_Buffer2(0), CLng(UBound(Wav_Buffer2))

Erase Wav_Buffer2

'Count Zero in wav. (Count every zero pitch)

New_Index = 0
I_CountZ = 0
    
Do While I_CountZ <= UBound(Wav_Buffer_T)
    If Wav_Buffer_T(I_CountZ) = 255 Then
          ReDim Preserve Wav_Z_Count(New_Index)
          Wav_Z_Count(New_Index) = -1
          I_CountZ = I_CountZ + 1
    Else
        Cnt = 0
                Do While (Wav_Buffer_T(I_CountZ) = 128) And (Cnt < &H7FFF)
                I_CountZ = I_CountZ + 1: Cnt = Cnt + 1
                If I_CountZ >= UBound(Wav_Buffer_T) Then Exit Do
                Loop
    ReDim Preserve Wav_Z_Count(New_Index)
    Wav_Z_Count(New_Index) = Cnt
    End If
    
    New_Index = New_Index + 1
   If I_CountZ >= UBound(Wav_Buffer_T) Then Exit Do
Loop

FrmSoundIn!progress.Move 0, 0, (FrmSoundIn!wait.width \ 6) * 3, FrmSoundIn!wait.height
DoEvents

' ------------------------------------------------------------------

'   Decode bits.
I_CountZ = 0 ' Pause counter offset array.
Cnt = 0 ' dot/pitch counter.

' cnt = 4 piches = false = 0
' cnt = 9 piches = true = 1
' Wav_Z_Count(I_CountZ)> 15 = end of bit

New_Index = 0
ReDim Preserve Bits_array(New_Index)

Do While I_CountZ <= UBound(Wav_Z_Count)

   Cnt = 0
   If (Wav_Z_Count(I_CountZ) > 15) And (I_CountZ < UBound(Wav_Z_Count)) Then I_CountZ = I_CountZ + 1
    
    
    Do While Wav_Z_Count(I_CountZ) <= 15 And I_CountZ < UBound(Wav_Z_Count)
        If Wav_Z_Count(I_CountZ) = -1 Then Cnt = Cnt + 1
        I_CountZ = I_CountZ + 1
    Loop
       
    If Cnt >= 2 And Cnt <= 4 Then
        ReDim Preserve Bits_array(New_Index)
        Bits_array(New_Index) = False
        New_Index = New_Index + 1
        I_CountZ = I_CountZ - 1
    End If
    
     If Cnt > 4 And Cnt <= 15 Then
        ReDim Preserve Bits_array(New_Index)
        Bits_array(New_Index) = True
        New_Index = New_Index + 1
        I_CountZ = I_CountZ - 1
    End If
    
    I_CountZ = I_CountZ + 1
    
Loop

FrmSoundIn!progress.Move 0, 0, (FrmSoundIn!wait.width \ 6) * 4, FrmSoundIn!wait.height
DoEvents

If New_Index = 0 Then
If FrmSoundIn.Visible Then FrmSoundIn.wait.Visible = False
Exit Sub
End If
' ------------------------------------------------------------------
'Kill "Bits_Array.bin"
'hFile = FreeFile
'On Error GoTo err_3
'     Open "Bits_Array.bin" For Binary Access Write Lock Write As #hFile
'     Put #hFile, 1, Bits_array
'     Close #hFile


 
' Convert Bits To Bytes
If UBound(Bits_array) = 0 Then GoTo Cant_conv
If ((UBound(Bits_array) + 1) \ 8) - ((UBound(Bits_array) + 1) / 8) <> 0 Then
 MsgBox "Unable to convert this wav!" & vbCrLf & "One or several bits harmonics are corrupt." & vbCrLf & "You had to move the capture line.", vbCritical, "Error..."
Exit Sub
End If

New_Index = 0
For a = 0 To UBound(Bits_array) Step 8
ReDim Preserve Bytes_array(New_Index)
Bytes_array(New_Index) = (128 And Bits_array(a)) + (64 And Bits_array(a + 1)) + (32 And Bits_array(a + 2)) + (16 And Bits_array(a + 3)) + (8 And Bits_array(a + 4)) + (4 And Bits_array(a + 5)) + (2 And Bits_array(a + 6)) + (1 And Bits_array(a + 7))
New_Index = New_Index + 1
Next

FrmSoundIn!progress.Move 0, 0, (FrmSoundIn!wait.width \ 6) * 5, FrmSoundIn!wait.height
DoEvents

' ------------------------------------------------------------------

' ------------------------------------------------------------------
Dim N_Off As Long
N_Off = 0

Do Until Bytes_array(N_Off) > 128
N_Off = N_Off + 1
Loop
N_Off = N_Off + 1

FrmSoundIn!progress.Move 0, 0, FrmSoundIn!wait.width, FrmSoundIn!wait.height
DoEvents

If UBound(Bytes_array) - N_Off <= 0 Then GoTo Cant_conv

If Load_mem Then
' Load P file in memory.
Dim name_off As Integer
name_off = 0
Tmp_Str = ""
Do Until Bytes_array(name_off) > 128
Tmp_Str = Tmp_Str + ZXCharToASCII(Bytes_array(name_off))
name_off = name_off + 1
Loop
Tmp_Str = Tmp_Str + ZXCharToASCII(Bytes_array(name_off))
FrmMainWnd.Caption = "VB81 XuR [" & Tmp_Str & "] "
name_off = name_off + 1
CopyMemory MEM(&H4009), Bytes_array(name_off), CLng(UBound(Bytes_array) - name_off)
FrmMainWnd.Caption = FrmMainWnd.Caption & GetMemName

    'Tape Info.
    ReDim VT_NameZX(0)
    VT_NameZX(0).Nom_Fichier = Tmp_Str
    VT_Index = 1
    VT_NameZX(0).Offset = 1
    VT_NameZX(0).Taille = CLng(UBound(Bytes_array) - name_off)
    VT_NameMs = Tmp_Str & ".P"

End If

' ------------------------------------------------------------------
' Save P file.
If Save_P = True Then
If file_name = "" Then

    Load Dialog_Frame
    
         With OpenDialog
             .CancelError = True
             .DialogTitle = "Save digitalised bloc to a P format..."
             .FileName = Tmp_Str & ".p"
             .hOwner = Dialog_Frame.hWnd
             .Filter = "P files.  (*.p)|*.p|Other files (*.*)|*.*"
             .ShowSave
         End With
  
    Unload Dialog_Frame
    
    If OpenDialog.FileName = "" Then Exit Sub
    file_name = OpenDialog.FileName
End If

name_off = 0
Do Until Bytes_array(name_off) > 128
name_off = name_off + 1
Loop
name_off = name_off + 1

Dim Tmp_P_File() As Byte
ReDim Tmp_P_File(UBound(Bytes_array) - name_off)
CopyMemory Tmp_P_File(0), Bytes_array(name_off), CLng(UBound(Bytes_array) - name_off + 1)

hFile = FreeFile
On Error GoTo err_3
     Open file_name For Binary Access Write Lock Write As #hFile
     Put #hFile, 1, Tmp_P_File
     Erase Tmp_P_File
     Close #hFile
    
err_3:
Close #hFile
End If


If M_Stop Then
   ' Refresh basic program (Magic offset!)
    regPC = &H46F
    
    'Reinit UDG Offset
    intR = &HFF
    intRTemp = 0
    
    intI = &H1E

    
   ' Reinit fast mode
    MEM(16443) = 128 + 64
End If
On Error Resume Next

    
         If FrmDisassembler.Visible Then FrmDisassembler.Mem_Changed
FrmSoundIn!progress.Move 0, 0, 0, FrmSoundIn!wait.height
FrmSoundIn!wait.Visible = False
    Exit Sub
error_Conv:
Cant_conv:
    MsgBox "Unable to convert this wav!", vbCritical, "Error..."
    FrmSoundIn!progress.Move 0, 0, 0, FrmSoundIn!wait.height
    FrmSoundIn!wait.Visible = False
End Sub
Public Function Cap_Line() As Byte
If UBound(Wav_Buffer) = 0 Then Exit Function
Dim a As Long
Dim b As Long
Dim c As Long
c = 0
For a = 0 To UBound(Wav_Buffer)
If Wav_Buffer(a) > 128 Then b = b + Wav_Buffer(a): c = c + 1
Next
If c = 0 Or b = 0 Then Exit Function
Cap_Line = (((((b \ c) - 128)) \ 4) * 3) + 128
End Function

Sub SuperBreak(Optional Mute As Boolean = False)

Dim D_FILE As Long

    '// Init Z80 registers.
    regA = 0
    setF 0
    setBC 0
    regDE = 0
    regHL = 0
    
        
    ' // Initialize everything
    intIFF1 = True
    intIFF2 = True
    intIM = 0
    
    ' Reinit Basic Buffer
        If G_Card = eCOLOUR16 Or G_Card = ePENTRON16 Then CreateScreenBuffer16 Else CreateScreenBuffer
  
    'Reinit HR programas (Micro-Mouse)
    lHiresLoc = 0
    
    'Reset WRX display mode.
    Display_WRX = 0
    
    'Reset Memotech display mode.
    Mem_Display = 0

    'Reset G007 display mode.
    G007_Display = 0
    lHiresLoc = 0
    
    FrmMainWnd.DispHRGon
    
   ' Refresh basic program (Magic offset!)
    regPC = &H46F
   ' regSP = peekw(&H4029) ' point to Next-Line (allow Cont function)
    
    'Reinit UDG Offset
    intR = &HFF
    intRTemp = 0
    
    intI = &H1E

    
   ' Reinit fast mode
    MEM(16443) = 128 + 64

    ' REINIT INPUT FLAG -> bit1 ($20) waite a value (string or a value)
    MEM(16429) = 2
         
If Not Mute Then
   D_FILE = ((CLng(MEM(&H400D)) * 256) + CLng(MEM(&H400C))) + 1
   
' Don't print Break if the D_File is colapsed.
If Not (MEM(D_FILE) = &H76 And MEM(D_FILE + 1) = &H76 And MEM(D_FILE + 2) = &H76) Then
D_FILE = D_FILE + (33 * 20)
      MEM(D_FILE) = &H17: MEM(D_FILE + 1) = &H17: MEM(D_FILE + 2) = &H0: MEM(D_FILE + 3) = &H27
      MEM(D_FILE + 4) = &H37: MEM(D_FILE + 5) = &H2A: MEM(D_FILE + 6) = &H26: MEM(D_FILE + 7) = &H30
      MEM(D_FILE + 8) = &H0: MEM(D_FILE + 9) = &H17: MEM(D_FILE + 10) = &H17
  
  D_FILE = D_FILE + 33
   MEM(D_FILE) = 28
   MEM(D_FILE + 1) = 24
   MEM(D_FILE + 2) = 28
End If
End If
End Sub
Sub Put_Footer()
Dim Footer
Dim a
'&h7fe4
Footer = Array(&HF3, &H19, &H5D, &H40, &HF3, &H19, &H49, &H47, &H5, &H0, &H0, &H8, &H0, &H8, &H0, &H0, &HC2, &HA, &H25, &H47, &H1C, &H21, &HDD, &H6, &H0, &H3E)
For a = &H7FE5 To &H7FFE
MEM(a) = CByte(Footer(a - &H7FE5))
Next a
End Sub

Sub Flash_Load(file_name As String, Optional Ram As String = "16K")
Dim MemoryLen As String
MemoryLen = TestRAMTOP(file_name)
If Ram = "32K" Then MemoryLen = Ram
If Not FreezeCPU Then FreezeCPU = True
If MemoryLen = "16K" Then Write_Flash_inMem16 Else Write_Flash_inMemRAMTOP "32K"
Load_File_In_Mem file_name, &H4009

bBooting = False
bInputWait = True
DoEvents
ShowDisplay True, True
ScreenPaint

End Sub

Sub Write_Flash_inMem16()
' Load the memory stat after a Load""
' grabbed after a blank Load_Tape function.

Dim Offset As Long, a As Long
Dim snap_buffer As String
Dim Data2 As String * 2
Dim Data4 As String * 4
Dim Mem_Offset As Long

' Init register
 regPC = &H34A
 regSP = &H7FFC
 regA = &HB
 setF (&H85)
 setAF (&HB85)
 setBC (&HFF)
 
 regHL = &HC399
 regDE = &H4399
 
 regHL_ = 0
 regDE_ = &H2B
 regBC_ = &H8102
 regAF_ = &HCA89
 regIX = &H281
 regIY = &H4000
 intIM = 1

 intIFF1 = False
 intIFF2 = False
 intI = &H1E
 intR = &H80
 'HT=0
 'Nmi=1
 regAF_ = regAF_ And (Not 32768)

' System and stacks statment.
snap_buffer = "FF 80 FC 7F 00 80 00 FE FF *0003 00 7D 40 7E 40 96 43 *0002 00 97 43 9A 43 *0002 00 9B 43 9B 43 00 5D 40 00 02 *0002 00 *0003 FF 37 7D 40 *0005 00 8A 0C *0002 00 6D FD *0002 00 BC 21 18 40 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 80 EF *0002 0B 76 0B 99 43 *3C46 00 F3 19 5D 40 F3 19 F3 19 01 40 99 43 C7 12 23 00 F8 0F 40 00 BB 03 43 03 76 06 00 3E"
 
   '//   Load the Z81 string.
   
  Mem_Offset = &H4000 ' To 7FFF

add_chars:
  
If Len(snap_buffer) = 0 Then GoTo err_eof
  
  Do
     Data2 = snap_buffer
    
    If Left(Data2, 1) <> "*" Then
    ' Load ASCII value in memory
    MEM(Mem_Offset) = val("&h" & Data2)
    Mem_Offset = Mem_Offset + 1
    Select Case Len(snap_buffer)
    Case Is <= 2
    GoTo err_eof
    Case Else
    snap_buffer = Right(snap_buffer, Len(snap_buffer) - 3)
    End Select
    
    Else
    snap_buffer = Right(snap_buffer, Len(snap_buffer) - 1) ' "*" ->x
    Data4 = snap_buffer
    snap_buffer = Right(snap_buffer, Len(snap_buffer) - 5) ' "xxxx " ->x
    Data2 = snap_buffer
    If Len(snap_buffer) < 3 Then
    GoTo err_eof
    Else
    snap_buffer = Right(snap_buffer, Len(snap_buffer) - 3) ' "xx " ->x
    End If
    
  ' Load ASCII values in memory.
   For a = 1 To CLng(val("&h" & Data4))
     MEM(Mem_Offset) = val("&h" & Data2)
     If snap_buffer = "" Then GoTo err_eof
     Mem_Offset = Mem_Offset + 1
   Next
   
    End If
    
    If snap_buffer = "" Then GoTo err_eof
    
  Loop Until Left(Data2, 1) = "*" Or Data2 = vbCrLf Or InStr(1, Data2, Chr(13)) <> 0
  
  If Left(Data2, 1) = "*" Then GoTo add_chars
err_eof:
bInputWait = True
End Sub

Sub Write_Flash_inMemRAMTOP(Optional Ramtop As String = "32K") ' else "64K"
' Load the memory stat after a Load""
' grabbed after a blank Load_Tape function.

Dim Offset As Long, a As Long
Dim snap_buffer As String
Dim Data2 As String * 2
Dim Data4 As String * 4
Dim Mem_Offset As Long

' Common registers.
 regPC = &H34A
 setBC (&HFF)
 regBC_ = &H8102

 regDE_ = &H2B
 setAF (&HB85)
 regAF_ = &HCA89 And (Not 32768)
 
 intI = &H1E
 intIM = 1

 regIY = &H4000
 


Select Case Ramtop
Case "32K"

' Init registers
 regDE = &H4399: regHL = &HC399: regSP = &HAFC4: regHL_ = &H0: intR = &H80: regIX = &H281
 intIFF1 = True: intIFF2 = False

snap_buffer = "FF 80 C4 AF C8 AF 00 FE FF *0003 00 7D 40 7E 40 96 43 *0002 00 97 43 9A 43 *0002 00 9B 43 9B 43 00 5D 40 00 02 *0002 00 *0003 FF 37 7D 40 *0005 00 8A 0C *0002 00 42 FB *0002 00 BC 21 18 40 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 80 EF *0002 0B 76 0B 99 43 *6C0E 00 F3 19 5D 40 F3 19 F3 19 01 40 99 43 C7 12 23 00 F8 0F 40 00 BB 03 43 03 76 06 00 3E *5038 00"

Case "64K"

' Init register'
regDE = &H4608: regHL = &HC608: intR = &H1E: regIX = &H4000: regSP = &HFFF5: regHL_ = &H0
intIFF1 = True: intIFF2 = True

' System and stacks statment.
snap_buffer = "FF 80 F9 *0003 FF 00 FE FF *0003 00 7D 40 7E 40 96 43 CE 33 06 46 09 46 *0002 00 0A 46 0A 46 00 5D 40 00 02 *0002 00 *0003 FF 37 7D 40 *0002 00 02 68 D0 8A 0C *0002 00 A3 DA *0002 00 BC 21 18 40 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 *0020 00 76 C6 3F 00 01 3C 00 04 0F *0006 01 FB FF 20 45 64 45 80 45 9C 45 06 46 7F FD 00 38 *0002 00 EA 40 01 *0008 00 01 *000B 00 80 *000A 00 71 86 1C *0003 00 68 86 48 *0003 00 9B 31 01 02 0F 00 04 *012D 00 DB 41 00 02 0F 00 04 *003D 00 CD A6 00 02 17 00 07 *00A2 00 80 EF *0002 0B 76 0B 08 46 *0003 00 3B 2A 00 28 3A 37 38 34 37 10 34 37 00 3C 2E 33 29 34 3C 00 2E 2B 38 2D 2E 2B 39 2A 29 11 1A A7 31 26 33 30 1A A8 26 31 28 3A 31 26 39 2A 1A A9 2A 28 34 29 2A 1A AC 34 39 34 1A AE 33 35 3A 39 1A "
snap_buffer = snap_buffer & "B1 34 26 29 18 2E 33 2E 39 1A B2 34 29 2A 1A B5 37 2E 33 39 1A B6 3A 2E 28 30 1A B7 2A 35 2A 26 39 1A B8 26 3B 2A 1A B9 34 39 26 31 1A BE 2A 26 37 76 00 76 2B *3966 00 98 43 94 43 93 *0002 00 08 *0003 00 08 00 08 *0002 00 97 04 00 3E *3FFF 00 02 *3FD0 00 80 *0002 00 D8 43 F3 19 2B 00 A7 19 C9 15 F3 19 5D 40 F3 19 F3 19 01 40 08 46 C7 12 23 00 F8 0F 40 00 BB 03 43 03 76 06 DD 3D *0002 33 AF 33 *0002 00 02 "
  End Select
  

   '//   Load the Z81 string.
   
  Mem_Offset = &H4000 ' To 7FFF

add_chars:
  
If Len(snap_buffer) = 0 Then GoTo err_eof
  
  Do
     Data2 = snap_buffer
    
 If Left(Data2, 1) <> "*" Then
    ' Load ASCII value in memory
    MEM(Mem_Offset) = val("&h" & Data2)
    Mem_Offset = Mem_Offset + 1
    Select Case Len(snap_buffer)
    Case Is <= 2
    GoTo err_eof
    Case Else
    snap_buffer = Right(snap_buffer, Len(snap_buffer) - 3)
    End Select
    
Else
    snap_buffer = Right(snap_buffer, Len(snap_buffer) - 1) ' "*" ->x
    Data4 = snap_buffer
    snap_buffer = Right(snap_buffer, Len(snap_buffer) - 5) ' "xxxx " ->x
    Data2 = snap_buffer
    If Len(snap_buffer) <= 3 Then
    snap_buffer = ""
    GoTo err_eof
    Else
    snap_buffer = Right(snap_buffer, Len(snap_buffer) - 3) ' "xx " ->x
    End If
    
  ' Load ASCII values in memory.
   For a = 1 To CLng(val("&h" & Data4))
     MEM(Mem_Offset) = val("&h" & Data2)
     Mem_Offset = Mem_Offset + 1
   Next

End If
    
    If snap_buffer = "" Then GoTo err_eof
    
  Loop Until Left(Data2, 1) = "*" Or Data2 = vbCrLf Or InStr(1, Data2, Chr(13)) <> 0
  
  If Left(Data2, 1) = "*" Then GoTo add_chars
err_eof:

Select Case Ramtop
Case "32K"
If MEM(16443) And 128 > 0 Then Debug.Print "Slow Mode"
'MEM(16443) = MEM(16443) And Not (&HC0)  ' &HC0  '(BF+128)
Case "64K"
End Select

        bInputWait = True
        bBooting = False
        
DoEvents


End Sub

Public Function TestRAMTOP(FileName As String) As String
Dim m_iLen, m_iFile, P_header As P_header

On Error GoTo IO_Error

m_iFile = FreeFile

   '//   Load P file.
    
      Open FileName For Binary Access Read Lock Write As #m_iFile
      m_iLen = LOF(m_iFile)
      If m_iLen <= 116 Then Close #m_iFile: TestRAMTOP = "": Exit Function
       Get #m_iFile, 1, P_header
      Close #m_iFile
      
      If m_iLen >= 16383 Then TestRAMTOP = "64K" Else TestRAMTOP = "16K"
'      Add_D_File = (UinttoS(P_header.) - 16509) + 117
      
      Exit Function
IO_Error:
          Close #m_iFile
          
End Function


'Read a S3F FILE.
Function S3F_TO_BIN(FileName As String, BinArray() As Byte)
Dim m_iFile, Tmp_Line As String, Tag As String * 2, Cnt, CRC1 As Long, CRC2 As Long, Strlenght As Double
Dim BDATA As String * 2, LCRC As Long, Tmp_Line2 As String

On Error GoTo IO_Error

m_iFile = FreeFile

   '//   Load S3F file.
    
      Open FileName For Input Access Read Lock Write As #m_iFile
      ReDim BinArray(0)
      Do While Not EOF(m_iFile)
      Line Input #m_iFile, Tmp_Line
      Tmp_Line = Trim(Tmp_Line)
      Tmp_Line2 = Right(Tmp_Line, Len(Tmp_Line) - 2)
      Tag = UCase(Tmp_Line)
      If Tmp_Line = "" Or Len(Tmp_Line) < 3 Then GoTo EndScan
      
If Tag = "S3" Then
 
      Tmp_Line = Right(Tmp_Line, Len(Tmp_Line) - 12)
      Do While Len(Tmp_Line) >= 4
      BDATA = Tmp_Line
      If BDATA = "" Then Exit Do
      BinArray(UBound(BinArray)) = val("&h" & BDATA)
      ReDim Preserve BinArray(UBound(BinArray) + 1)
      LCRC = (LCRC + val("&h" & BDATA))
      Tmp_Line = Right(Tmp_Line, Len(Tmp_Line) - 2)
      Loop
      BDATA = Tmp_Line
      
LCRC = 0
      For Cnt = 1 To Len(Tmp_Line2) - 3 Step 2
         LCRC = LCRC + val("&h" & Mid(Tmp_Line2, Cnt, 2))
      Next

      LCRC = (LCRC And 255) Xor 255

    If LCRC <> val("&h" & Right(Tmp_Line2, 2)) Then MsgBox "Line CRC error: ?" & Right(Tmp_Line2, 2) & "? [" & Hex8(LCRC) & "]", vbCritical, "SF3 ERROR"
End If
      
EndScan:
       Loop

      Close #m_iFile
      
      
      Exit Function
IO_Error:
MsgBox "File read error !", vbCritical, "SF3 ERROR"
          Close #m_iFile
          reset
End Function

'Save A S3F FILE.
Function BIN_TO_S3F(FileName As String, ByRef BinArray() As Byte)
Dim m_iFile, Tmp_Line As String, Cnt As Long, Cnt2 As Long, Cnt3 As Long
Dim BDATA As String * 2, LCRC As Long, Tmp_Line2 As String
Dim ENDdata As Long

On Error GoTo IO_Error

m_iFile = FreeFile


If Dir(FileName) <> "" Then
If MsgBox("This file allready existe !" & vbCrLf & "Do you want to overwrite it?", vbYesNo) = vbNo Then Exit Function
Kill FileName
End If

   '//   Save S3F file.
      Open FileName For Output Access Write Lock Read As #m_iFile
For Cnt = 0 To UBound(BinArray) - 33 Step 32
Tmp_Line = "S3250000" & Hex16(Cnt)

For Cnt2 = 0 To 31
Tmp_Line = Tmp_Line & Hex8(CLng(BinArray(Cnt + Cnt2)))

Next Cnt2

Tmp_Line2 = Right(UCase(Tmp_Line), Len(Tmp_Line) - 2)
LCRC = 0
      For Cnt3 = 1 To Len(Tmp_Line2) - 1 Step 2
         LCRC = LCRC + val("&h" & Mid(Tmp_Line2, Cnt3, 2))
      Next
      

LCRC = (LCRC And 255) Xor 255

Tmp_Line = Tmp_Line & Hex8(LCRC)

      Print #m_iFile, Tmp_Line
Next Cnt

If (UBound(BinArray) + 1) / 32 - ((UBound(BinArray) + 1) \ 32) <> 0 Then
ENDdata = (UBound(BinArray) - Cnt) + 6
Tmp_Line = "S3" & Hex8(ENDdata) & "0000" & Hex16(Cnt)

For Cnt = Cnt To UBound(BinArray)
Tmp_Line = Tmp_Line & Hex8(CLng(BinArray(Cnt)))
Next Cnt

Tmp_Line2 = Right(UCase(Tmp_Line), Len(Tmp_Line) - 2)
LCRC = 0
      For Cnt3 = 1 To Len(Tmp_Line2) - 1 Step 2
         LCRC = LCRC + val("&h" & Mid(Tmp_Line2, Cnt3, 2))
      Next
      

LCRC = (LCRC And 255) Xor 255
Tmp_Line = Tmp_Line & Hex8(LCRC)
      Print #m_iFile, Tmp_Line
 End If
 

Tmp_Line = "050000" & Hex16(Cnt)

LCRC = 0
      For Cnt3 = 1 To Len(Tmp_Line) - 1 Step 2
         LCRC = LCRC + val("&h" & Mid(Tmp_Line, Cnt3, 2))
      Next
      

LCRC = (LCRC And 255) Xor 255

Tmp_Line = Tmp_Line & Hex8((LCRC And 255) Xor 255)

      Print #m_iFile, "S7" & Tmp_Line
    
      Close #m_iFile
       reset
      
      Exit Function
IO_Error:
MsgBox "File read error !" & vbCrLf & err.Description, vbCritical, "SF3 ERROR"
          Close #m_iFile
          reset
End Function

Sub Get_UserKeys(down As Boolean, Key As Integer)
Dim KbVC, Cnt, tBloc, col, row
Dim Shift As Boolean

If (eCursors.K5.keyCAPS_V And 1) = 0 Then Shift = True
If (keyCAPS_V And 1) = 0 Then Shift = Not Shift
 

Kempston.Stat = IIf(GetAsyncKeyState(39), 1, 0) + IIf(GetAsyncKeyState(37), 2, 0) + _
IIf(GetAsyncKeyState(40), 4, 0) + IIf(GetAsyncKeyState(38), 8, 0)


Select Case Key

Case 5
key1_5 = eCursors.K5.key1_5 Or IIf(down, 0, 255 - eCursors.K5.key1_5)
key6_0 = eCursors.K5.key6_0 Or IIf(down, 0, 255 - eCursors.K5.key6_0)
keyQ_T = eCursors.K5.keyQ_T Or IIf(down, 0, 255 - eCursors.K5.keyQ_T)
keyY_P = eCursors.K5.keyY_P Or IIf(down, 0, 255 - eCursors.K5.keyY_P)
keyA_G = eCursors.K5.keyA_G Or IIf(down, 0, 255 - eCursors.K5.keyA_G)
keyH_ENT = eCursors.K5.keyH_ENT Or IIf(down, 0, 255 - (eCursors.K5.keyH_ENT))
keyCAPS_V = IIf(Shift, 0, 1) + (eCursors.K5.keyCAPS_V And 254) Or (IIf(down, 0, 255 - (eCursors.K5.keyCAPS_V And 254)))
keyB_SPC = eCursors.K5.keyB_SPC Or IIf(down, 0, 255 - eCursors.K5.keyB_SPC)

Case 6
key1_5 = eCursors.K6.key1_5 Or IIf(down, 0, 255 - eCursors.K6.key1_5)
key6_0 = eCursors.K6.key6_0 Or IIf(down, 0, 255 - eCursors.K6.key6_0)
keyQ_T = eCursors.K6.keyQ_T Or IIf(down, 0, 255 - eCursors.K6.keyQ_T)
keyY_P = eCursors.K6.keyY_P Or IIf(down, 0, 255 - eCursors.K6.keyY_P)
keyA_G = eCursors.K6.keyA_G Or IIf(down, 0, 255 - eCursors.K6.keyA_G)
keyH_ENT = eCursors.K6.keyH_ENT Or IIf(down, 0, 255 - eCursors.K6.keyH_ENT)
keyCAPS_V = IIf(Shift, 0, 1) + (eCursors.K6.keyCAPS_V And 254) Or (IIf(down, 0, 255 - (eCursors.K6.keyCAPS_V And 254)))
keyB_SPC = eCursors.K6.keyB_SPC Or IIf(down, 0, 255 - eCursors.K6.keyB_SPC)

Case 7
key1_5 = eCursors.K7.key1_5 Or IIf(down, 0, 255 - eCursors.K7.key1_5)
key6_0 = eCursors.K7.key6_0 Or IIf(down, 0, 255 - eCursors.K7.key6_0)
keyQ_T = eCursors.K7.keyQ_T Or IIf(down, 0, 255 - eCursors.K7.keyQ_T)
keyY_P = eCursors.K7.keyY_P Or IIf(down, 0, 255 - eCursors.K7.keyY_P)
keyA_G = eCursors.K7.keyA_G Or IIf(down, 0, 255 - eCursors.K7.keyA_G)
keyH_ENT = eCursors.K7.keyH_ENT Or IIf(down, 0, 255 - eCursors.K7.keyH_ENT)
keyCAPS_V = IIf(Shift, 0, 1) + (eCursors.K7.keyCAPS_V And 254) Or (IIf(down, 0, 255 - (eCursors.K7.keyCAPS_V And 254)))
keyB_SPC = eCursors.K7.keyB_SPC Or IIf(down, 0, 255 - eCursors.K7.keyB_SPC)

Case 8
key1_5 = eCursors.K8.key1_5 Or IIf(down, 0, 255 - eCursors.K8.key1_5)
key6_0 = eCursors.K8.key6_0 Or IIf(down, 0, 255 - eCursors.K8.key6_0)
keyQ_T = eCursors.K8.keyQ_T Or IIf(down, 0, 255 - eCursors.K8.keyQ_T)
keyY_P = eCursors.K8.keyY_P Or IIf(down, 0, 255 - eCursors.K8.keyY_P)
keyA_G = eCursors.K8.keyA_G Or IIf(down, 0, 255 - eCursors.K8.keyA_G)
keyH_ENT = eCursors.K8.keyH_ENT Or IIf(down, 0, 255 - eCursors.K8.keyH_ENT)
keyCAPS_V = IIf(Shift, 0, 1) + (eCursors.K8.keyCAPS_V And 254) Or (IIf(down, 0, 255 - (eCursors.K8.keyCAPS_V And 254)))
keyB_SPC = eCursors.K8.keyB_SPC Or IIf(down, 0, 255 - eCursors.K8.keyB_SPC)

End Select


If key1_5 <> 255 Then row = 247: col = key1_5: key1_5 = Rot(key1_5)
If key6_0 <> 255 Then row = 239: col = key6_0: key6_0 = Rot(key6_0)
If keyQ_T <> 255 Then row = 251: col = keyQ_T: keyQ_T = Rot(keyQ_T)
If keyY_P <> 255 Then row = 223: col = keyY_P: keyY_P = Rot(keyY_P)
If keyA_G <> 255 Then row = 253: col = keyA_G: keyA_G = Rot(keyA_G)
If keyH_ENT <> 255 Then row = 191: col = keyH_ENT: keyH_ENT = Rot(keyH_ENT)
If (keyCAPS_V And 254) <> 254 Then row = 254: col = keyCAPS_V: keyCAPS_V = Rot(keyCAPS_V)
If keyB_SPC <> 255 Then row = 127:: keyB_SPC = Rot(keyB_SPC)

'MEM(16421) = Row
'MEM(16422) = Col

End Sub

Function Rot(value As Long) As Long
Dim dummy, x, y, max
Dim rest

    value = Abs(value)
    x = -1
    While dummy <= value
        x = x + 1
        dummy = 2 ^ x
    Wend
    
    max = x - 1
    rest = value
    'Construire la chane binaire
    For y = max To 1 Step -1
        rest = rest - (2 ^ y)
        If rest >= 0 Then Rot = Rot + (2 ^ (y - 1))
        If rest < 0 Then
            rest = rest + (1 * 2 ^ y)
        End If
    Next y
    
End Function

Sub OpenCommand()

      FreezeCPU = True
      Dim lname As String
      lname = Cut_Path_Name(Command$)
      If Left(lname, 1) = Chr(&H22) Then lname = Right(lname, Len(lname) - 1)
      SaveSetting "Grok", "vb81", "LastPath", lname
      
      On Error Resume Next
      lname = ""
        If InStr(1, Command$, Chr(34)) > 0 Then
         lname = Right(Trim(Command$), Len(Trim(Command$)) - 1)
         lname = Left(lname, Len(lname) - 1)
          Else
         lname = Trim(Command$)
          End If
      ChDir Cut_Path_Name(Cut_Path_Name(lname) & "\")
      
      'P files.
      If InStr(1, UCase(Right(lname, 2)), ".P") > 0 Or InStr(1, UCase(Right(lname, 3)), ".81") Then
      Flash_Load lname
      AutoStart = False
      FreezeCPU = False
      
      'Z81 files.
           ElseIf InStr(1, UCase(Right(lname, 4)), ".Z81") > 0 Then
      Snap_CPU lname
      AutoStart = False
      FreezeCPU = False
      
      'TEXT files.
            ElseIf InStr(1, UCase(Right(lname, 4)), ".TXT") > 0 Or InStr(1, UCase(Right(lname, 4)), ".BAS") > 0 Then
      Zx_Name = Cut_Name(lname)
      FrmSelectDir.Open_TextFile lname
      TextFile = eTextMemory
      FrmTextCompil.Show 1, FrmMainWnd
      AutoStart = False
      FreezeCPU = False
      
      'SBB files.
           ElseIf InStr(1, UCase(Right(lname, 4)), ".SBB") > 0 Then
      Enum_SBBfiles lname
      FrmMainWnd.Init_VTape
      Write_Flash_inMem16
      Load_SBBFile lname, 0
      AutoStart = False
      FreezeCPU = False
      
      'T81 files.
            ElseIf InStr(1, UCase(Right(lname, 4)), ".T81") > 0 Then
      Enum_T81files lname
      FrmMainWnd.Init_VTape
      Write_Flash_inMem16
      Load_T81File lname, 0
      AutoStart = False
      FreezeCPU = False
      
      'TZX files.
            ElseIf InStr(1, UCase(Right(lname, 4)), ".TZX") > 0 Then
      bSaveP = False
      bPicture = False
      ZX81name = ""
      TZXname = lname

      If Not OpenTZX Then MsgBox "File error", vbCritical: GoTo Tzx_Error ' Open the TZX if True=OK False=File error.
      Dim Get_Blocks() As Blocinfo
      ReDim VT_NameZX(0)
      Dim Vt_Cmp As Integer
      Vt_Cmp = 0
      VT_NameMs = lname
   
      ' Enum TZX blocs.
      Enum_TZX Get_Blocks()
   
        If Tzx_Spectrum = True Then
          MsgBox "TZX spectrum or another computer!", vbCritical, "File Error."
          Tzx_Spectrum = False
          GoTo Tzx_Error
        End If
   
        If TZX_Corrupt = True Then
         MsgBox "TZX file corrupt!", vbCritical, "File Error."
        TZX_Corrupt = False
        GoTo Tzx_Error
        End If

         If Nb_Files > 0 Then
         For cmp = 0 To UBound(Get_Blocks)
          If Get_Blocks(cmp).Tag <> "" Then
                  ReDim Preserve VT_NameZX(Vt_Cmp)
                  VT_NameZX(Vt_Cmp).Nom_Fichier = Get_Blocks(cmp).Tag
                  VT_NameZX(Vt_Cmp).Offset = Get_Blocks(cmp).Offset
                  VT_NameZX(Vt_Cmp).Taille = Get_Blocks(cmp).Len
                  Vt_Cmp = Vt_Cmp + 1
           End If
          Next cmp
         Else
          GoTo Tzx_Error
          End If
    
        ' Load bloc in memory.
            Write_Flash_inMem16
            bLoadBloc = True
           ZX81name = VT_NameZX(0).Nom_Fichier
           Enum_TZX Get_Blocks()
    
Tzx_Error:
           Dispose ' Close the TZX hook.
           FrmMainWnd.Init_VTape
           AutoStart = False
           FreezeCPU = False

'Wav files.
      ElseIf InStr(1, UCase(Right(lname, 4)), ".WAV") > 0 Then
        Zx_Name = Cut_Name(lname)
        If Not wHeadInfo(lname) Then
           MsgBox "Bad Wav header.", vbCritical, "Bad Wav file format!"
        Else
        '   Write_Flash_inMem16
           Call Get_WAV(lname)
           Call DecodeWav(Cap_Line, False, True, True)
        End If
        AutoStart = False
        FreezeCPU = False

End If ' ElseIf auto-loader

      Zx_Name = Cut_Name(lname)
      FrmMainWnd.Caption = "VB81 XuR [" & Zx_Name & "] " & GetMemName
      FrmMainWnd.Sound.BackColor = &HC0C0C0
      
      
End Sub

Sub SaveOptScrBas(FileName As String)

Dim Lix As Integer, Cnt As Integer, Cnt2 As Integer, Cnt3 As Integer, TempStr As String
Dim D_FILE As Long
Dim TmpLine As String
Dim TMPline1 As String
Dim TMPline2 As String


   D_FILE = ((CLng(MEM(16397)) * 256) + CLng(MEM(16396))) + 1
   
    TempStr = Format(10, "@@@@ ") & "POKE 16418,0" & vbCrLf
        For Cnt = 0 To 21
        TempStr = TempStr & Format(((Cnt + 1) * 10) + 10, "@@@@ ") & "PRINT "
     
     TmpLine = ""
     If peekb(D_FILE) <> 118 Then
        Do
        If peekb(D_FILE) = 11 Then
        TmpLine = TmpLine & Chr(192)
        Else
        TmpLine = TmpLine & Zx2ASCII(peekb(D_FILE))
        End If
        D_FILE = D_FILE + 1
        Loop Until peekb(D_FILE) = 118
     End If
     
     TMPline1 = Left(TmpLine, 16)
     TMPline2 = Right(TmpLine, 16)
          
          If Trim(TMPline1) = "" Then TempStr = TempStr & ","
     
     Cnt2 = 0
     For Cnt3 = Len(TMPline1) To 1 Step -1
     If Mid(TMPline1, Cnt3, 1) = " " Then Cnt2 = Cnt2 + 1 Else Exit For
     Next Cnt3
     
     If Cnt2 > 0 Then TMPline1 = Left(TMPline1, Len(TMPline1) - Cnt2)
     
     
   
     Cnt2 = 0
     For Cnt3 = Len(TMPline2) To 1 Step -1
     If Mid(TMPline2, Cnt3, 1) = " " Then Cnt2 = Cnt2 + 1 Else Exit For
     Next Cnt3
     
     If Cnt2 > 0 Then TMPline2 = Left(TMPline2, Len(TMPline2) - Cnt2)
     
    
         
     
     If Trim(TMPline2) = "" Then
     TempStr = TempStr & IIf(Trim(TMPline1) <> "", Chr(34) & TMPline1 & Chr(34), "") & IIf(Trim(TMPline1) <> "" And Len(TMPline1) < 16, ",", "")
     Else
        If Len(TMPline1) < 16 Then
        TempStr = TempStr & IIf(Trim(TMPline1) <> "", Chr(34) & TMPline1 & Chr(34) & ",", "")
        Else
        TempStr = TempStr & IIf(Trim(TMPline1) <> "", Chr(34) & TMPline1, "")
        End If
     End If
        
  
      If Trim(TMPline1) = "" Then
      
     If Trim(TMPline2) <> "" Then
     TempStr = TempStr & IIf(Trim(TMPline2) <> "", Chr(34) & TMPline2 & Chr(34), "")
     Else
     TempStr = TempStr & ","
     End If
     
     
     Else
     If Trim(TMPline2) <> "" Then
     If Len(TMPline1) < 16 Then
         TempStr = TempStr & Chr(34) & TMPline2 & Chr(34)
         Else
         TempStr = TempStr & TMPline2 & Chr(34)
         End If
     Else
     TempStr = TempStr & ","
     End If
     End If
     
     If Len(TMPline2) < 16 And Trim(TMPline2) <> "" Then TempStr = TempStr & ","
    
     
        TempStr = TempStr & vbCrLf
        D_FILE = D_FILE + 1
        Next
        
    TempStr = TempStr & Format(((Cnt + 1) * 10) + 10, "@@@@ ") & "PRINT " & Chr(34)
     If peekb(D_FILE) <> 118 Then
        Do
        If peekb(D_FILE) = 11 Then
        TempStr = TempStr & Chr(192)
        Else
        TempStr = TempStr & Zx2ASCII(peekb(D_FILE))
        End If
        D_FILE = D_FILE + 1
        Loop Until peekb(D_FILE) = 118
        D_FILE = D_FILE + 1
        
     End If
     If peekb(D_FILE) <> 118 Then
        Do
        If peekb(D_FILE) = 11 Then
        TempStr = TempStr & Chr(192)
        Else
        TempStr = TempStr & Zx2ASCII(peekb(D_FILE))
        End If
        D_FILE = D_FILE + 1
        Loop Until peekb(D_FILE) = 118
        D_FILE = D_FILE + 1
     End If

        TempStr = TempStr & Chr(34) & vbCrLf
        
Lix = FreeFile
Open OpenDialog.FileName For Output As #Lix
Print #Lix, TempStr
Close #Lix
End Sub

Function PutPrinterCode(Zxcode As Byte, SPChar As Byte) As Byte

Select Case SPChar
Case 155 'Print Mode Normal/graphic

    If Zxcode = 36 Then ' "8"
    'Graphic on
    ElseIf Zxcode = 43 Then ' "F"
    'Graphic off
    End If

    Select Case Zxcode
    Case Is > 27 And Zxcode < 60
    PutPrinterCode = Zxcode - 28
    Exit Function
    End Select
    
Case 6
    PutPrinterCode = 6
    
Case 136
    PutPrinterCode = 136
Case 137

    PutPrinterCode = 137
    ' Print HRG graphics.
    
Case Else
    Select Case Zxcode
    Case 0
     PutPrinterCode = 32
    Case 143
     PutPrinterCode = 33
    Case 11
     PutPrinterCode = 34
    Case 12
     PutPrinterCode = 35
    Case 13
     PutPrinterCode = 36
    Case 140
     PutPrinterCode = 37
    Case 141
     PutPrinterCode = 38
    Case 142
     PutPrinterCode = 39
    Case 149
     PutPrinterCode = 64
    Case Is > 139 And Zxcode < 91
     PutPrinterCode = Zx2ASCII(Zxcode)
    Case 148
     PutPrinterCode = 91
    Case 152
     PutPrinterCode = 92
    Case 150
     PutPrinterCode = 93
    Case 153
     PutPrinterCode = 94
    Case 147
     PutPrinterCode = 95
    Case 154
     PutPrinterCode = 96
    Case Is > 165 And Zxcode < 192
     PutPrinterCode = Zx2ASCII(Zxcode)
    Case 144
     PutPrinterCode = 123
    Case 139
     PutPrinterCode = 124
    Case 145
     PutPrinterCode = 125
    Case 146
     PutPrinterCode = 126
    Case 151
     PutPrinterCode = 127
    End Select
    
End Select

 End Function

Function Get_Prt_String(code As Byte) As String
Get_Prt_String = Chr$(code)
Select Case code
    Case 0
         Get_Prt_String = "[NUL]"
    Case 1
         Get_Prt_String = "[SOH]"
    Case 2
         Get_Prt_String = "[STX]"
    Case 3
         Get_Prt_String = "[ETX]"
    Case 4
         Get_Prt_String = "[EOT]"
    Case 5
         Get_Prt_String = "[ENQ]"
    Case 6
         Get_Prt_String = "[ACK]"
    Case 7
         Get_Prt_String = "[BEL]"
    Case 8
         Get_Prt_String = "[BS]"
    Case 9
         Get_Prt_String = "[HT]"
    Case 10
         Get_Prt_String = "[LF]"
    Case 11
         Get_Prt_String = "[VT]"
    Case 12
         Get_Prt_String = "[FF]"
    Case 13
         Get_Prt_String = "[CR]"
    Case 14
         Get_Prt_String = "[SO]"
    Case 15
         Get_Prt_String = "[SI]"
    Case 16
         Get_Prt_String = "[DLE]"
    Case 17
         Get_Prt_String = "[DC1]"
    Case 18
         Get_Prt_String = "[DC2]"
    Case 19
         Get_Prt_String = "[DC3]"
    Case 20
         Get_Prt_String = "[DC4]"
    Case 21
         Get_Prt_String = "[NAK]"
    Case 22
         Get_Prt_String = "[SYN]"
    Case 23
         Get_Prt_String = "[ETB]"
    Case 24
         Get_Prt_String = "[CAN]"
    Case 25
         Get_Prt_String = "[EM]"
    Case 26
         Get_Prt_String = "[SUB]"
    Case 27
         Get_Prt_String = "[ESC]"
    Case 28
         Get_Prt_String = "[FS]"
    Case 29
         Get_Prt_String = "[GS]"
    Case 30
         Get_Prt_String = "[RS]"
    Case 31
         Get_Prt_String = "[US]"
    Case 127
         Get_Prt_String = "[DEL]"
     End Select
     

End Function

